package slgfp;

/**
 * La clase nos sirve de contenedar de datos primario mantiendo los valores
 * originales de base de datos. Los valores se utilizan para crear UPDATE y
 * WHERE cuyas campos se basan en java.sql.Resultset.
 * 
 * @version 1.0, 03-05-2011
 * @author Sergio Lovillo
 */

@SuppressWarnings("serial")
public class ContenedorDatos implements java.io.Serializable, Cloneable {

	public ContenedorDatos() {
	}

	public ContenedorDatos(int contadorCampos) {
		this.contadorCampos = contadorCampos;
		datos = new Object[contadorCampos];
	}

	protected int contadorCampos = 0;
	protected Object[] datos = null;
	protected String[] nombreCampos = null;
	protected boolean ignorarNombresCamposNoDefinidos = false;

	public int getContadorCampos() {
		return contadorCampos;
	}

	public void setContadorCampos(int contadorCampos) {
		this.contadorCampos = contadorCampos;
	}

	public void setObjecto(int indiceCampo, Object valor) {
		this.datos[indiceCampo - 1] = valor;
	}

	public Object getObject(int indiceCampo) {
		return this.datos[indiceCampo - 1];
	}

	public void setNombreCampoYValor(int indiceCampo, String nombre,
			Object valor) {
		this.datos[indiceCampo - 1] = valor;
		if (this.nombreCampos == null) {
			this.nombreCampos = new String[this.contadorCampos];
		}
		this.nombreCampos[indiceCampo - 1] = nombre;
	}

	public void setObjecto(String nombre, Object valor) {
		int indiceCampo = buscaIndiceCampo(nombre);
		if (indiceCampo < 0) {
			if (this.nombreCampos == null) {
				if (!ignorarNombresCamposNoDefinidos) {
					throw new IllegalArgumentException(
							"Nombre del campo no definido");
				}
				System.out.println("Nombre del campo no definido");
			} else {
				if (!ignorarNombresCamposNoDefinidos) {
					throw new IllegalArgumentException(
							"Nombre del campo no definido: " + nombre);
				}
			}
		}
		this.datos[indiceCampo - 1] = valor;
	}

	public Object getObject(String nombre) {
		if (nombre == null) {
			return null;
		}
		int indiceCampo = buscaIndiceCampo(nombre);
		if (indiceCampo < 0) {
			if (this.nombreCampos == null) {
				if (!ignorarNombresCamposNoDefinidos) {
					throw new IllegalArgumentException(
							"Nombre del campo no definido");
				}
			} else {
				if (!ignorarNombresCamposNoDefinidos) {
					throw new IllegalArgumentException(
							"Nombre del campo no definido: " + nombre);
				}
			}
			return null;
		}
		return this.datos[indiceCampo - 1];
	}

	public void setNombreCampo(int indiceCampo, String nombre) {
		if (this.nombreCampos == null) {
			this.nombreCampos = new String[this.contadorCampos];
		}
		this.nombreCampos[indiceCampo - 1] = nombre;
	}

	public String getNombreCampo(int fieldIndex) {
		if (this.nombreCampos == null || this.nombreCampos.length == 0) {
			return null;
		}
		return this.nombreCampos[fieldIndex - 1];
	}

	public int buscaIndiceCampo(String nombre) {
		int fieldIndex = -1;
		if (this.nombreCampos == null || nombre == null) {
			return fieldIndex;
		}
		for (int i = 0; i < this.contadorCampos; i++) {
			if (nombreCampos[i] == null) {
				continue;
			}
			if (nombreCampos[i].equals(nombre)) {
				fieldIndex = i + 1;
				break;
			}
		}
		return fieldIndex;
	}

	public String toString() {
		StringBuffer sb = new StringBuffer();
		if (this.nombreCampos != null
				&& this.nombreCampos.length == contadorCampos) {
			for (int i = 0; i < contadorCampos; i++) {
				sb.append(this.nombreCampos[i]).append("=");
				if (this.datos[i] == null) {
				} else {
					sb.append(this.datos[i]);
				}
				sb.append("&");
			}
			sb.deleteCharAt(sb.length() - 1);
		} else {
			for (int i = 0; i < contadorCampos; i++) {
				if (this.datos[i] == null) {
					sb.append("null");
				} else {
					sb.append(this.datos[i]);
				}
				sb.append(", ");
			}
			sb.deleteCharAt(sb.length() - 2);
		}
		return sb.toString();
	}

	public java.lang.Object[] getDatos() {
		return this.datos;
	}

	public void setDatos(java.lang.Object[] datos) {
		this.datos = datos;
	}

	public boolean isIgnorarNombresCamposNoDefinidos() {
		return ignorarNombresCamposNoDefinidos;
	}

	public void setIgnorarNombresCamposNoDefinidos(
			boolean ignorarNombresCamposNoDefinidos) {
		this.ignorarNombresCamposNoDefinidos = ignorarNombresCamposNoDefinidos;
	}

}
