package slgfp;

import java.util.*;
import java.sql.*;
import java.lang.reflect.*;

/**
 * La clase nos permite controlar de manera general los cambios que requieren
 * modificacin en la base de datos.
 * 
 * @version 1.0, 03-05-2011
 * @author Sergio Lovillo
 */

@SuppressWarnings({ "serial", "rawtypes", "unchecked", "unused",
		"static-access" })
public class DBGrabarDatos implements GrabarDatos, java.io.Serializable,
		Cloneable {

	public DBGrabarDatos() {
		if (Boolean.getBoolean("debug-slgfp")
				|| Boolean.getBoolean("debug-" + getClass().getName())) {
			setDebug(true);
		}
	}

	protected int numeroEliminados = 0;
	protected int numeroCreados = 0;
	protected int numeroModificados = 0;
	protected List objetosEliminados = null;
	protected List objetosCreados = null;
	protected List objetosModificados = null;
	protected List generadoSQL = null;
	protected FabricaObjPersistencia solicitante = null;
	protected String nombreTablaCompleto = null;
	protected String catalogoModificado = null;
	protected String esquemaModificado = null;
	protected String tablaModificada = null;
	protected Object[] LECTURA_VACIA_ARG = new Object[0];
	protected boolean debug = false;
	protected Throwable queEsIncorrecto = null;
	protected boolean excepcionListaObjetosVacia = false;

	protected void iniciar() throws SQLException {
		numeroEliminados = 0;
		numeroCreados = 0;
		numeroModificados = 0;
		if (solicitante.getMetaDatos() == null) {
			solicitante.poblarMetaDatos();
		}
		catalogoModificado = this.solicitante.getCatalogo();
		esquemaModificado = this.solicitante.getEsquema();
		tablaModificada = this.solicitante.getTabla();
		queEsIncorrecto = null;
		if (objetosEliminados == null)
			objetosEliminados = new ArrayList();
		if (objetosCreados == null)
			objetosCreados = new ArrayList();
		if (objetosModificados == null)
			objetosModificados = new ArrayList();
		if (tablaModificada == null) {
			throw new SQLException("No se encuentra el nombre de la tabla");
		}
		nombreTablaCompleto = tablaModificada;
		if (esquemaModificado != null) {
			nombreTablaCompleto = esquemaModificado + "." + nombreTablaCompleto;
			if (catalogoModificado != null) {
				nombreTablaCompleto = catalogoModificado + "."
						+ nombreTablaCompleto;
			}
		}
	}

	public boolean grabarDatos(FabricaObjPersistencia solicitante) {
		if (solicitante.isSoloLectura()) {
			setQueEsIncorrecto(new SQLException("Solo lectura"));
			return false;
		}
		this.solicitante = solicitante;

		try {
			this.iniciar();
		} catch (SQLException sqle) {
			setQueEsIncorrecto(sqle);
			return false;
		}
		boolean exito = true;
		boolean vacio = false;
		if (debug)
			System.out
					.println("slgfp ============= "
							+ new java.util.Date()
							+ " "
							+ this.getClass()
							+ "::"
							+ this
							+ "::writeData()::solicitante.getListaObjetos() =================== "
							+ solicitante.getListaObjetos());
		if (solicitante.getListaObjetos() == null
				|| solicitante.getListaObjetos().isEmpty()) {
			vacio = true;
		}
		objetosEliminados = solicitante.getEliminarObjetos();
		if (debug)
			System.out.println("slgfp ============= " + new java.util.Date()
					+ " " + this.getClass() + "::" + this
					+ "::grabarDatos()::objetosEliminados =================== "
					+ objetosEliminados);
		if ((objetosEliminados == null || objetosEliminados.isEmpty()) && vacio) {
			if (excepcionListaObjetosVacia) {
				setQueEsIncorrecto(new SQLException("No se ha escrito nada"));
				return false;
			}
			return true;
		}
		if (solicitante.getSecuenciaModificacionDatos() != null) {
			Iterator it = solicitante.getListaObjetos().iterator();
			while (it.hasNext()) {
				ObjPersistencia objPersistencia = (ObjPersistencia) it.next();
				if (debug)
					System.out
							.println("slgfp ============= "
									+ new java.util.Date()
									+ " "
									+ this.getClass()
									+ "::"
									+ this
									+ "::grabarDatos()::domainObject =================== "
									+ objPersistencia);
				if (objPersistencia.getEstadoPersistencia() == ObjPersistencia.ELIMINADO
						&& objPersistencia.getValoresOriginales() != null
						&& objPersistencia.getValoresOriginales()
								.getContadorCampos() > 0) {
					objetosEliminados.add(objPersistencia);
				} else if (objPersistencia.getEstadoPersistencia() == ObjPersistencia.ALMACENADO
						&& objPersistencia.getValoresOriginales() != null
						&& objPersistencia.getValoresOriginales()
								.getContadorCampos() > 0) {
					objetosModificados.add(objPersistencia);
				} else if (objPersistencia.getEstadoPersistencia() == ObjPersistencia.CREADO
						&& objPersistencia.getValoresOriginales() == null
						|| objPersistencia.getValoresOriginales()
								.getContadorCampos() < 1) {
					objetosCreados.add(objPersistencia);
				}
			}
			for (int i = 0; i < solicitante.getSecuenciaModificacionDatos().length; i++) {
				if (solicitante.getSecuenciaModificacionDatos(i) == FabricaObjPersistencia.SECUENCIA_MODIFICACION_DATOS_DELETE) {
					eliminarObjetos();
				} else if (solicitante.getSecuenciaModificacionDatos(i) == FabricaObjPersistencia.SECUENCIA_MODIFICACION_DATOS_UPDATE) {
					almacenarObjetos();
				} else if (solicitante.getSecuenciaModificacionDatos(i) == FabricaObjPersistencia.SECUENCIA_MODIFICACION_DATOS_INSERT) {
					crearObjetos();
				}
			}
		} else { // Secuencia no DML
			solicitante.getListaObjetos().addAll(objetosEliminados);
			Iterator it = solicitante.getListaObjetos().iterator();
			while (it.hasNext()) {
				ObjPersistencia objPersistencia = (ObjPersistencia) it.next();
				if (debug)
					System.out
							.println("slgfp ============= "
									+ new java.util.Date()
									+ " "
									+ this.getClass()
									+ "::"
									+ this
									+ "::grabarDatos()::objPersistencia =================== "
									+ objPersistencia);
				if (objPersistencia.getEstadoPersistencia() == ObjPersistencia.ELIMINADO
						&& objPersistencia.getValoresOriginales() != null
						&& objPersistencia.getValoresOriginales()
								.getContadorCampos() > 0) {
					if (!eliminarObjetos(objPersistencia)) {
						return false;
					}
				} else if (objPersistencia.getEstadoPersistencia() == ObjPersistencia.ALMACENADO
						&& objPersistencia.getValoresOriginales() != null
						&& objPersistencia.getValoresOriginales()
								.getContadorCampos() > 0) {
					if (!almacenarObjetos(objPersistencia)) {
						return false;
					}
				} else if (objPersistencia.getEstadoPersistencia() == ObjPersistencia.CREADO
						&& objPersistencia.getValoresOriginales() == null
						|| objPersistencia.getValoresOriginales()
								.getContadorCampos() < 1) {
					if (!crearObjetos(objPersistencia)) {
						return false;
					}
				} else if (objPersistencia.getEstadoPersistencia() == ObjPersistencia.ORIGINAL) {
					// No se hace nada
				} else {
					setQueEsIncorrecto(new SQLException(
							"EstadoPersistencia no valido"));
					return false;
				}
			}
		}
		return exito;
	}

	protected boolean eliminarObjetos() {
		if (objetosEliminados.isEmpty()) {
			return true;
		}
		// Empleo del patron iterator
		Iterator iter = objetosEliminados.iterator();
		while (iter.hasNext()) {
			ObjPersistencia objPersistencia = (ObjPersistencia) iter.next();
			if (!eliminarObjetos(objPersistencia)) {
				return false;
			}
		}
		return true;
	}

	protected boolean almacenarObjetos() {
		Iterator iter = objetosModificados.iterator();
		while (iter.hasNext()) {
			ObjPersistencia objPersistencia = (ObjPersistencia) iter.next();
			if (!almacenarObjetos(objPersistencia)) {
				return false;
			}
		}
		return true;
	}

	protected boolean crearObjetos() {
		Iterator iter = objetosCreados.iterator();
		while (iter.hasNext()) {
			ObjPersistencia objPersistencia = (ObjPersistencia) iter.next();
			if (!crearObjetos(objPersistencia)) {
				return false;
			}
		}
		return true;
	}

	protected boolean eliminarObjetos(ObjPersistencia objPersistencia) {
		String nombreCampo = null;
		Collection p = new ArrayList();
		StringBuffer sql = new StringBuffer("DELETE FROM ");
		try {
			sql.append(nombreTablaCompleto);
			sql.append(" WHERE ");
			/* Usar siempre columnas PK en sentencias WHERE */
			List pk = solicitante.getClavePrimaria(catalogoModificado,
					esquemaModificado, tablaModificada);
			Iterator iter = pk.iterator();
			while (iter.hasNext()) {
				String nombreColumna = (String) iter.next(); // Se usa nombre de
																// columna
																// completo

				nombreCampo = (String) solicitante.getMapping().get(
						nombreColumna);
				int i = solicitante.getMetaDatos()
						.getIndiceColumnaPorNombreCampo(nombreCampo);
				Method readMethod = solicitante.getMetaDatos()
						.getMetodoLectura(i);
				Object valorOriginal = objPersistencia
						.getValoresOriginales(nombreCampo);
				if (valorOriginal == null) {
					sql.append(nombreColumna).append(" IS NULL AND ");
				} else {
					sql.append(nombreColumna).append(" = ? AND ");
					p.add(valorOriginal);
				}
			}
			if (solicitante.getBloqueoOptimista() == FabricaObjPersistencia.KEY_COLUMNAS) {
				/*
				 * Para DELETE hacer KEY_Y_COLUMNA_MODIFICADA y
				 * KEY_Y_COLUMNA_ACTUALIZADA lo mismo: incluir todas las
				 * columnas de la tabla WHERE en la declaracin
				 */
			} else if (solicitante.getBloqueoOptimista() == solicitante.KEY_Y_COLUMNA_MODIFICADA
					|| solicitante.getBloqueoOptimista() == FabricaObjPersistencia.KEY_Y_COLUMNA_ACTUALIZADA) {
				for (int i = 1; i <= solicitante.getMetaDatos()
						.getNumeroColumnas(); i++) {
					String catalogo = solicitante.getMetaDatos()
							.getNombreCatalogo(i);
					String esquema = solicitante.getMetaDatos()
							.getNombreEsquema(i);
					String tabla = solicitante.getMetaDatos().getNombreTabla(i);
					if (catalogoModificado != null) {
						if (!catalogoModificado.equals(catalogo)) {
							continue;
						}
					}
					if (esquemaModificado != null) {
						if (!esquemaModificado.equals(esquema)) {
							continue;
						}
					}
					if (tablaModificada != null) {
						if (!tablaModificada.equals(tabla)) {
							continue;
						}
					}
					String nombreColumna = solicitante.getMetaDatos()
							.getNombreColumnaCompleto(i);
					if (sql.indexOf(nombreColumna) > -1) {
						continue;// Esta es una columna PK
					}
					if (!solicitante.getMetaDatos().isEscrito(i)
							|| solicitante.getMetaDatos().isSoloLectura(i)) {
						continue;
					}
					nombreCampo = solicitante.getMetaDatos().getNombreCampo(i);
					try {
						Object valor = valorLeido(objPersistencia, solicitante
								.getMetaDatos().getMetodoLectura(i));
						Object valorOriginal = objPersistencia
								.getValoresOriginales(nombreCampo);
						if (valorOriginal == null) {
							sql.append(nombreColumna).append(" IS NULL AND ");
						} else {
							sql.append(nombreColumna).append(" = ? AND ");
							p.add(valorOriginal);
						}
					} catch (Throwable t) {
						throw new SQLException(t.getMessage());
					}
				}
			} else {
				throw new SQLException(
						"Configuracion de bloqueo potimista no valida");
			}
			int len = sql.length();
			sql = sql.delete(len - 5, len - 1);
			if (solicitante.isSoloGenerarSQL()) {
				if (generadoSQL == null) {
					generadoSQL = new ArrayList();
				}
				Object[] e = new Object[2];
				e[0] = sql.toString();
				e[1] = p.toArray();
				generadoSQL.add(e);
			} else {
				int filasAfectadas = solicitante.getDBServicios().ejecutar(
						sql.toString(), p);
				if (solicitante.isLanzarBloqueoOptimistaDeleteExcepcion()
						&& filasAfectadas != 1) {
					return false;
				}
				setNumeroEliminadas(numeroEliminados + filasAfectadas);
			}
		} catch (SQLException e) {
			if (debug)
				System.out.println("slgfp ============= "
						+ new java.util.Date() + " " + this.getClass() + "::"
						+ this
						+ "::eliminarObjetos::Excepcion en eliminarObjetos: "
						+ e);
			setQueEsIncorrecto(e);
			e.printStackTrace();
			return manejadorExcepcion(objPersistencia, sql.toString(), p, e);
		}
		return true;
	}

	protected boolean almacenarObjetos(ObjPersistencia objPersistencia) {
		String nombreCampo = null;
		StringBuffer sql = new StringBuffer("UPDATE ");
		List p = new ArrayList();
		if (debug)
			System.out.println("slgfp ============= " + new java.util.Date()
					+ " " + this.getClass() + "::" + this
					+ "::almacenarObjetos ===================");
		try {
			sql.append(nombreTablaCompleto).append(" SET ");
			for (int i = 1; i <= solicitante.getMetaDatos().getNumeroColumnas(); i++) {
				String catalogo = solicitante.getMetaDatos().getNombreCatalogo(
						i);
				String esquema = solicitante.getMetaDatos().getNombreEsquema(i);
				String tabla = solicitante.getMetaDatos().getNombreTabla(i);
				if (catalogoModificado != null) {
					if (!catalogoModificado.equals(catalogo)) {
						continue;
					}
				}
				if (esquemaModificado != null) {
					if (!esquemaModificado.equals(esquema)) {
						continue;
					}
				}
				if (tablaModificada != null) {
					if (!tablaModificada.equals(tabla)) {
						continue;
					}
				}
				if (!solicitante.getMetaDatos().isEscrito(i)
						|| solicitante.getMetaDatos().isSoloLectura(i)) {
					continue;
				}
				String nombreColumna = null;
				if (solicitante.isNoNombreColumnaCompleto()) {
					nombreColumna = solicitante.getMetaDatos()
							.getNombreColumna(i);
				} else {
					nombreColumna = solicitante.getMetaDatos()
							.getNombreColumnaCompleto(i);
				}
				nombreCampo = solicitante.getMetaDatos().getNombreCampo(i);
				Object valor = valorLeido(objPersistencia, solicitante
						.getMetaDatos().getMetodoLectura(i));
				Object origValue = objPersistencia
						.getValoresOriginales(nombreCampo);
				if (debug)
					System.out.println("slgfp ============= "
							+ new java.util.Date() + " " + this.getClass()
							+ "::" + this
							+ "::almacenarObjetos::nombreCampos: "
							+ nombreCampo + " valor: " + valor
							+ " valorOriginal: " + origValue + " isEditable: "
							+ solicitante.getMetaDatos().isEscrito(i)
							+ " isSoloLectura: "
							+ solicitante.getMetaDatos().isSoloLectura(i)
							+ " ===================");
				if (origValue == null) {
					if (valor == null) {
						continue; // Si se va a escribir el mismo valor no se
									// incluye en la actualizacin. Eficiencia
					} else {
						sql.append(nombreColumna).append(" = ?, ");
						p.add(valor);
					}
				} else {
					if (valor == null) {
						sql.append(nombreColumna).append(" = NULL, ");
					} else {
						if (!origValue.equals(valor)) {
							sql.append(nombreColumna).append(" = ?, ");
							p.add(valor);
						} else {
							continue;
						}
					}
				}
			}
			int idx = sql.lastIndexOf(", ");
			if (idx > -1) {
				sql = sql.delete(idx, idx + 2);
			} else { // No se ha modificado la columna de datos
				objPersistencia.setEstadoPersistencia(ObjPersistencia.ORIGINAL);
				return true;
			}
			/* Contruyendo WHERE */
			/*
			 * Usaremos siempre PK en las columnas contenidas en la sentencia
			 * WHERE
			 */
			StringBuffer where = new StringBuffer(" WHERE ");
			List pk = solicitante.getClavePrimaria(catalogoModificado,
					esquemaModificado, tablaModificada);
			if (debug)
				System.out.println("slgfp ============= "
						+ new java.util.Date() + " " + this.getClass() + "::"
						+ this + "::almacenarObjecto::pk: " + pk
						+ " esquemaModificado: " + esquemaModificado
						+ " tablaModificada: " + tablaModificada);
			Iterator iter = pk.iterator();
			while (iter.hasNext()) {
				String nombreColumna = (String) iter.next();
				nombreCampo = (String) solicitante.getMapping().get(
						nombreColumna);
				if (debug)
					System.out.println("slgfp ============= "
							+ new java.util.Date() + " " + this.getClass()
							+ "::" + this
							+ "::almacenarObjecto::nombreColumna: "
							+ nombreColumna + " nombreCampos: " + nombreCampo);
				int i = solicitante.getMetaDatos()
						.getIndiceColumnaPorNombreCampo(nombreCampo);
				Object valorOriginal = objPersistencia
						.getValoresOriginales(nombreCampo);
				if (valorOriginal == null) {
					where.append(nombreColumna).append(" IS NULL AND ");
				} else {
					where.append(nombreColumna).append(" = ? AND ");
					p.add(valorOriginal);
				}
			}
			if (solicitante.getBloqueoOptimista() == FabricaObjPersistencia.KEY_COLUMNAS) {
				idx = where.lastIndexOf(" AND ");
				where = where.delete(idx + 1, idx + 5);
			}
			if (solicitante.getBloqueoOptimista() == FabricaObjPersistencia.KEY_COLUMNAS) {
				// ... ya construido
			} else if (solicitante.getBloqueoOptimista() == FabricaObjPersistencia.KEY_Y_COLUMNA_MODIFICADA) {
				for (int i = 1; i <= solicitante.getMetaDatos()
						.getNumeroColumnas(); i++) {
					String catalogo = solicitante.getMetaDatos()
							.getNombreCatalogo(i);
					String esquema = solicitante.getMetaDatos()
							.getNombreEsquema(i);
					String tabla = solicitante.getMetaDatos().getNombreTabla(i);
					if (catalogoModificado != null) {
						if (!catalogoModificado.equals(catalogo)) {
							continue;
						}
					}
					if (esquemaModificado != null) {
						if (!esquemaModificado.equals(esquema)) {
							continue;
						}
					}
					if (tablaModificada != null) {
						if (!tablaModificada.equals(tabla)) {
							continue;
						}
					}
					String nombreColumna = solicitante.getMetaDatos()
							.getNombreColumnaCompleto(i);
					if (where.indexOf(nombreColumna) > -1) {
						continue;// Es una columna PK y ya ha sido procesada
					}
					if (!solicitante.getMetaDatos().isEscrito(i)
							|| solicitante.getMetaDatos().isSoloLectura(i)) {
						continue;
					}
					nombreCampo = solicitante.getMetaDatos().getNombreCampo(i);
					Object valor = valorLeido(objPersistencia, solicitante
							.getMetaDatos().getMetodoLectura(i));
					Object valorOriginal = objPersistencia
							.getValoresOriginales(nombreCampo);
					if (valorOriginal == null) {
						if (valor == null) {
							continue;
						} else {
							where.append(nombreColumna).append(" IS NULL AND ");
						}
					} else {
						if (valor == null) {
							where.append(nombreColumna).append(" = ? AND ");
							p.add(valorOriginal);
						} else {
							if (valorOriginal.equals(valor)) {
								continue;
							} else {
								where.append(nombreColumna).append(" = ? AND ");
								p.add(valorOriginal);
							}
						}
					}
				}
			} else if (solicitante.getBloqueoOptimista() == FabricaObjPersistencia.KEY_Y_COLUMNA_ACTUALIZADA) {
				for (int i = 1; i <= solicitante.getMetaDatos()
						.getNumeroColumnas(); i++) {
					String catalogo = solicitante.getMetaDatos()
							.getNombreCatalogo(i);
					String esquema = solicitante.getMetaDatos()
							.getNombreEsquema(i);
					String tabla = solicitante.getMetaDatos().getNombreTabla(i);
					if (catalogoModificado != null) {
						if (!catalogoModificado.equals(catalogo)) {
							continue;
						}
					}
					if (esquemaModificado != null) {
						if (!esquemaModificado.equals(esquema)) {
							continue;
						}
					}
					if (tablaModificada != null) {
						if (!tablaModificada.equals(tabla)) {
							continue;
						}
					}
					String nombreColumna = solicitante.getMetaDatos()
							.getNombreColumnaCompleto(i);
					if (where.indexOf(nombreColumna) > -1) {
						continue; // Esta es una columna tipo PK. Ya ha sido
									// procesada
					}
					if (!solicitante.getMetaDatos().isEscrito(i)
							|| solicitante.getMetaDatos().isSoloLectura(i)) {
						continue;
					}
					nombreCampo = solicitante.getMetaDatos().getNombreCampo(i);
					Object value = valorLeido(objPersistencia, solicitante
							.getMetaDatos().getMetodoLectura(i));
					Object origValue = objPersistencia
							.getValoresOriginales(nombreCampo);
					if (origValue == null) {
						where.append(nombreColumna).append(" IS NULL AND ");
					} else {
						where.append(nombreColumna).append(" = ? AND ");
						p.add(origValue);
					}
				}
			} else {
				throw new SQLException(
						"Configuracion de bloqueo optimista no valido");
			}
			if (debug)
				System.out.println("slgfp ============= "
						+ new java.util.Date() + " " + this.getClass() + "::"
						+ this + "::almacenarObjectos::where 1 : " + where);
			if (debug)
				System.out.println("slgfp ============= "
						+ new java.util.Date() + " " + this.getClass() + "::"
						+ this
						+ "::almacenarObjectos::solicitante.KEY_COLUMNAS 1 : "
						+ solicitante.KEY_COLUMNAS);
			if (solicitante.getBloqueoOptimista() != FabricaObjPersistencia.KEY_COLUMNAS) {
				idx = where.length();
				where = where.delete(idx - 5, idx - 1);
			}
			if (debug)
				System.out.println("slgfp ============= "
						+ new java.util.Date() + " " + this.getClass() + "::"
						+ this + "::almacenarObjectos::where 2 : " + where);
			sql.append(where);
			if (debug)
				System.out.println("slgfp ============= "
						+ new java.util.Date() + " " + this.getClass() + "::"
						+ this + "::almacenarObjectos::almacenarObjectos: "
						+ sql + "; parametros: " + p);
			if (solicitante.isSoloGenerarSQL()) {
				if (generadoSQL == null) {
					generadoSQL = new ArrayList();
				}
				Object[] e = new Object[2];
				e[0] = sql.toString();
				e[1] = p.toArray();
				generadoSQL.add(e);
			} else {
				int filasAfectadas = solicitante.getDBServicios().ejecutar(
						sql.toString(), p);
				if (solicitante.isLanzarBloqueoOptimistaUpdateExcepcion()
						&& filasAfectadas != 1) {
					if (debug)
						System.out.println("slgfp ============= "
								+ new java.util.Date() + " " + this.getClass()
								+ "::" + this
								+ "::almacenarObjectos::filasAfectadas: "
								+ filasAfectadas);
					return false;
				}
				setNumeroModificados(numeroModificados + filasAfectadas);
			}
		} catch (SQLException e) {
			if (debug)
				System.out
						.println("slgfp ============= "
								+ new java.util.Date()
								+ " "
								+ this.getClass()
								+ "::"
								+ this
								+ "::almacenarObjectos::Se ha producido excepcion en almacenarObjectos: "
								+ e);
			setQueEsIncorrecto(e);
			e.printStackTrace();
			return manejadorExcepcion(objPersistencia, sql.toString(), p, e);
		}
		return true;
	}

	protected boolean crearObjetos(ObjPersistencia objPersistencia) {
		String nombreCampo = null;
		StringBuffer sql = new StringBuffer("INSERT INTO ");
		List p = new ArrayList();
		try {
			sql.append(nombreTablaCompleto);
			sql.append(" (");
			for (int i = 1; i <= solicitante.getMetaDatos().getNumeroColumnas(); i++) {
				String catalogo = solicitante.getMetaDatos().getNombreCatalogo(
						i);
				String esquema = solicitante.getMetaDatos().getNombreEsquema(i);
				String tabla = solicitante.getMetaDatos().getNombreTabla(i);
				if (catalogoModificado != null) {
					if (!catalogoModificado.equals(catalogo)) {
						continue;
					}
				}
				if (esquemaModificado != null) {
					if (!esquemaModificado.equals(esquema)) {
						continue;
					}
				}
				if (tablaModificada != null) {
					if (!tablaModificada.equals(tabla)) {
						continue;
					}
				}
				String nombreCampoTemporal = solicitante.getMetaDatos()
						.getNombreCampo(i);
				if (!solicitante.getMetaDatos().isEscrito(i)
						|| solicitante.getMetaDatos().isSoloLectura(i)) {
					continue;
				}
				String nombreColumna = null;
				if (solicitante.isNoNombreColumnaCompleto()) {
					nombreColumna = solicitante.getMetaDatos()
							.getNombreColumna(i);
				} else {
					nombreColumna = solicitante.getMetaDatos()
							.getNombreColumnaCompleto(i);
				}
				sql.append(nombreColumna).append(", ");
				Object valor = valorLeido(objPersistencia, solicitante
						.getMetaDatos().getMetodoLectura(i));
				p.add(valor);
			}
			int i = sql.lastIndexOf(", ");
			sql = sql.delete(i, i + 2);
			sql.append(") VALUES (");
			ListIterator li = p.listIterator();
			while (li.hasNext()) {
				Object value = li.next();
				if (value == null) {
					sql.append("NULL, ");
					li.remove();
				} else {
					sql.append("?, ");
				}
			}
			i = sql.lastIndexOf(",");
			sql = sql.delete(i, i + 2);
			sql.append(")");
			if (solicitante.isSoloGenerarSQL()) {
				if (generadoSQL == null) {
					generadoSQL = new ArrayList();
				}
				Object[] entry = new Object[2];
				entry[0] = sql.toString();
				entry[1] = p.toArray();
				generadoSQL.add(entry);
			} else {
				int filasAfectadas = solicitante.getDBServicios().ejecutar(
						sql.toString(), p);
				if (solicitante.isLanzarFalloInsertExcepcion()
						&& filasAfectadas != 1) {
					throw new SQLException("INSERT INTO " + nombreTablaCompleto
							+ " has failed");
				}
				setNumeroCreados(numeroCreados + filasAfectadas);
			}
		} catch (SQLException e) {
			if (debug)
				System.out
						.println("slgfp ============= "
								+ new java.util.Date()
								+ " "
								+ this.getClass()
								+ "::"
								+ this
								+ "::objetosCreados::Excepcion en objetosCreados: "
								+ e);
			setQueEsIncorrecto(e);
			e.printStackTrace();
			return manejadorExcepcion(objPersistencia, sql.toString(), p, e);
		}
		return true;
	}

	public int getNumeroEliminados() {
		return numeroEliminados;
	}

	protected void setNumeroEliminadas(int numeroEliminados) {
		this.numeroEliminados = numeroEliminados;
	}

	public int getNumeroCreados() {
		return numeroCreados;
	}

	protected void setNumeroCreados(int numeroCreados) {
		this.numeroCreados = numeroCreados;
	}

	public int getNumeroDeModificados() {
		return numeroModificados;
	}

	protected void setNumeroModificados(int numeroModificados) {
		this.numeroModificados = numeroModificados;
	}

	public void resetear() {
		numeroEliminados = 0;
		numeroCreados = 0;
		numeroModificados = 0;
		generadoSQL = null;
		objetosEliminados = null;
		objetosCreados = null;
		objetosModificados = null;
		nombreTablaCompleto = null;
		tablaModificada = null;
		esquemaModificado = null;
		catalogoModificado = null;
		queEsIncorrecto = null;
	}

	public java.util.List getGeneradosSQL() {
		return generadoSQL;
	}

	public java.lang.Throwable getQueEstaMal() {
		return queEsIncorrecto;
	}

	public void setQueEsIncorrecto(java.lang.Throwable queEsIncorrecto) {
		this.queEsIncorrecto = queEsIncorrecto;
	}

	public java.lang.String getCatalogoModificado() {
		return catalogoModificado;
	}

	public void setModifiedCatalog(java.lang.String catalogoModificado) {
		this.catalogoModificado = catalogoModificado;
	}

	public java.lang.String getGeneradosSQLComoXML() {
		if (this.generadoSQL == null || this.generadoSQL.isEmpty()) {
			return "";
		}
		StringBuffer sb = new StringBuffer(
				"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n\n<generated-sql>\n");
		Iterator it = this.generadoSQL.iterator();
		while (it.hasNext()) {
			sb.append(" <sql>\n");
			Object[] obj = (Object[]) it.next();
			sb.append("  <statement><![CDATA[").append(obj[0])
					.append("]]></statement>\n");
			Object[] p = (Object[]) obj[1];
			if (p != null && p.length > 0) {
				sb.append("  <parametros>\n");
				for (int i = 0; i < p.length; i++) {
					sb.append("   <param>\n");
					sb.append("    <valor>").append("<![CDATA[")
							.append(p[i].toString()).append("]]>")
							.append("</valor>\n");
					sb.append("    <datatype>")
							.append(p[i].getClass().getName())
							.append("</datatype>\n");
					sb.append("   </param>\n");
				}
				sb.append("  </parametros>\n");
			}
			sb.append(" </sql>\n");
		}
		sb.append("</generated-sql>\n");
		return sb.toString();
	}

	protected boolean manejadorExcepcion(ObjPersistencia objPersistencia,
			String sql, Collection p, Throwable t) {
		boolean ignore = false;
		if (t instanceof java.lang.StringIndexOutOfBoundsException) {
			System.out
					.println("Asegurese de usar el correcto esquema y/o tabla");
		}
		if (solicitante.getManejadorExcepciones() != null) {
			ignore = solicitante.getManejadorExcepciones()
					.manejadorExcepciones(objPersistencia, sql, p, t);
		}
		return ignore;
	}

	public boolean isDebug() {
		return debug;
	}

	public void setDebug(boolean debug) {
		this.debug = debug;
	}

	public boolean esExcepcionListaObjetosVacia() {
		return excepcionListaObjetosVacia;
	}

	public void setExcepcionListaObjetosVacia(boolean excepcionListaObjetosVacia) {
		this.excepcionListaObjetosVacia = excepcionListaObjetosVacia;
	}

	protected Object valorLeido(ObjPersistencia objPersistencia, Method metodo)
			throws SQLException {
		Object valor = null;
		try {
			valor = metodo.invoke(objPersistencia, LECTURA_VACIA_ARG);
		} catch (Throwable t) {
			if (t instanceof InvocationTargetException) {
				throw new RuntimeException(
						((InvocationTargetException) t).getTargetException());
			} else {
				throw new RuntimeException(t);
			}
		}
		return valor;
	}
}
