package slgfp;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import javax.naming.*;
import javax.sql.*;
import java.sql.*;
import java.util.*;

/**
 * La clase nos sirve para generar la conexion con la base de datos y establecer
 * el control necesario para que los cambios sean procesados.
 * 
 * @version 1.0, 03-05-2011
 * @author Sergio Lovillo
 */

@SuppressWarnings({ "serial", "rawtypes", "unchecked", "unused" })
public class DBServicios extends Object implements java.io.Serializable,
		Cloneable {

	public static final char NADA = 'N';
	public static final char ROLLBACK = 'R';
	public static final char COMMIT = 'C';

	protected boolean conexionCerrada = false;
	protected transient Context contexto;
	protected transient DataSource dataSource;
	protected transient Connection conexion;
	protected transient Statement sentencia;
	protected transient PreparedStatement sentenciasPreparadas;
	protected transient CallableStatement sentenciasLlamadas = null;
	protected transient ResultSet resultSet;

	protected Properties propiedadesConexion = null;
	protected String nombreDataSource = null;
	protected String nombreCursor = null;
	protected int tamaoMaximoCampo = 0;
	protected int filasMaximo = 0;
	protected int timeoutConsultas = 0;
	protected int buscarDireccion = 0;
	protected int buscarTamao = 0;
	protected int tipoResultSet = ResultSet.TYPE_FORWARD_ONLY;
	protected int concurrencia = ResultSet.CONCUR_READ_ONLY;
	protected boolean soloLectura = false;
	protected boolean cerrarSentenciaDespuesEjecutar = true;
	protected boolean escaparProcesamiento = true;
	protected boolean tranEnProceso = false;
	protected boolean reiniciarDataSource = false;
	protected boolean debug = false;
	protected boolean proccesarAlertas = false;
	protected boolean proccesarProcesosAlmacenados = false;
	protected boolean cerrarResultSet = true;
	protected char proccesarTransaccionsEnConexionCerrada = NADA;
	protected int maxFilasAProcesarEnTransaccion = 0; // Define cuantas filas se
														// procesaran
														// update/insert/delete
														// por transaccion
	protected int filasAfectadasEnTransaccion = 0;
	protected String conexionId = "<none>";
	protected String espacioReservado = null;
	protected int maxNumeroDeParametros = -1;
	protected transient List resultSets = null;
	protected List alertas = null;
	// Indicadores de cmo manejar de resultados (s) de procedimientos
	// almacenados y funciones
	public static final int IGNORAR_RESULTADOS = 0; // no procesar el
													// ResultSet(s)
	public static final int RESULTADO_COMO_LISTA_MANEJADORES_DATOS = 1;
	public static final int RESULTADO_COMO_VECTOR = 2; // Vector de Vectores
	public static final int RESULTADO_COMO_MAP = 3; // Map de Lista
	public static final int RESULTADO_COMO_ROWSET = 4; // ROWSET

	protected boolean cacheSentencias = false;
	protected int maxTamaoCacheSentencias = 20; // Se establece antes de
													// cacheSentencias,
													// por defecto: 20
	protected List sentenciasId; // identificador del cache de sentencias (sql
									// String)
	protected List sentenciasCache; // Sentencias almacenadas en la cache
	protected List sentenciasContadorCache; // La cache de sentencias usan un
											// contador
	protected String cacheoFilaFijaNombreClase = "com.sun.rowset.CachedRowSetImpl";
	// Si EJBContext no es nulo, es indicacin de que se trata de EJB Contaner
	// que maneja la transaccin y en lugar de conexion.rollback () ser
	// EJBContext.setRollbackSolo y no conexion.setAutoCommit (false), no
	// conexion.commit ().
	protected Object EJBContext = null;

	public DBServicios() {
		if (Boolean.getBoolean("debug-slgfp")
				|| Boolean.getBoolean("debug-" + getClass().getName())) {
			debug = true;
		}
	}

	public javax.naming.Context getContexto() {
		if (contexto == null) {
			try {
				contexto = new InitialContext();
				if (debug)
					System.out.println("slgfp ============= "
							+ new java.util.Date() + " " + this.getClass()
							+ "::" + this + "::getContexto()::contexto: "
							+ contexto + " \n");
			} catch (NamingException ne) {
				throw new RuntimeException(ne);
			}
		}
		return contexto;
	}

	public void setContexto(javax.naming.Context contexto) {
		this.contexto = contexto;
	}

	public java.sql.Connection getConexion() throws SQLException {
		if (debug)
			System.out.println("slgfp ============= " + new java.util.Date()
					+ " " + this.getClass() + "::" + this
					+ "::getConexion()::conexion 1: " + conexion
					+ "::isTransEnProceso(): " + tranEnProceso + " \n");
		if (isTranEnProceso()) {
			return this.conexion;
		}
		if (conexion == null || conexion.isClosed()) {
			dataSource = this.getDataSource();
			if (debug)
				System.out.println("slgfp ============= "
						+ new java.util.Date() + " " + this.getClass() + "::"
						+ this + "::getConnection()::dataSource 1: "
						+ dataSource + " \n");
			conexion = dataSource.getConnection();
			if (debug)
				System.out.println("slgfp ============= "
						+ new java.util.Date() + " " + this.getClass() + "::"
						+ this + "::getConexion()::conexion 2: " + conexion
						+ " \n");
		}
		if (isTranEnProceso()) {
			if (debug)
				System.out.println("slgfp ============= "
						+ new java.util.Date() + " " + this.getClass() + "::"
						+ this + "::getConnection()::isTranInProccess(): "
						+ isTranEnProceso() + "::conexionId: " + conexionId
						+ "\n");
			if (!this.conexionId.equals(conexion.toString())) {
				throw new SQLException(
						"La conexion fue cerrada durante la transaccion");
			}
		}
		if (isProccesarAlertas()) {
			poblacionAlertas(conexion.getWarnings());
			conexion.clearWarnings();
		}
		return conexion;
	}

	public void setConexion(java.sql.Connection conexion) {
		this.conexion = conexion;
	}

	public javax.sql.DataSource getDataSource() throws SQLException {
		try {
			if (dataSource == null || isReiniciarDataSource()) {
				if (nombreDataSource == null) {
					throw new SQLException(
							"El nombre del DataSource no es valido");
				} else if (nombreDataSource.indexOf("/") == -1) {
					dataSource = (DataSource) this.getContexto().lookup(
							"java:comp/env/jdbc/" + nombreDataSource);
				} else {
					dataSource = (DataSource) this.getContexto().lookup(
							nombreDataSource);
				}
			}
		} catch (NamingException ne) {
			throw new RuntimeException(ne.getMessage());
		}
		if (debug)
			System.out.println("slgfp ============= " + new java.util.Date()
					+ " " + this.getClass() + "::" + this
					+ "::getDataSource()::dataSource: " + dataSource + " \n");
		return dataSource;
	}

	public void setDataSource(javax.sql.DataSource dataSource) {
		this.dataSource = dataSource;
	}

	protected void prepararSentencia(String sql) throws SQLException {
		this.sentenciasPreparadas = this.getConexion().prepareStatement(sql,
				getTipoResultSet(), getConcurrencia());
		if (debug)
			System.out.println("slgfp ============= " + new java.util.Date()
					+ " " + this.getClass() + "::" + this
					+ "::prepararSentencia()::sql: \n" + sql + " \n");
		this.sentenciasPreparadas.setEscapeProcessing(this
				.getEscaparProcesamiento());
		this.sentenciasPreparadas.setFetchSize(this.getBuscarTamao());
		this.sentenciasPreparadas.setMaxRows(this.getMaxFilas());
		this.sentenciasPreparadas.setMaxFieldSize(this.getMaxTamaoCampo());
		this.sentenciasPreparadas.setQueryTimeout(this.getTimeoutConsultas());
		if (this.getCursorNombre() != null) {
			this.sentenciasPreparadas.setCursorName(this.getCursorNombre());
		}
	}

	public PreparedStatement getPrepararSentencia(String sql, Collection p)
			throws SQLException {
		if (sql == null || sql.trim().length() == 0) {
			throw new SQLException("La cadena SQL es vacio o nula");
		}
		if (debug)
			System.out.println("slgfp ============= " + new java.util.Date()
					+ " " + this.getClass() + "::" + this
					+ "::=============== getPrepararSentencias(): \n" + sql
					+ " ::parametros: " + p + " \n");
		int idx = -1;
		if (cacheSentencias) {
			// sentenciaPreparada es reutilizado, de lo contrario deben estar
			// cerrado y definido a null
			idx = sentenciasId.indexOf(sql);
			if (idx >= 0) {
				Object obj = sentenciasCache.get(idx);
				if (debug) {
					System.out
							.println("slgfp ============= "
									+ new java.util.Date()
									+ " "
									+ this.getClass()
									+ "::"
									+ this
									+ "::=============== getSentenciaPreparada() ============\n"
									+ "idx: "
									+ idx
									+ " "
									+ sql
									+ "::"
									+ (obj == null ? "<null>" : obj.toString()
											+ obj.getClass()) + " \n");
					System.out.println("slgfp ============= sentenciasId: "
							+ sentenciasId);
					System.out.println("slgfp ============= sentenciasCache: "
							+ sentenciasCache);
					System.out
							.println("slgfp ============= sentenciasContadorCache: "
									+ sentenciasContadorCache);
				}
				sentenciasPreparadas = (PreparedStatement) sentenciasCache
						.get(idx);
				if (debug)
					System.out
							.println("slgfp ============= "
									+ new java.util.Date()
									+ " "
									+ this.getClass()
									+ "::"
									+ this
									+ "::=============== getSentenciaPreparada() ============\n"
									+ sql + "::" + sentenciasPreparadas + " \n");
				if (sentenciasPreparadas == null) {
					sentenciasId.remove(idx);
					sentenciasCache.remove(idx);
					idx = -1;
				} else {
					sentenciasPreparadas.clearParameters();
				}
			}
		}
		if (idx == -1) {
			prepararSentencia(sql);
		}
		Iterator pIter = p.iterator();
		int pIdx = 1;
		while (pIter.hasNext()) {
			Object param = pIter.next();
			if (param != null) {
				if (param.getClass().getName().equals("java.util.Date")) {
					param = new java.sql.Date(
							((java.util.Date) param).getTime());
				}
				sentenciasPreparadas.setObject(pIdx, param);
			} else {
				sentenciasPreparadas.setNull(pIdx, Types.JAVA_OBJECT);
			}
			pIdx++;
		}
		if (cacheSentencias) {
			if (idx == -1) {
				idx = aadirCacheSentencias(sql, sentenciasPreparadas);
				sentenciasContadorCache.add(new Integer(1));
			} else {
				Object c = sentenciasContadorCache.get(idx);
				if (c == null) {
					sentenciasContadorCache.add(new Integer(1));
				} else {
					sentenciasContadorCache.set(idx,
							new Integer(((Integer) c).intValue() + 1));
				}
			}
		}
		if (isProccesarAlertas()) {
			poblacionAlertas(sentenciasPreparadas.getWarnings());
			sentenciasPreparadas.clearWarnings();
		}
		return sentenciasPreparadas;
	}

	public void setSentenciasPreparadas(
			java.sql.PreparedStatement sentenciasPreparadas) {
		this.sentenciasPreparadas = sentenciasPreparadas;
	}

	public ContenedorDatos[] getResultadoComoManejadorDatosArray(String sql,
			Collection p, boolean poblarNombreColumnas) throws SQLException {
		return (ContenedorDatos[]) getResultadoComoManejadorDatoLista(sql, p,
				poblarNombreColumnas).toArray(new ContenedorDatos[0]);
	}

	public ContenedorDatos[] getResultadoComoManejadorDatosArray(String sql,
			boolean poblarNombreColumnas) throws SQLException {
		return (ContenedorDatos[]) poblarNombreColumnas(sql,
				poblarNombreColumnas).toArray(new ContenedorDatos[0]);
	}

	public ContenedorDatos[] getResultadoComoManejadorDatosArray(String sql)
			throws SQLException {
		return (ContenedorDatos[]) getResultadoComoManejadorDatoLista(sql)
				.toArray(new ContenedorDatos[0]);
	}

	public ContenedorDatos[] getResultadoComoManejadorDatosArray(String sql,
			Collection p) throws SQLException {
		return (ContenedorDatos[]) getResultadoComoManejadorDatoLista(sql, p)
				.toArray(new ContenedorDatos[0]);
	}

	public ContenedorDatos[] getResultadosComoManejadorDatosArray(String sql,
			Collection p) throws SQLException {
		return (ContenedorDatos[]) getResultadosComoManejadorDatoLista(sql, p,
				true).toArray(new ContenedorDatos[0]);
	}

	public ContenedorDatos[] getResultadosComoManejadorDatosArray(String sql,
			Collection p, boolean poblarNombreColumnas) throws SQLException {
		return (ContenedorDatos[]) getResultadosComoManejadorDatoLista(sql, p,
				poblarNombreColumnas).toArray(new ContenedorDatos[0]);
	}

	public ContenedorDatos[] getResultadosComoManejadorDatosArray(String sql,
			Collection p, int maximoNumeroDeParametros, String h)
			throws SQLException {
		return (ContenedorDatos[]) getResultadosComoManejadorDatoLista(sql, p,
				maximoNumeroDeParametros, h, true).toArray(
				new ContenedorDatos[0]);
	}

	public ContenedorDatos[] getResultadosComoManejadorDatosArray(String sql,
			Collection ps, int maximoNumeroDeParametros, String p,
			boolean poblarNombreColumnas) throws SQLException {
		return (ContenedorDatos[]) getResultadosComoManejadorDatoLista(sql, ps,
				maximoNumeroDeParametros, p, poblarNombreColumnas).toArray(
				new ContenedorDatos[0]);
	}

	public Object getValorUnico(String sql) throws SQLException {
		Object valor = null;
		getResultSet(sql);
		if (this.resultSet.next()) {
			valor = this.resultSet.getObject(1);
		} else {
			throw new SQLException("No datos");
		}
		if (isProccesarAlertas()) {
			poblacionAlertas(this.resultSet.getWarnings());
			this.resultSet.clearWarnings();
		}
		return valor;
	}

	public Object getValorUnico(String sql, Collection p) throws SQLException {
		Object valor = null;
		getResultSet(sql, p);
		if (this.resultSet.next()) {
			valor = this.resultSet.getObject(1);
		} else {
			throw new SQLException("No datos");
		}
		if (isProccesarAlertas()) {
			poblacionAlertas(this.resultSet.getWarnings());
			this.resultSet.clearWarnings();
		}
		return valor;
	}

	public List getResultadoUnicoColumnaComoLista(String sql)
			throws SQLException {
		getResultSet(sql);
		List filas = new ArrayList();
		while (this.resultSet.next()) {
			filas.add(this.resultSet.getObject(1));
		}
		if (isProccesarAlertas()) {
			poblacionAlertas(this.resultSet.getWarnings());
			this.resultSet.clearWarnings();
		}
		return filas;
	}

	public List getResultadoUnicoColumnaComoLista(String sql, Collection p)
			throws SQLException {
		getResultSet(sql, p);
		List filas = new ArrayList();
		while (this.resultSet.next()) {
			filas.add(this.resultSet.getObject(1));
		}
		if (isProccesarAlertas()) {
			poblacionAlertas(this.resultSet.getWarnings());
			this.resultSet.clearWarnings();
		}
		return filas;
	}

	public List getResultadoUnicoColumnaComoLista(String sql, Collection p,
			int maximoNumeroDeParametros, String pH) throws SQLException {
		getResultSets(sql, p, maximoNumeroDeParametros, pH);
		List filas = new ArrayList();
		for (int i = 0; i < resultSets.size(); i++) {
			while (((ResultSet) resultSets.get(i)).next()) {
				filas.add(((ResultSet) resultSets.get(i)).getObject(1));
			}
			if (isProccesarAlertas()) {
				poblacionAlertas(((ResultSet) resultSets.get(i)).getWarnings());
				((ResultSet) resultSets.get(i)).clearWarnings();
			}
		}
		return filas;
	}

	public List getResultadoUnicoColumnaComoLista(String sql, List p)
			throws SQLException {
		getResultSets(sql, p);
		List filas = new ArrayList();
		for (int i = 0; i < resultSets.size(); i++) {
			while (((ResultSet) resultSets.get(i)).next()) {
				filas.add(((ResultSet) resultSets.get(i)).getObject(1));
			}
			if (isProccesarAlertas()) {
				poblacionAlertas(((ResultSet) resultSets.get(i)).getWarnings());
				((ResultSet) resultSets.get(i)).clearWarnings();
			}
		}
		return filas;
	}

	public Map getResultadosDosColumnasComoMap(String sql) throws SQLException {
		getResultSet(sql);
		Map filas = new LinkedHashMap();
		while (this.resultSet.next()) {
			filas.put(this.resultSet.getObject(1), this.resultSet.getObject(2));
		}
		if (isProccesarAlertas()) {
			poblacionAlertas(this.resultSet.getWarnings());
			this.resultSet.clearWarnings();
		}
		return filas;
	}

	public Map getResultadosDosColumnasComoMap(String sql, Collection p)
			throws SQLException {
		getResultSet(sql, p);
		Map filas = new LinkedHashMap();
		while (this.resultSet.next()) {
			filas.put(this.resultSet.getObject(1), this.resultSet.getObject(2));
		}
		return filas;
	}

	public Map getResultadosDosColumnasComoMap(String sql, Collection p,
			int maximoNumeroDeParametros, String pH) throws SQLException {
		getResultSets(sql, p, maximoNumeroDeParametros, pH);
		Map filas = new LinkedHashMap();
		for (int i = 0; i < resultSets.size(); i++) {
			while (((ResultSet) resultSets.get(i)).next()) {
				filas.put(this.resultSet.getObject(1),
						this.resultSet.getObject(2));
			}
			if (isProccesarAlertas()) {
				poblacionAlertas(((ResultSet) resultSets.get(i)).getWarnings());
				((ResultSet) resultSets.get(i)).clearWarnings();
			}
		}
		return filas;
	}

	public Map getResultadosDosColumnasComoMap(String sql, List p)
			throws SQLException {
		getResultSets(sql, p);
		Map filas = new LinkedHashMap();
		for (int i = 0; i < resultSets.size(); i++) {
			while (((ResultSet) resultSets.get(i)).next()) {
				filas.put(this.resultSet.getObject(1),
						this.resultSet.getObject(2));
			}
			if (isProccesarAlertas()) {
				poblacionAlertas(((ResultSet) resultSets.get(i)).getWarnings());
				((ResultSet) resultSets.get(i)).clearWarnings();
			}
		}
		return filas;
	}

	public List getResultadoComoListaObjetoValor(String sql)
			throws SQLException {
		getResultSet(sql);
		List filas = new ArrayList();
		while (this.resultSet.next()) {
			filas.add(new ValorObjeto(this.resultSet.getObject(1),
					this.resultSet.getString(2)));
		}
		if (isProccesarAlertas()) {
			poblacionAlertas(this.resultSet.getWarnings());
			this.resultSet.clearWarnings();
		}
		return filas;
	}

	public List getResultadoComoListaObjetoValor(String sql, Collection p)
			throws SQLException {
		getResultSet(sql, p);
		List filas = new ArrayList();
		while (this.resultSet.next()) {
			filas.add(new ValorObjeto(this.resultSet.getObject(1),
					this.resultSet.getString(2)));
		}
		return filas;
	}

	public List getResultadoComoListaObjetoValor(String sql, Collection p,
			int maximoNumeroDeParametros, String pH) throws SQLException {
		getResultSets(sql, p, maximoNumeroDeParametros, pH);
		List filas = new ArrayList();
		for (int i = 0; i < resultSets.size(); i++) {
			while (((ResultSet) resultSets.get(i)).next()) {
				filas.add(new ValorObjeto(this.resultSet.getObject(1),
						this.resultSet.getString(2)));
			}
			if (isProccesarAlertas()) {
				poblacionAlertas(((ResultSet) resultSets.get(i)).getWarnings());
				((ResultSet) resultSets.get(i)).clearWarnings();
			}
		}
		return filas;
	}

	public List getResultadoComoListaObjetoValor(String sql, List p)
			throws SQLException {
		getResultSets(sql, p);
		List filas = new ArrayList();
		for (int i = 0; i < resultSets.size(); i++) {
			while (((ResultSet) resultSets.get(i)).next()) {
				filas.add(new ValorObjeto(this.resultSet.getObject(1),
						this.resultSet.getString(2)));
			}
			if (isProccesarAlertas()) {
				poblacionAlertas(((ResultSet) resultSets.get(i)).getWarnings());
				((ResultSet) resultSets.get(i)).clearWarnings();
			}
		}
		return filas;
	}

	public List getResultadoComoManejadorDatoLista(String sql, Collection p,
			boolean poblarNombreColumnas) throws SQLException {
		getResultSet(sql, p);
		ResultSetMetaData rsmd = this.resultSet.getMetaData();
		int contadorColumnas = rsmd.getColumnCount();
		List filas = new ArrayList();
		String[] nombreColumnas = new String[contadorColumnas];
		Map d = new HashMap();
		if (poblarNombreColumnas) {
			// Renombrar columnas si hay nombres duplicados
			for (int col = 1; col <= contadorColumnas; col++) {
				String nombreColumna = rsmd.getColumnName(col);
				if (!d.containsKey(nombreColumna)) {
					d.put(nombreColumna, new Integer(0));
				} else {
					Integer n = (Integer) d.get(nombreColumna);
					int valorInt = n.intValue();
					n = new Integer(valorInt + 1);
					d.put(nombreColumna, n);
					nombreColumna = nombreColumna + n;
				}
				nombreColumnas[col - 1] = nombreColumna;
			}
		}
		while (this.resultSet.next()) {
			ContenedorDatos contenedorDatos = new ContenedorDatos(
					contadorColumnas);
			for (int col = 1; col <= contadorColumnas; col++) {
				if (poblarNombreColumnas) {
					contenedorDatos.setNombreCampoYValor(col,
							nombreColumnas[col - 1],
							this.resultSet.getObject(col));
				} else {
					contenedorDatos.setObjecto(col,
							this.resultSet.getObject(col));
				}
			}
			filas.add(contenedorDatos);
		}
		if (isProccesarAlertas()) {
			poblacionAlertas(this.resultSet.getWarnings());
			this.resultSet.clearWarnings();
		}
		return filas;
	}

	public List poblarNombreColumnas(String sql, boolean poblarNombreColumnas)
			throws SQLException {
		getResultSet(sql);
		ResultSetMetaData rsmd = this.resultSet.getMetaData();
		int contadorColumna = rsmd.getColumnCount();
		List filas = new ArrayList();
		String[] nombreColumnas = new String[contadorColumna];
		Map d = new HashMap();
		if (poblarNombreColumnas) {
			// Renombrar columnas si hay nombres duplicados
			for (int c = 1; c <= contadorColumna; c++) {
				String nombreColumna = rsmd.getColumnName(c);
				if (!d.containsKey(nombreColumna)) {
					d.put(nombreColumna, new Integer(0));
				} else {
					Integer n = (Integer) d.get(nombreColumna);
					int valorInt = n.intValue();
					n = new Integer(valorInt + 1);
					d.put(nombreColumna, n);
					nombreColumna = nombreColumna + n;
				}
				nombreColumnas[c - 1] = nombreColumna;
			}
		}
		while (this.resultSet.next()) {
			ContenedorDatos contenedorDatos = new ContenedorDatos(
					contadorColumna);
			for (int col = 1; col <= contadorColumna; col++) {
				if (poblarNombreColumnas) {
					contenedorDatos.setNombreCampoYValor(col,
							nombreColumnas[col - 1],
							this.resultSet.getObject(col));
				} else {
					contenedorDatos.setObjecto(col,
							this.resultSet.getObject(col));
				}
			}
			filas.add(contenedorDatos);
		}
		if (isProccesarAlertas()) {
			poblacionAlertas(this.resultSet.getWarnings());
			this.resultSet.clearWarnings();
		}
		return filas;
	}

	public List getResultadoComoManejadorDatoLista(String sql)
			throws SQLException {
		return poblarNombreColumnas(sql, true);
	}

	public List getResultadoComoManejadorDatoLista(String sql, Collection p)
			throws SQLException {
		return getResultadoComoManejadorDatoLista(sql, p, true);
	}

	public List getResultadosComoManejadorDatoLista(String sql, Collection p)
			throws SQLException {
		return getResultadosComoManejadorDatoLista(sql, p, true);
	}

	public List getResultadosComoManejadorDatoLista(String sql, Collection p,
			int maximoNumeroDeParametros, String pH) throws SQLException {
		return getResultadosComoManejadorDatoLista(sql, p,
				maximoNumeroDeParametros, pH, true);
	}

	public List getResultadosComoManejadorDatoLista(String sql, Collection p,
			int maximoNumeroDeParametros, String pH,
			boolean poblarNombreColumnas) throws SQLException {
		getResultSets(sql, p, maximoNumeroDeParametros, pH);
		ResultSetMetaData rsmd = ((ResultSet) this.resultSets.get(0))
				.getMetaData();
		int contadorColumnas = rsmd.getColumnCount();
		List filas = new ArrayList();
		String[] nombresColumnas = new String[contadorColumnas];
		Map d = new HashMap();
		// Renombrar columnas si hay nombres duplicados
		for (int c = 1; c <= contadorColumnas; c++) {
			String nombreColumna = rsmd.getColumnName(c);
			if (!d.containsKey(nombreColumna)) {
				d.put(nombreColumna, new Integer(0));
			} else {
				Integer n = (Integer) d.get(nombreColumna);
				int valorInt = n.intValue();
				n = new Integer(valorInt + 1);
				d.put(nombreColumna, n);
				nombreColumna = nombreColumna + n;
			}
			nombresColumnas[c - 1] = nombreColumna;
		}
		for (int i = 0; i < resultSets.size(); i++) {
			while (((ResultSet) resultSets.get(i)).next()) {
				ContenedorDatos contenedorDatos = new ContenedorDatos(
						contadorColumnas);
				for (int c = 1; c <= contadorColumnas; c++) {
					if (poblarNombreColumnas) {
						Object valor = ((ResultSet) resultSets.get(i))
								.getObject(c);
						contenedorDatos.setNombreCampoYValor(c,
								nombresColumnas[c - 1],
								((ResultSet) resultSets.get(i)).getObject(c));
					} else {
						contenedorDatos.setObjecto(c,
								((ResultSet) resultSets.get(i)).getObject(c));
					}
				}
				filas.add(contenedorDatos);
			}
			if (isProccesarAlertas()) {
				poblacionAlertas(((ResultSet) resultSets.get(i)).getWarnings());
				((ResultSet) resultSets.get(i)).clearWarnings();
			}
		}
		return filas;
	}

	public List getResultadosComoManejadorDatoLista(String sql, Collection p,
			boolean poblarNombresColumnas) throws SQLException {
		getResultSets(sql, p, maxNumeroDeParametros, espacioReservado);
		ResultSetMetaData rsmd = ((ResultSet) this.resultSets.get(0))
				.getMetaData();
		int contadorColumnas = rsmd.getColumnCount();
		List filas = new ArrayList();
		String[] nombresColumnas = new String[contadorColumnas];
		Map d = new HashMap();
		// Renombrar columnas si hay nombres duplicados
		for (int col = 1; col <= contadorColumnas; col++) {
			String nombreColumna = rsmd.getColumnName(col);
			if (!d.containsKey(nombreColumna)) {
				d.put(nombreColumna, new Integer(0));
			} else {
				Integer n = (Integer) d.get(nombreColumna);
				int valorInt = n.intValue();
				n = new Integer(valorInt + 1);
				d.put(nombreColumna, n);
				nombreColumna = nombreColumna + n;
			}
			nombresColumnas[col - 1] = nombreColumna;
		}
		for (int i = 0; i < resultSets.size(); i++) {
			while (((ResultSet) resultSets.get(i)).next()) {
				ContenedorDatos contenedorDatos = new ContenedorDatos(
						contadorColumnas);
				for (int col = 1; col <= contadorColumnas; col++) {
					if (poblarNombresColumnas) {
						Object value = ((ResultSet) resultSets.get(i))
								.getObject(col);
						contenedorDatos.setNombreCampoYValor(col,
								nombresColumnas[col - 1],
								((ResultSet) resultSets.get(i)).getObject(col));
					} else {
						contenedorDatos.setObjecto(col,
								((ResultSet) resultSets.get(i)).getObject(col));
					}
				}
				filas.add(contenedorDatos);
			}
			if (isProccesarAlertas()) {
				poblacionAlertas(((ResultSet) resultSets.get(i)).getWarnings());
				((ResultSet) resultSets.get(i)).clearWarnings();
			}
		}
		return filas;
	}

	public Vector getResultadosComoVector(String sql, Collection p,
			int maximoNumeroDeParametros, String pH) throws SQLException {
		getResultSets(sql, p, maximoNumeroDeParametros, pH);
		int contadorColumnas = ((ResultSet) this.resultSets.get(0))
				.getMetaData().getColumnCount();
		Vector filas = new Vector();
		for (int i = 0; i < resultSets.size(); i++) {
			while (((ResultSet) resultSets.get(i)).next()) {
				Vector fila = new Vector(contadorColumnas);
				for (int col = 1; col <= contadorColumnas; col++) {
					Object value = ((ResultSet) resultSets.get(i))
							.getObject(col);
					fila.add(value);
				}
				filas.add(fila);
			}
			if (isProccesarAlertas()) {
				poblacionAlertas(((ResultSet) resultSets.get(i)).getWarnings());
				((ResultSet) resultSets.get(i)).clearWarnings();
			}
		}
		return filas;
	}

	public Vector getResultadosComoVector(String sql, Collection p)
			throws SQLException {
		getResultSets(sql, p);
		int colCount = ((ResultSet) this.resultSets.get(0)).getMetaData()
				.getColumnCount();
		Vector filas = new Vector();
		for (int i = 0; i < resultSets.size(); i++) {
			while (((ResultSet) resultSets.get(i)).next()) {
				Vector fila = new Vector(colCount);
				for (int columna = 1; columna <= colCount; columna++) {
					Object valor = ((ResultSet) resultSets.get(i))
							.getObject(columna);
					fila.add(valor);
				}
				filas.add(fila);
			}
			if (isProccesarAlertas()) {
				poblacionAlertas(((ResultSet) resultSets.get(i)).getWarnings());
				((ResultSet) resultSets.get(i)).clearWarnings();
			}
		}
		return filas;
	}

	public Vector getResultadosComoVector(String sql) throws SQLException {
		getResultSet(sql);
		int contadorColumnas = this.resultSet.getMetaData().getColumnCount();
		Vector filas = new Vector();
		while (this.resultSet.next()) {
			Vector fila = new Vector(contadorColumnas);
			for (int columna = 1; columna <= contadorColumnas; columna++) {
				fila.add(this.resultSet.getObject(columna));
			}
			filas.add(fila);
		}
		if (isProccesarAlertas()) {
			poblacionAlertas(this.resultSet.getWarnings());
			this.resultSet.clearWarnings();
		}
		return filas;
	}

	public Vector getResultadoComoVector(String sql, Collection p)
			throws SQLException {
		getResultSet(sql, p);
		int contadorColumna = this.resultSet.getMetaData().getColumnCount();
		Vector filas = new Vector();
		while (this.resultSet.next()) {
			Vector fila = new Vector(contadorColumna);
			for (int col = 1; col <= contadorColumna; col++) {
				fila.add(this.resultSet.getObject(col));
			}
			filas.add(fila);
		}
		if (isProccesarAlertas()) {
			poblacionAlertas(this.resultSet.getWarnings());
			this.resultSet.clearWarnings();
		}
		return filas;
	}

	public Map getResultadosComoMap(String sql, Collection p)
			throws SQLException {
		getResultSets(sql, p);
		ResultSetMetaData rsmd = ((ResultSet) this.resultSets.get(0))
				.getMetaData();
		int contadorColumna = rsmd.getColumnCount();
		Map columnas = new LinkedHashMap(contadorColumna);
		List[] columna = new ArrayList[contadorColumna];
		for (int i = 0; i < columna.length; i++) {
			columna[i] = new ArrayList();
		}
		for (int i = 0; i < resultSets.size(); i++) {
			while (((ResultSet) resultSets.get(i)).next()) {
				for (int col = 1; col <= contadorColumna; col++) {
					Object valor = ((ResultSet) resultSets.get(i))
							.getObject(col);
					columna[col - 1].add(valor);
				}
			}
			if (isProccesarAlertas()) {
				poblacionAlertas(((ResultSet) resultSets.get(i)).getWarnings());
				((ResultSet) resultSets.get(i)).clearWarnings();
			}
		}
		Map d = new HashMap();
		for (int col = 1; col <= contadorColumna; col++) {
			String nombreColumna = rsmd.getColumnName(col);
			if (!d.containsKey(nombreColumna)) {
				d.put(nombreColumna, new Integer(0));
			} else {
				Integer i = (Integer) d.get(nombreColumna);
				int intValue = i.intValue();
				i = new Integer(intValue + 1);
				d.put(nombreColumna, i);
				nombreColumna = nombreColumna + i;
			}
			columnas.put(nombreColumna, columna[col - 1]);
		}
		return columnas;
	}

	public Map getResultadosComoMap(String sql, Collection p,
			int maximoNumeroDeParametros, String pH) throws SQLException {
		getResultSets(sql, p, maximoNumeroDeParametros, pH);
		ResultSetMetaData rsmd = ((ResultSet) this.resultSets.get(0))
				.getMetaData();
		int contadorColumnas = rsmd.getColumnCount();
		Map columnas = new LinkedHashMap(contadorColumnas);
		List[] columna = new ArrayList[contadorColumnas];
		for (int i = 0; i < columna.length; i++) {
			columna[i] = new ArrayList();
		}
		for (int i = 0; i < resultSets.size(); i++) {
			while (((ResultSet) resultSets.get(i)).next()) {
				for (int col = 1; col <= contadorColumnas; col++) {
					Object value = ((ResultSet) resultSets.get(i))
							.getObject(col);
					columna[col - 1].add(value);
				}
			}
			if (isProccesarAlertas()) {
				poblacionAlertas(((ResultSet) resultSets.get(i)).getWarnings());
				((ResultSet) resultSets.get(i)).clearWarnings();
			}
		}
		Map d = new HashMap();
		for (int col = 1; col <= contadorColumnas; col++) {
			String nombreColumna = rsmd.getColumnName(col);
			if (!d.containsKey(nombreColumna)) {
				d.put(nombreColumna, new Integer(0));
			} else {
				Integer i = (Integer) d.get(nombreColumna);
				int valorInt = i.intValue();
				i = new Integer(valorInt + 1);
				d.put(nombreColumna, i);
				nombreColumna = nombreColumna + i;
			}
			columnas.put(nombreColumna, columna[col - 1]);
		}
		return columnas;
	}

	public Map getResultadoComoMap(String sql, Collection p)
			throws SQLException {
		getResultSet(sql, p);
		ResultSetMetaData rsmd = this.resultSet.getMetaData();
		int contadorColumnas = rsmd.getColumnCount();
		Map columnas = new LinkedHashMap(contadorColumnas);
		List[] columna = new ArrayList[contadorColumnas];
		for (int i = 0; i < columna.length; i++) {
			columna[i] = new ArrayList();
		}
		while (this.resultSet.next()) {
			for (int col = 1; col <= contadorColumnas; col++) {
				Object valor = this.resultSet.getObject(col);
				columna[col - 1].add(valor);
			}
		}
		Map d = new HashMap();
		for (int col = 1; col <= contadorColumnas; col++) {
			String nombreColumna = rsmd.getColumnName(col);
			if (!d.containsKey(nombreColumna)) {
				d.put(nombreColumna, new Integer(0));
			} else {
				Integer i = (Integer) d.get(nombreColumna);
				int valorInt = i.intValue();
				i = new Integer(valorInt + 1);
				d.put(nombreColumna, i);
				nombreColumna = nombreColumna + i;
			}
			columnas.put(nombreColumna, columna[col - 1]);
		}
		if (isProccesarAlertas()) {
			poblacionAlertas(this.resultSet.getWarnings());
			this.resultSet.clearWarnings();
		}
		return columnas;
	}

	public Map getResultadoComoMapa(String sql) throws SQLException {
		getResultSet(sql);
		ResultSetMetaData rsmd = this.resultSet.getMetaData();
		int contadorColumna = rsmd.getColumnCount();
		Map columnas = new LinkedHashMap(contadorColumna);
		List[] columna = new ArrayList[contadorColumna];
		for (int i = 0; i < columna.length; i++) {
			columna[i] = new ArrayList();
		}
		while (this.resultSet.next()) {
			for (int col = 1; col <= contadorColumna; col++) {
				Object valor = this.resultSet.getObject(col);
				columna[col - 1].add(valor);
			}
		}
		Map d = new HashMap();
		for (int col = 1; col <= contadorColumna; col++) {
			String nombreColumna = rsmd.getColumnName(col);
			if (!d.containsKey(nombreColumna)) {
				d.put(nombreColumna, new Integer(0));
			} else {
				Integer i = (Integer) d.get(nombreColumna);
				int valorInt = i.intValue();
				i = new Integer(valorInt + 1);
				d.put(nombreColumna, i);
				nombreColumna = nombreColumna + i;
			}
			columnas.put(nombreColumna, columna[col - 1]);
		}
		if (isProccesarAlertas()) {
			poblacionAlertas(this.resultSet.getWarnings());
			this.resultSet.clearWarnings();
		}
		return columnas;
	}

	public java.sql.ResultSet getResultSet(String sql) throws SQLException {
		if (resultSet != null && isCerradoResultSet()) {
			if (debug)
				System.out.println("slgfp ============= "
						+ new java.util.Date() + " " + this.getClass() + "::"
						+ this + "::getResultSet()::cerrando ResultSet: "
						+ resultSet + " \n");
			resultSet.close();
		}
		resultSet = this.getSentencia(sql).executeQuery(sql);
		if (debug)
			System.out.println("slgfp ============= " + new java.util.Date()
					+ " " + this.getClass() + "::" + this
					+ "::getResultSet()::sql: \n" + sql + " \n");
		if (isProccesarAlertas()) {
			poblacionAlertas(resultSet.getWarnings());
			this.resultSet.clearWarnings();
		}
		return resultSet;
	}

	public java.sql.ResultSet getResultSet(String sql, Collection p)
			throws SQLException {
		if (resultSet != null && isCerradoResultSet()) {
			resultSet.close();
			if (debug)
				System.out.println("slgfp ============= "
						+ new java.util.Date() + " " + this.getClass() + "::"
						+ this + "::getResultSet()::cerrando ResultSet: "
						+ resultSet + " \n");
		}
		resultSet = this.getPrepararSentencia(sql, p).executeQuery();
		if (debug)
			System.out.println("slgfp ============= " + new java.util.Date()
					+ " " + this.getClass() + "::" + this
					+ "::getResultSet(): " + resultSet + " \n");
		if (isProccesarAlertas()) {
			poblacionAlertas(resultSet.getWarnings());
		}
		return resultSet;
	}

	public java.sql.ResultSet[] getResultSets(String sql, Collection p,
			int maximoNumeroDeParametros, String pH) throws SQLException {
		this.maxNumeroDeParametros = maximoNumeroDeParametros;
		this.espacioReservado = pH;
		return getResultSets(sql, p);
	}

	public java.sql.ResultSet[] getResultSets(String sql, Collection p)
			throws SQLException {
		/**
		 * Este mtodo resulta adecuado para construir "IN" cadenas sql si la
		 * cantidad de argumentos en favor de "IN" es superior al nmero mximo
		 * de parmetros que la base de datos puede manejar. Creando varios
		 * Resultset's. La lista de argumentos que es mayor a max nmero de
		 * parmetros que la base de datos puede manejar debe ser el ultimo en
		 * el parametros.
		 */
		cerrarResultados();
		resultSets = new ArrayList();
		Collection nuevaListaArgumentos = new ArrayList();
		String nuevaSql = null;
		Collection nuevaListaArgumentos1 = null;
		boolean cargado = false;
		boolean cerrarResultOriginal = this.cerrarResultSet;
		this.cerrarResultSet = false;
		int componerParametros = 0;
		int tamaoParametros = p.size();
		Iterator it = p.iterator();
		while (it.hasNext()) {
			Object param = it.next();
			if (debug)
				System.out.println("slgfp ============= "
						+ new java.util.Date() + " " + this.getClass() + "::"
						+ this + "::getResultSets()::parametros:" + param
						+ "::instanceof java.util.Collection: "
						+ (param instanceof java.util.Collection) + " \n");
			if (param instanceof java.util.Collection) { // Es un parmetro
															// compuesto
				componerParametros++;
			}
		}
		String[] pHT = sql.split(this.espacioReservado);
		if (debug)
			System.out.println("slgfp ============= " + new java.util.Date()
					+ " " + this.getClass() + "::" + this
					+ "::getResultSets()::sql: \n" + sql + "::pHT: "
					+ Arrays.asList(pHT) + " \n");
		if (debug)
			System.out.println("slgfp ============= " + new java.util.Date()
					+ " " + this.getClass() + "::" + this
					+ "::getResultSets()::sql: \n" + sql
					+ "::componerParametros: " + componerParametros
					+ "::pHT.length: " + pHT.length + " \n");
		if ((pHT.length - 1) != componerParametros) {
			throw new SQLException(
					"Nmero de marcadores de posicin en sql no es igual al nmero de parmetros compuestos");
		}
		try {
			Iterator it1 = p.iterator();
			int contadorParametros = 0;
			while (it1.hasNext()) {
				Object thisArg = it1.next();
				if (thisArg instanceof java.util.Collection) { // Es un
																// parmetro
																// compuesto
					if (maxNumeroDeParametros < 0) {
						throw new SQLException(
								"maxNumeroDeParametros no valido: "
										+ maxNumeroDeParametros);
					}
					List lista = new ArrayList((Collection) thisArg);
					int a = lista.size();
					if (a > maxNumeroDeParametros) {
						if (contadorParametros < (tamaoParametros - 1)) {
							throw new SQLException(
									"Solo la ultima entrada puede tener mas argumentos que el maximo numero de parametros");
						}
						nuevaSql = sql;
						StringBuffer sb = new StringBuffer();
						Iterator it3 = lista.iterator();
						int contador = 0;
						int total = 0;
						List temp = new ArrayList();
						Object valorParametro = null;
						for (int i = 0; i < a; i++) {
							if (contador < (maxNumeroDeParametros)) {
								valorParametro = lista.get(i);
								sb.append("?,");
								temp.add(valorParametro);
								if (i == (a - 1)) {
									int idx = sb.length() - 1;
									char c = sb.charAt(idx);
									if (c == ',') {
										sb.deleteCharAt(idx);
									}
									nuevaSql = sql.replaceFirst(
											espacioReservado, sb.toString());
									nuevaListaArgumentos1 = new ArrayList(
											nuevaListaArgumentos);
									nuevaListaArgumentos1.addAll(temp);
									if (debug)
										System.out
												.println("slgfp ============= "
														+ new java.util.Date()
														+ " "
														+ this.getClass()
														+ "::"
														+ this
														+ "::getResultSets()::sql: "
														+ nuevaSql
														+ "::argumentos: "
														+ nuevaListaArgumentos1
														+ " \n");
									resultSets.add(this.getPrepararSentencia(
											nuevaSql, nuevaListaArgumentos1)
											.executeQuery());
									cargado = true;
									break;
								}
								contador++;
							} else if (contador == (maxNumeroDeParametros)) {
								int idx = sb.length() - 1;
								char c = sb.charAt(idx);
								if (c == ',') {
									sb.deleteCharAt(idx);
								}
								nuevaSql = sql.replaceFirst(espacioReservado,
										sb.toString());
								nuevaListaArgumentos1 = new ArrayList(
										nuevaListaArgumentos);
								nuevaListaArgumentos1.addAll(temp);
								if (debug)
									System.out.println("slgfp ============= "
											+ new java.util.Date() + " "
											+ this.getClass() + "::" + this
											+ "::getResultSets()::sql: "
											+ nuevaSql + "::argumentos: "
											+ nuevaListaArgumentos1 + " \n");
								resultSets.add(this.getPrepararSentencia(
										nuevaSql, nuevaListaArgumentos1)
										.executeQuery());
								cargado = true;
								temp.clear();
								sb.setLength(0);
								nuevaSql = sql;
								contador = 0;
								i--;
							}
						}
					} else {
						StringBuffer sb = new StringBuffer();
						Iterator iter2 = lista.iterator();
						while (iter2.hasNext()) {
							sb.append("?,");
							nuevaListaArgumentos.add(iter2.next());
						}
						sb.deleteCharAt(sb.lastIndexOf(","));
						sql = sql.replaceFirst(espacioReservado, sb.toString());
					}
				} else { // Este parmetro es el normal
					nuevaListaArgumentos.add(thisArg);
				}
				contadorParametros++;
			}
			if (!cargado) {
				if (debug)
					System.out.println("slgfp ============= "
							+ new java.util.Date() + " " + this.getClass()
							+ "::" + this + "::getResultSets()::sql: \n" + sql
							+ "::args: " + nuevaListaArgumentos + " \n");
				resultSets.add(this.getPrepararSentencia(sql,
						nuevaListaArgumentos).executeQuery());
			}
		} catch (Exception e) {
			e.printStackTrace();
			throw new SQLException(e.getMessage());
		}
		setCerrarResultSet(cerrarResultOriginal);
		return (java.sql.ResultSet[]) resultSets.toArray(new ResultSet[0]);
	}

	public List ejecutarProcesosAlmacenados(String sql, Parametros p,
			int resultadoProceso) throws SQLException {
		if (debug)
			System.out.println("slgfp ============= " + new java.util.Date()
					+ " " + this.getClass() + "::" + this
					+ "::ejecutarProcesosAlmacenados()::sql: \n" + sql + "\n");
		List valorDevuelto = new ArrayList();
		try {
			sentenciasLlamadas = getConexion().prepareCall(sql);
			for (int i = 1; i <= p.getParameterCount(); i++) {
				int parameterMode = p.getParameterMode(i);
				if (debug)
					System.out
							.println("slgfp ============= "
									+ new java.util.Date()
									+ " "
									+ this.getClass()
									+ "::"
									+ this
									+ "::ejecutarProcesosAlmacenados()::parametroModo: "
									+ parameterMode + "::indiceParametro: " + i
									+ "::parametroTipo: "
									+ p.getParameterType(i)
									+ "::parametroNombre: "
									+ p.getParametroNombre(i)
									+ "::parameterValue: " + p.getParametros(i)
									+ "\n");
				if (parameterMode == Parametros.parameterModeIn) {
					sentenciasLlamadas.setObject(i, p.getParametros(i));
				} else if (parameterMode == Parametros.parameterModeOut) {
					sentenciasLlamadas.registerOutParameter(i,
							p.getParameterType(i));
				} else if (parameterMode == Parametros.parameterModeInOut) {
					sentenciasLlamadas.registerOutParameter(i,
							p.getParameterType(i));
					sentenciasLlamadas.setObject(i, p.getParametros(i));
				}
			}

			ResultSet result = null;
			int filasAfectadas = 0;
			boolean tieneResult = sentenciasLlamadas.execute();
			if (debug)
				System.out.println("slgfp ============= "
						+ new java.util.Date() + " " + this.getClass() + "::"
						+ this + "::executeStoredProcedure()::hasResult: "
						+ tieneResult + "\n");

			// Proceso ResultSet(s) y/o condatosActualizacion
			while (tieneResult || filasAfectadas != -1) {
				if (tieneResult == true) {
					result = sentenciasLlamadas.getResultSet();
					try {
						if (resultadoProceso == IGNORAR_RESULTADOS) {
							// No hace nada
						} else if (resultadoProceso == RESULTADO_COMO_LISTA_MANEJADORES_DATOS) {
							ResultSetMetaData rsmd = result.getMetaData();
							int contadorColumnas = rsmd.getColumnCount();
							List filas = new ArrayList();
							String[] nombresColumnas = new String[contadorColumnas];
							Map d = new HashMap();
							// Renombra las columnas si presentan ducplicidades
							for (int col = 1; col <= contadorColumnas; col++) {
								String nombreColumna = rsmd.getColumnName(col);
								if (!d.containsKey(nombreColumna)) {
									d.put(nombreColumna, new Integer(0));
								} else {
									Integer n = (Integer) d.get(nombreColumna);
									int valorInt = n.intValue();
									n = new Integer(valorInt + 1);
									d.put(nombreColumna, n);
									nombreColumna = nombreColumna + n;
								}
								nombresColumnas[col - 1] = nombreColumna;
							}
							while (result.next()) {
								ContenedorDatos contenedorDatos = new ContenedorDatos(
										contadorColumnas);
								for (int col = 1; col <= contadorColumnas; col++) {
									contenedorDatos.setNombreCampoYValor(col,
											nombresColumnas[col - 1],
											result.getObject(col));
								}
								filas.add(contenedorDatos);
							}
							if (isProccesarAlertas()) {
								poblacionAlertas(result.getWarnings());
								result.clearWarnings();
							}
							valorDevuelto.add(filas);
						} else if (resultadoProceso == RESULTADO_COMO_VECTOR) {
							int colCount = result.getMetaData()
									.getColumnCount();
							Vector filas = new Vector();
							while (result.next()) {
								Vector row = new Vector(colCount);
								for (int col = 1; col <= colCount; col++) {
									row.add(result.getObject(col));
								}
								filas.add(row);
							}
							if (isProccesarAlertas()) {
								poblacionAlertas(result.getWarnings());
								result.clearWarnings();
							}
							valorDevuelto.add(filas);
						} else if (resultadoProceso == RESULTADO_COMO_MAP) {
							ResultSetMetaData rsmd = result.getMetaData();
							int contadorColumna = rsmd.getColumnCount();
							Map columnas = new LinkedHashMap(contadorColumna);
							List[] columna = new ArrayList[contadorColumna];
							for (int i = 0; i < columna.length; i++) {
								columna[i] = new ArrayList();
							}
							while (result.next()) {
								for (int col = 1; col <= contadorColumna; col++) {
									Object value = result.getObject(col);
									columna[col - 1].add(value);
								}
							}
							Map d = new HashMap();
							for (int col = 1; col <= contadorColumna; col++) {
								String nombreColumna = rsmd.getColumnName(col);
								if (!d.containsKey(nombreColumna)) {
									d.put(nombreColumna, new Integer(0));
								} else {
									Integer i = (Integer) d.get(nombreColumna);
									int intValue = i.intValue();
									i = new Integer(intValue + 1);
									d.put(nombreColumna, i);
									nombreColumna = nombreColumna + i;
								}
								columnas.put(nombreColumna, columna[col - 1]);
							}
							if (isProccesarAlertas()) {
								poblacionAlertas(result.getWarnings());
								result.clearWarnings();
							}
							valorDevuelto.add(columnas);

						} else if (resultadoProceso == RESULTADO_COMO_ROWSET) {
							javax.sql.rowset.CachedRowSet crs = (javax.sql.rowset.CachedRowSet) Class
									.forName(cacheoFilaFijaNombreClase)
									.newInstance();
							crs.populate(result);
							valorDevuelto.add(crs);
						}
					} finally {
						if (isProccesarAlertas()) {
							poblacionAlertas(result.getWarnings());
							result.clearWarnings();
						}
						if (debug)
							System.out
									.println("slgfp ============= "
											+ new java.util.Date()
											+ " "
											+ this.getClass()
											+ "::"
											+ this
											+ "::ejecutarProcedimientoAlmacenado()::cerrando ResultSet: "
											+ result + " \n");
						result.close();
						result = null;
					}
				} else {
					filasAfectadas = sentenciasLlamadas.getUpdateCount();
					if (filasAfectadas != -1) {
						valorDevuelto.add(new Integer(filasAfectadas));
					}
				}
				tieneResult = sentenciasLlamadas.getMoreResults();
			}

			// Procesos OUT y/o INOUT parametros
			for (int i = 1; i <= p.getParameterCount(); i++) {
				int parametrosModo = p.getParameterMode(i);
				if (parametrosModo == Parametros.parameterModeOut
						|| parametrosModo == Parametros.parameterModeInOut) {
					Object valor = sentenciasLlamadas.getObject(i);
					p.setParametros(i, valor);
				}
			}
			if (isProccesarAlertas()) {
				poblacionAlertas(sentenciasLlamadas.getWarnings());
				this.sentenciasLlamadas.clearWarnings();
			}
			valorDevuelto.add(p);
		} catch (Exception e) {
			e.printStackTrace();
			throw new SQLException(e.getMessage());
		} finally {
			if (isCerrarSentenciasDespuesEjecutar()) {
				if (debug)
					System.out
							.println("slgfp ============= "
									+ new java.util.Date()
									+ " "
									+ this.getClass()
									+ "::"
									+ this
									+ "::ejecutarProcedimientoAlmacenado()::cerrando SentenciasLlamadas: "
									+ sentenciasLlamadas + " \n");
				if (sentenciasLlamadas != null) {
					sentenciasLlamadas.close();
					sentenciasLlamadas = null;
				}
			}
		}
		return valorDevuelto;
	}

	public int[] ejecutarBatch(String sql, Collection batch)
			throws SQLException {
		int[] filasAfectadas;
		try {
			this.prepararSentencia(sql);
			Iterator it = batch.iterator();
			while (it.hasNext()) {
				Collection p = (Collection) it.next();
				Iterator pIt = p.iterator();
				int parIdx = 1;
				while (pIt.hasNext()) {
					Object param = pIt.next();
					if (param != null) {
						if (param.getClass().getName().equals("java.util.Date")) {
							param = new java.sql.Date(
									((java.util.Date) param).getTime());
						}
						sentenciasPreparadas.setObject(parIdx, param);
					} else {
						sentenciasPreparadas.setNull(parIdx, Types.JAVA_OBJECT);
					}
					parIdx++;
				}
				sentenciasPreparadas.addBatch();
			}
			filasAfectadas = this.sentenciasPreparadas.executeBatch();
			if (debug)
				System.out.println("slgfp ============= "
						+ new java.util.Date() + " " + this.getClass() + "::"
						+ this + "::ejecutarBatch(" + sql + ")::batch: "
						+ batch + "::filasAfectadas: "
						+ Arrays.asList(filasAfectadas) + "\n");
		} finally {
			if (isProccesarAlertas()) {
				poblacionAlertas(sentenciasPreparadas.getWarnings());
				this.sentenciasPreparadas.clearWarnings();
			}
			if (isCerrarSentenciasDespuesEjecutar()) {
				if (sentenciasPreparadas != null) {
					if (debug)
						System.out
								.println("slgfp ============= "
										+ new java.util.Date()
										+ " "
										+ this.getClass()
										+ "::"
										+ this
										+ "::ejecutarBatch()::cerrando SenteiciasPreparadas: "
										+ sentenciasPreparadas + " \n");
					sentenciasPreparadas.close();
					sentenciasPreparadas = null;
				}
			}
		}
		return filasAfectadas;
	}

	public int[] ejecutarBatch(Collection batch) throws SQLException {
		int[] filasAfectadas;
		crearSentencia();
		try {
			Iterator bIt = batch.iterator();
			while (bIt.hasNext()) {
				String sql = (String) bIt.next();
				this.sentencia.addBatch(sql);
			}
			filasAfectadas = this.sentencia.executeBatch();
			if (debug)
				System.out.println("slgfp ============= "
						+ new java.util.Date() + " " + this.getClass() + "::"
						+ this + "::ejecutarBatch()::batch: " + batch
						+ "::filasAfectadas: " + Arrays.asList(filasAfectadas)
						+ "\n");
		} finally {
			if (isProccesarAlertas()) {
				poblacionAlertas(sentencia.getWarnings());
				this.sentencia.clearWarnings();
			}
			if (isCerrarSentenciasDespuesEjecutar()) {
				if (sentencia != null) {
					if (debug)
						System.out.println("slgfp ============= "
								+ new java.util.Date() + " " + this.getClass()
								+ "::" + this
								+ "::ejecutarBatch()::cerrando Sentencia: "
								+ sentencia + " \n");
					sentencia.close();
					sentencia = null;
				}
			}
		}
		return filasAfectadas;
	}

	public int ejecutar(String sql) throws SQLException {
		int filasAfectadas = -1;
		try {
			filasAfectadas = this.getSentencia(sql).executeUpdate(sql);
		} finally {
			if (isProccesarAlertas()) {
				poblacionAlertas(sentencia.getWarnings());
				this.sentencia.clearWarnings();
			}
			if (isCerrarSentenciasDespuesEjecutar()) {
				if (sentencia != null) {
					if (debug)
						System.out.println("slgfp ============= "
								+ new java.util.Date() + " " + this.getClass()
								+ "::" + this
								+ "::ejecutarBatch()::cerrando Sentencia: "
								+ sentencia + " \n");
					sentencia.close();
					sentencia = null;
				}
			}
		}
		this.setFilasAfectadasEnTransaccion(filasAfectadasEnTransaccion
				+ filasAfectadas);
		if (debug)
			System.out.println("slgfp ============= " + new java.util.Date()
					+ " " + this.getClass() + "::" + this
					+ "::ejecutar()::sql: \n" + sql + "::filasAfectadas: "
					+ filasAfectadas + "\n");
		return filasAfectadas;
	}

	public int ejecutar(String sql, Collection p) throws SQLException {
		int filasAfectadas = -1;
		try {
			filasAfectadas = this.getPrepararSentencia(sql, p).executeUpdate();
		} finally {
			if (isProccesarAlertas()) {
				poblacionAlertas(sentencia.getWarnings());
				this.sentencia.clearWarnings();
			}
			if (isCerrarSentenciasDespuesEjecutar()) {
				if (sentenciasPreparadas != null) {
					if (debug)
						System.out
								.println("slgfp ============= "
										+ new java.util.Date()
										+ " "
										+ this.getClass()
										+ "::"
										+ this
										+ "::ejecutarBatch()::cerrando sentenciasPreparadas: "
										+ sentenciasPreparadas + " \n");
					sentenciasPreparadas.close();
					sentenciasPreparadas = null;
				}
			}
		}
		this.setFilasAfectadasEnTransaccion(filasAfectadasEnTransaccion
				+ filasAfectadas);
		if (debug)
			System.out.println("slgfp ============= " + new java.util.Date()
					+ " " + this.getClass() + "::" + this
					+ "::ejecutar()::sql: \n" + sql + " parametros: " + p
					+ "::filasAfectadas: " + filasAfectadas + "\n");
		return filasAfectadas;
	}

	public void setResultSet(java.sql.ResultSet resultSet) {
		this.resultSet = resultSet;
	}

	protected void crearSentencia() throws SQLException {
		sentencia = this.getConexion().createStatement(getTipoResultSet(),
				getConcurrencia());
		sentencia.setEscapeProcessing(this.getEscaparProcesamiento());
		sentencia.setFetchSize(this.getBuscarTamao());
		sentencia.setMaxRows(this.getMaxFilas());
		sentencia.setMaxFieldSize(this.getMaxTamaoCampo());
		sentencia.setQueryTimeout(this.getTimeoutConsultas());
		if (this.getCursorNombre() != null) {
			sentencia.setCursorName(this.getCursorNombre());
		}
	}

	public java.sql.Statement getSentencia(String sql) throws SQLException {
		if (sql == null || sql.trim().length() == 0) {
			throw new SQLException("La cadena SQL esta vacia o es nula");
		}
		int idx = -1;
		if (cacheSentencias) { // La sentencia es reusada, si no se debe cerrar
								// y fijar como valor a null
			idx = sentenciasId.indexOf(sql);
			if (idx >= 0) {
				sentencia = (Statement) sentenciasCache.get(idx);
				if (debug)
					System.out
							.println("slgfp ============= "
									+ new java.util.Date()
									+ " "
									+ this.getClass()
									+ "::"
									+ this
									+ "::=============== getStatement() ============ found cached statement: \n"
									+ sql + "::" + sentencia + " \n");
				if (sentencia == null) {
					sentenciasId.remove(idx);
					sentenciasCache.remove(idx);
					idx = -1;
				}
			}
		}
		if (idx == -1) { // no se usa cache o esta sentencia es no cacheable
			crearSentencia();
		}
		if (cacheSentencias) {
			if (idx == -1) {
				idx = addSencenciaCacheada(sql, sentencia);
				sentenciasContadorCache.add(new Integer(1));
			} else {
				Object count = sentenciasContadorCache.get(idx);
				if (count == null) {
					sentenciasContadorCache.add(new Integer(1));
				} else {
					sentenciasContadorCache.set(idx, new Integer(
							((Integer) count).intValue() + 1));
				}
			}
		}
		if (isProccesarAlertas()) {
			poblacionAlertas(sentencia.getWarnings());
			sentencia.clearWarnings();
		}
		return sentencia;
	}

	protected void cerrarResultados() throws SQLException {
		if (resultSets != null) {
			for (int i = 0; i < resultSets.size(); i++) {
				Object obj = resultSets.get(i);
				if (obj != null) {
					((ResultSet) obj).close();
					obj = null;
				}
			}
			resultSets = null;
		}
	}

	public void setSentencia(java.sql.Statement sentencia) {
		this.sentencia = sentencia;
	}

	public void liberacion(boolean cerrarConexion) throws SQLException {
		if (debug)
			System.out
					.println("slgfp ============= "
							+ new java.util.Date()
							+ " "
							+ this.getClass()
							+ "::"
							+ this
							+ "::=============== LIBERACION ============ cerrarConexion: "
							+ cerrarConexion + " \n");
		if (resultSet != null) {
			resultSet.close();
			resultSet = null;
		}
		cerrarResultados();
		if (!cacheSentencias) {
			if (sentencia != null) {
				sentencia.close();
				sentencia = null;
			}
			if (sentenciasPreparadas != null) {
				sentenciasPreparadas.close();
				sentenciasPreparadas = null;
			}
			if (sentenciasLlamadas != null) {
				sentenciasLlamadas.close();
				sentenciasLlamadas = null;
			}
		}
		if (alertas != null) {
			alertas.clear();
		}
		if (conexion != null) {
			if (cerrarConexion == true) {
				limpiarCacheSentencias();
				if (proccesarTransaccionsEnConexionCerrada == ROLLBACK) {
					conexion.rollback();
				} else if (proccesarTransaccionsEnConexionCerrada == COMMIT) {
					conexion.commit();
				}
				conexion.close();
				setConexionCerrada(true);
				conexion = null;
				this.setTransaccionEnProceso(false);
				this.setConexionId("<none>");
			} else {
				// No se hace nada en este caso
			}
		}
	}

	public void comenzandoTransaccion() throws SQLException {
		if (EJBContext == null) {
			if (debug)
				System.out
						.println("slgfp ============= "
								+ new java.util.Date()
								+ " "
								+ this.getClass()
								+ "::"
								+ this
								+ "::=============== COMENZANDO TRANSACCION ============ anterior conexionId: "
								+ conexionId + " \n");
			this.setFilasAfectadasEnTransaccion(0);
			this.getConexion().setAutoCommit(false);
			this.setConexionId(this.conexion.toString());
			this.setTransaccionEnProceso(true);
			if (isProccesarAlertas()) {
				poblacionAlertas(this.conexion.getWarnings());
				this.conexion.clearWarnings();
			}
			if (debug)
				System.out
						.println("slgfp ============= "
								+ new java.util.Date()
								+ " "
								+ this.getClass()
								+ "::"
								+ this
								+ "::=============== COMENZANDO TRANSACCION ============ actual conexionId: "
								+ conexionId + " \n");
		} else {
			if (debug)
				System.out.println("slgfp ============= "
						+ new java.util.Date() + " " + this.getClass() + "::"
						+ this + "::=============== EJBContext: " + EJBContext
						+ " \n");
		}
	}

	public void validandoTransaccion() throws SQLException {
		if (EJBContext == null) {
			if (debug)
				System.out
						.println("slgfp ============= "
								+ new java.util.Date()
								+ " "
								+ this.getClass()
								+ "::"
								+ this
								+ "::=============== VALIDANDO TRANSACCION ============= actual conexionId: "
								+ conexionId + " \n");
			this.getConexion().commit();
			if (debug)
				System.out.println("slgfp ============= "
						+ new java.util.Date() + " " + this.getClass() + "::"
						+ this
						+ "::=============== VALIDADA ================\n");
			this.getConexion().setAutoCommit(true);// ?? Sybase bug
			this.setTransaccionEnProceso(false);
			this.setConexionId("<none>");
			if (isProccesarAlertas()) {
				poblacionAlertas(this.conexion.getWarnings());
				this.conexion.clearWarnings();
			}
		} else {
			if (debug)
				System.out.println("slgfp ============= "
						+ new java.util.Date() + " " + this.getClass() + "::"
						+ this + "::=============== EJBContext: " + EJBContext
						+ " \n");
		}
	}

	public void deshacerTransaccion() throws SQLException {
		if (EJBContext != null) {
			try {
				if (debug)
					System.out.println("slgfp ============= "
							+ new java.util.Date() + " " + this.getClass()
							+ "::" + this + "::=============== EJBContext: "
							+ EJBContext
							+ ".setRollbackOnly ============= conexionId: "
							+ conexionId + " \n");
				Method m = EJBContext.getClass().getMethod("setRollbackSolo",
						new Class[] {});
				m.invoke(EJBContext, new Object[] {});
			} catch (InvocationTargetException ete) {
				throw new SQLException(ete.getTargetException().getMessage());
			} catch (Exception e) {
				throw new SQLException(e.getMessage());
			}
		} else {
			if (debug)
				System.out
						.println("slgfp ============= "
								+ new java.util.Date()
								+ " "
								+ this.getClass()
								+ "::"
								+ this
								+ "::=============== DESHACIENDO TRANSACCION ============= conexionId: "
								+ conexionId + " \n");
			this.getConexion().rollback();
			if (debug)
				System.out
						.println("slgfp ============= "
								+ new java.util.Date()
								+ " "
								+ this.getClass()
								+ "::"
								+ this
								+ "::=============== DESPUES DE DESHACER TRANSACCION =========== conexionId: "
								+ conexionId + " \n");
			this.getConexion().setAutoCommit(true);
			this.setTransaccionEnProceso(false);
			this.setConexionId("<none>");
			if (isProccesarAlertas()) {
				poblacionAlertas(this.conexion.getWarnings());
				this.conexion.clearWarnings();
			}
		}
	}

	public java.lang.String getNombreDataSource() {
		return nombreDataSource;
	}

	public void setNombreDataSource(java.lang.String nombreDataSource) {
		this.nombreDataSource = nombreDataSource;
	}

	public int getBuscarTamao() {
		return buscarTamao;
	}

	public void setBuscarTamao(int buscarTamao) {
		this.buscarTamao = buscarTamao;
	}

	public int getBuscarDireccion() {
		return buscarDireccion;
	}

	public void setBuscarDireccion(int buscarDireccion) {
		this.buscarDireccion = buscarDireccion;
	}

	public int getTimeoutConsultas() {
		return timeoutConsultas;
	}

	public void setTimeoutConsultas(int timeoutConsultas) {
		this.timeoutConsultas = timeoutConsultas;
	}

	public int getMaxFilas() {
		return filasMaximo;
	}

	public void setFilasMaximo(int filasMaximo) {
		this.filasMaximo = filasMaximo;
	}

	public int getMaxTamaoCampo() {
		return tamaoMaximoCampo;
	}

	public void setTamaoMaximoCampo(int tamaoMaximoCampo) {
		this.tamaoMaximoCampo = tamaoMaximoCampo;
	}

	public boolean getEscaparProcesamiento() {
		return escaparProcesamiento;
	}

	public void setEscaparProcesamiento(boolean escaparProcesamiento) {
		this.escaparProcesamiento = escaparProcesamiento;
	}

	public int getTipoResultSet() {
		return tipoResultSet;
	}

	public void setResultSetType(int resultSetType) {
		this.tipoResultSet = resultSetType;
	}

	public int getConcurrencia() {
		return concurrencia;
	}

	public void setConcurrencia(int concurrencia) {
		this.concurrencia = concurrencia;
	}

	public boolean isConexionCerrada() {
		return conexionCerrada;
	}

	protected void setConexionCerrada(boolean conexionCerrada) {
		this.conexionCerrada = conexionCerrada;
	}

	public int getMaxFilasAProcesarEnTransaccion() {
		return maxFilasAProcesarEnTransaccion;
	}

	public void setMaxFilasAProcesarEnTransaccion(
			int maxFilasAProcesarEnTransaccion) throws SQLException {
		if (maxFilasAProcesarEnTransaccion < 0) {
			throw new SQLException(
					"Invalid maxRowsToProccesInTransaction valor");
		}
		this.maxFilasAProcesarEnTransaccion = maxFilasAProcesarEnTransaccion;
	}

	public int getFilasAfectadasEnTransaccion() {
		return filasAfectadasEnTransaccion;
	}

	public void setFilasAfectadasEnTransaccion(int filasAfectadasEnTransaccion)
			throws SQLException {
		this.filasAfectadasEnTransaccion = filasAfectadasEnTransaccion;
		if (this.maxFilasAProcesarEnTransaccion != 0
				&& this.filasAfectadasEnTransaccion >= this.maxFilasAProcesarEnTransaccion) {
			try {
				this.validandoTransaccion();
				this.liberacion(false);
				this.comenzandoTransaccion();
			} catch (SQLException e) {
				this.deshacerTransaccion();
				throw e;
			}
		}
	}

	public java.util.Properties getPropiedadesConexion() {
		return propiedadesConexion;
	}

	public void setPropiedadesConexion(java.util.Properties propiedadesConexion) {
		this.propiedadesConexion = propiedadesConexion;
	}

	public boolean isCerrarSentenciasDespuesEjecutar() {
		return cerrarSentenciaDespuesEjecutar;
	}

	public void setCerrarSentenciaDespuesEjecutar(
			boolean cerrarSentenciaDespuesEjecutar) {
		this.cerrarSentenciaDespuesEjecutar = cerrarSentenciaDespuesEjecutar;
	}

	public java.lang.String getConexionId() {
		return conexionId;
	}

	public void setConexionId(java.lang.String conexionId) {
		this.conexionId = conexionId;
	}

	public boolean isTranEnProceso() {
		return tranEnProceso;
	}

	public void setTransaccionEnProceso(boolean tranEnProceso) {
		this.tranEnProceso = tranEnProceso;
	}

	public boolean isDebug() {
		return debug;
	}

	public void setDepurador(boolean debug) {
		this.debug = debug;
	}

	public boolean isReiniciarDataSource() {
		return reiniciarDataSource;
	}

	public void setReiniciarDataSource(boolean reiniciarDataSource) {
		this.reiniciarDataSource = reiniciarDataSource;
	}

	public java.lang.String getEspacioReservado() {
		return espacioReservado;
	}

	public void setEspacioReservado(java.lang.String espacioReservado) {
		this.espacioReservado = espacioReservado;
	}

	public int getMaxNumeroDeParametros() {
		return maxNumeroDeParametros;
	}

	public void setMaxNumeroDeParametros(int maxNumeroDeParametros) {
		this.maxNumeroDeParametros = maxNumeroDeParametros;
	}

	public List getAlertas() {
		return (alertas == null) ? new ArrayList() : alertas;
	}

	public void setProccesarAlertas(boolean proccesarAlertas) {
		if (proccesarAlertas) {
			alertas = new ArrayList();
		} else {
			alertas.clear();
			alertas = null;
		}
		this.proccesarAlertas = proccesarAlertas;
	}

	public boolean isProccesarAlertas() {
		return this.proccesarAlertas;
	}

	/* Asegrese de leer las advertencias antes de la liberacin() */
	public void poblacionAlertas(SQLWarning alerta) {
		while (alerta != null) {
			ContenedorDatos h = new ContenedorDatos(3);
			h.setNombreCampoYValor(1, "message", alerta.getMessage());
			h.setNombreCampoYValor(2, "SQLState", alerta.getSQLState());
			h.setNombreCampoYValor(3, "vErrorCodigo",
					new Integer(alerta.getErrorCode()));
			alertas.add(h);
			alerta = alerta.getNextWarning();
		}
	}

	public boolean isCacheSentencias() {
		return cacheSentencias;
	}

	public void limpiarCacheSentencias() {
		if (sentenciasCache != null) {
			try {
				Iterator it = sentenciasCache.iterator();
				while (it.hasNext()) {
					Object obj = it.next();
					if (obj != null) {
						((Statement) obj).close();
					}
				}
				sentenciasCache.clear();
				sentenciasContadorCache.clear();
				sentenciasId.clear();
				if (!cacheSentencias) {
					sentenciasContadorCache = null;
					sentenciasCache = null;
					sentenciasId = null;
				}
			} catch (SQLException sqle) {
				sqle.printStackTrace();
			}
		}
	}

	public void setCacheSentencias(boolean cacheSentencias) {
		if (debug)
			System.out
					.println("slgfp ============= "
							+ new java.util.Date()
							+ " "
							+ this.getClass()
							+ "::"
							+ this
							+ "::=============== setCacheSentencias() ============= cacheSentencias: "
							+ cacheSentencias + " \n");
		if (cacheSentencias) {
			if (sentenciasCache == null) {
				sentenciasCache = new ArrayList(maxTamaoCacheSentencias);
				sentenciasId = new ArrayList(maxTamaoCacheSentencias);
				sentenciasContadorCache = new ArrayList(
						maxTamaoCacheSentencias);
			}
			setCerrarSentenciaDespuesEjecutar(false);
		} else {
			limpiarCacheSentencias();
		}
		this.cacheSentencias = cacheSentencias;
	}

	public int getMaxTamaoCacheSentencias() {
		return this.maxTamaoCacheSentencias;
	}

	public void setMaxTamaoCacheSentencias(int maxTamaoCacheSentencias) {
		this.maxTamaoCacheSentencias = maxTamaoCacheSentencias;
	}

	protected synchronized int aadirCacheSentencias(String sentenciaId,
			PreparedStatement sentenciaACachear) throws SQLException {
		if (debug)
			System.out
					.println("slgfp ============= "
							+ new java.util.Date()
							+ " "
							+ this.getClass()
							+ "::"
							+ this
							+ "::=============== aadirCacheSentencias() ============= sentenciaId: "
							+ sentenciaId + "::sentenciaACachear: "
							+ sentenciaACachear + "\n");
		if (sentenciasId.isEmpty()) {
			sentenciasId.add(sentenciaId);
			sentenciasCache.add(sentencia);
			return 0;
		}
		int idx = sentenciasId.indexOf(sentenciaId);
		if (idx < 0) {
			if (sentenciasId.size() + 1 == maxTamaoCacheSentencias) {
				// Buscar y eliminar
				int contadorIt = 0;
				int sI = 0;
				int s = ((Integer) sentenciasContadorCache.get(0)).intValue();
				Iterator it = sentenciasContadorCache.iterator();
				while (it.hasNext()) {
					int valorActual = ((Integer) it.next()).intValue();
					if (valorActual < s) {
						s = valorActual;
						sI = contadorIt;
					}
					contadorIt++;
				}
				sentenciasId.remove(sI);
				if (debug)
					System.out
							.println("slgfp ============= "
									+ new java.util.Date()
									+ " "
									+ this.getClass()
									+ "::"
									+ this
									+ "::aadirCacheSentencias()::cerrando sentenciaPreparada: "
									+ ((Statement) sentenciasCache.get(sI))
									+ " \n");
				((Statement) sentenciasCache.get(sI)).close();
				sentenciasCache.remove(sI);
				sentenciasContadorCache.remove(sI);
				if (debug)
					System.out
							.println("slgfp ============= "
									+ new java.util.Date()
									+ " "
									+ this.getClass()
									+ "::"
									+ this
									+ "::=============== aadirCacheSentencias()::eliminar ============= sI: "
									+ sI + "\n");
			}
			sentenciasId.add(sentenciaId);
			if (debug)
				System.out
						.println("slgfp ============= "
								+ new java.util.Date()
								+ " "
								+ this.getClass()
								+ "::"
								+ this
								+ "::=============== aadirCacheSentencias()::sentenciaACache: "
								+ sentenciaACachear + " "
								+ sentenciaACachear.getClass() + "\n");
			if (debug)
				System.out
						.println("slgfp ============= "
								+ new java.util.Date()
								+ " "
								+ this.getClass()
								+ "::"
								+ this
								+ "::=============== aadirCacheSentencias()::sentenciaACache instanceof sentenciaPreparada: "
								+ (sentenciaACachear instanceof PreparedStatement)
								+ "\n");
			if (sentenciaACachear instanceof PreparedStatement) {
				sentenciasCache.add((PreparedStatement) sentenciaACachear);
			} else {
				sentenciasCache.add(sentenciaACachear);
			}
			idx = sentenciasId.size() - 1;
		} else {
			// Como la sentencia ya est en cache no hay nada que hacer
		}
		return idx;
	}

	protected synchronized int addSencenciaCacheada(String sentenciaId,
			Statement sentenciaACache) throws SQLException {
		if (debug)
			System.out
					.println("slgfp ============= "
							+ new java.util.Date()
							+ " "
							+ this.getClass()
							+ "::"
							+ this
							+ "::=============== addSencenciaCacheada() ============= sentenciaId: "
							+ sentenciaId + "::sentenciaACache: "
							+ sentenciaACache + "\n");
		if (sentenciasId.isEmpty()) {
			sentenciasId.add(sentenciaId);
			sentenciasCache.add(sentencia);
			return 0;
		}
		int idx = sentenciasId.indexOf(sentenciaId);
		if (idx < 0) {
			if (sentenciasId.size() + 1 == maxTamaoCacheSentencias) {
				// busca y elimina
				int iC = 0;
				int sI = 0;
				int s = ((Integer) sentenciasContadorCache.get(0)).intValue();
				Iterator iter = sentenciasContadorCache.iterator();
				while (iter.hasNext()) {
					int currValue = ((Integer) iter.next()).intValue();
					if (currValue < s) {
						s = currValue;
						sI = iC;
					}
					iC++;
				}
				sentenciasId.remove(sI);
				if (debug)
					System.out
							.println("slgfp ============= "
									+ new java.util.Date()
									+ " "
									+ this.getClass()
									+ "::"
									+ this
									+ "::aadirSentenciaCacheada()::cerrando sentencia: "
									+ ((Statement) sentenciasCache.get(sI))
									+ " \n");
				((Statement) sentenciasCache.get(sI)).close();
				sentenciasCache.remove(sI);
				sentenciasContadorCache.remove(sI);
				if (debug)
					System.out
							.println("slgfp ============= "
									+ new java.util.Date()
									+ " "
									+ this.getClass()
									+ "::"
									+ this
									+ "::=============== aadirSentenciaCacheada()::eliminando ============= sI: "
									+ sI + "\n");
			}
			sentenciasId.add(sentenciaId);
			if (debug)
				System.out
						.println("slgfp ============= "
								+ new java.util.Date()
								+ " "
								+ this.getClass()
								+ "::"
								+ this
								+ "::=============== aadirSentenciaCacheada()::sentenciaACache: "
								+ sentenciaACache + " "
								+ sentenciaACache.getClass() + "\n");
			if (debug)
				System.out
						.println("slgfp ============= "
								+ new java.util.Date()
								+ " "
								+ this.getClass()
								+ "::"
								+ this
								+ "::=============== aadirSentenciaCacheada()::sentenciaACache instanceof PreparedStatement: "
								+ (sentenciaACache instanceof PreparedStatement)
								+ "\n");
			if (sentenciaACache instanceof PreparedStatement) {
				sentenciasCache.add((PreparedStatement) sentenciaACache);
			} else {
				sentenciasCache.add(sentenciaACache);
			}
			idx = sentenciasId.size() - 1;
		} else {
			// Como la sentencia ya est en cache no tenemos nada que hacer
		}
		return idx;
	}

	public String getCacheoFilaFijaNombreClase() {
		return cacheoFilaFijaNombreClase;
	}

	public void setCacheoFilaFijaNombreClase(String cacheoFilaFijaNombreClase) {
		this.cacheoFilaFijaNombreClase = cacheoFilaFijaNombreClase;
	}

	public void setCerrarResultSet(boolean cerrarResultSet) {
		this.cerrarResultSet = cerrarResultSet;
	}

	public boolean isCerradoResultSet() {
		return this.cerrarResultSet;
	}

	public String getCursorNombre() {
		return nombreCursor;
	}

	public void setCursorName(String cursorName) {
		this.nombreCursor = cursorName;
	}

	public char getProccesarTransaccionsEnConexionCerrada() {
		return proccesarTransaccionsEnConexionCerrada;
	}

	public void setProccesarTransaccionsEnConexionCerrada(
			char proccesarTransaccionsEnConexionCerrada) {
		this.proccesarTransaccionsEnConexionCerrada = proccesarTransaccionsEnConexionCerrada;
	}

	public Object getEJBContext() {
		return EJBContext;
	}

	public void setEJBContext(Object EJBContext) {
		this.EJBContext = EJBContext;
	}
}
