package slgfp;

import javax.sql.DataSource;
import java.io.Serializable;
import java.sql.*;
import java.util.Properties;
import java.util.StringTokenizer;

/**
 * La clase se usa para realizar pruebas como cliente autnomo. El objeto en si
 * no tiene la capacidad de crear el pool.
 * 
 * @version 1.0, 03-05-2011
 * @author Sergio Lovillo
 */

@SuppressWarnings({ "serial", "rawtypes", "unchecked" })
public class DataSourceBasicoImpl implements DataSource, Serializable {

	/**
	 * DataSourceBasicoImpl no tiene capacidad en si mismo y debe ser utilizado
	 * para probar o a modo de cliente stand-along
	 */

	public DataSourceBasicoImpl() {
		if (Boolean.getBoolean("debug-slgfp")
				|| Boolean.getBoolean("debug-" + getClass().getName())) {
			debug = true;
		}
	}

	protected transient Connection conexion;
	protected Driver controlador;
	protected Properties propiedadesConexion = null;
	private String nombreControlador;
	private String dbURL;
	private String nombreUsuario;
	private String password;
	protected String conexionId = "";
	protected boolean debug = false;
	protected String delimitadorPropiedadesConexion = "&";

	// Nos permite clonar el objeto
	protected DataSourceBasicoImpl clone() {
		DataSourceBasicoImpl newOne = new DataSourceBasicoImpl();
		try {
			newOne.setNombreUsuario(nombreUsuario);
			newOne.setPassword(password);
			newOne.setPropiedadesConexion(propiedadesConexion);
			newOne.setDbURL(dbURL);
			newOne.setDebug(debug);
			newOne.setNombreControlador(nombreControlador);
			newOne.setLogWriter(DriverManager.getLogWriter());
			newOne.setLoginTimeout(DriverManager.getLoginTimeout());
		} catch (SQLException sqle) {
			throw new RuntimeException(sqle);
		}
		return newOne;
	}

	// Nos devuelve loa conexion del objeto
	public java.sql.Connection getConnection() throws SQLException {
		if (conexion == null || conexion.isClosed()) {
			if (debug)
				System.out.println("slgfp ============= "
						+ new java.util.Date() + " " + this.getClass() + "::"
						+ this + "::getConnection()::conexion 1: " + conexion
						+ " \n");
			try {
				controlador = (Driver) Class.forName(getNombreControlador())
						.newInstance();
				if (this.getPropiedadesConexion() == null
						|| this.getPropiedadesConexion().isEmpty()) {
					conexion = DriverManager.getConnection(getDbURL(),
							getNombreUsuario(), getPassword());
				} else {
					conexion = DriverManager.getConnection(getDbURL(),
							this.getPropiedadesConexion());
				}
			} catch (ClassNotFoundException ex) {
				ex.printStackTrace();
				throw new RuntimeException(ex);
			} catch (SQLException sqle) {
				sqle.printStackTrace();
				throw new RuntimeException(sqle);
			} catch (Exception e) {
				e.printStackTrace();
				throw new RuntimeException(e);
			}
		}
		if (debug)
			System.out.println("slgfp ============= " + new java.util.Date()
					+ " " + this.getClass() + "::" + this
					+ "::getConnection()::conexion 2: " + conexion + " \n");
		return conexion;
	}

	public Connection getConnection(String username, String password)
			throws SQLException {
		setNombreUsuario(nombreUsuario);
		setPassword(password);
		return getConnection();
	}

	public java.io.PrintWriter getLogWriter() throws SQLException {
		return DriverManager.getLogWriter();
	}

	public int getLoginTimeout() throws SQLException {
		return DriverManager.getLoginTimeout();
	}

	public void setLogWriter(java.io.PrintWriter out) throws SQLException {
		DriverManager.setLogWriter(out);
	}

	public void setLoginTimeout(int seconds) throws SQLException {
		DriverManager.setLoginTimeout(seconds);
	}

	public java.lang.String getConexionId() {
		return conexionId;
	}

	public boolean isDebug() {
		return debug;
	}

	public void setDebug(boolean debug) {
		this.debug = debug;
	}

	public String getNombreControlador() {
		return this.nombreControlador;
	}

	public void setNombreControlador(String nombreControlador) {
		this.nombreControlador = nombreControlador;
	}

	public java.lang.String getDbURL() {
		return dbURL;
	}

	public void setDbURL(java.lang.String dbURL) {
		this.dbURL = dbURL;
	}

	public java.lang.String getNombreUsuario() {
		return nombreUsuario;
	}

	public void setNombreUsuario(java.lang.String nombreUsuario) {
		this.nombreUsuario = nombreUsuario;
	}

	public java.lang.String getPassword() {
		return password;
	}

	public void setPassword(java.lang.String password) {
		this.password = password;
	}

	public void cerrarConexion() throws SQLException {
		if (conexion != null) {
			conexion.close();
			conexion = null;
		}
	}

	public void setPropiedadesConexion(String propiedadesConexion) {
		StringTokenizer tokenizer = new StringTokenizer(propiedadesConexion,
				delimitadorPropiedadesConexion, false);
		if (this.propiedadesConexion == null) {
			this.propiedadesConexion = new Properties();
		} else {
			this.propiedadesConexion.clear();
		}
		while (tokenizer.hasMoreTokens()) {
			String token = tokenizer.nextToken();
			int idx = token.indexOf("=");
			if (idx == -1) {
				throw new IllegalArgumentException(
						"DataSourceBasicoImpl::setPropiedadesConexion(String propiedadesConexion): "
								+ propiedadesConexion);
			}
			this.propiedadesConexion.put(token.substring(0, idx).trim(), token
					.substring(idx + 1).trim());
		}
	}

	public Properties getPropiedadesConexion() {
		return propiedadesConexion;
	}

	public void setPropiedadesConexion(Properties propiedadesConexion) {
		this.propiedadesConexion = propiedadesConexion;
	}

	public String getDelimitadorPropiedadesConexion() {
		return delimitadorPropiedadesConexion;
	}

	public void setDelimitadorPropiedadesConexion(
			String delimitadorPropiedadesConexion) {
		this.delimitadorPropiedadesConexion = delimitadorPropiedadesConexion;
	}

	public boolean isWrapperFor(Class iface) {
		return false;
	}

	public Class unwrap(Class iface) {
		return null;
	}
}
