package slgfp;

import java.util.*;
import java.sql.*;
import java.lang.reflect.*;
import javax.sql.DataSource;

/**
 * Nos permite la gestin de la capa de persistencia del framework
 * 
 * @version 1.0, 03-05-2011
 * @author Sergio Lovillo
 */

@SuppressWarnings({ "serial", "rawtypes", "unchecked", "static-access" })
public abstract class FabricaObjPersistencia implements java.io.Serializable,
		Cloneable {

	public FabricaObjPersistencia() {
		if (Boolean.getBoolean("debug-slgfp")
				|| Boolean.getBoolean("debug-" + getClass().getName())) {
			debug = true;
		}
	}

	public FabricaObjPersistencia(boolean accesoSincronizado) {
		this.setAccesoSincronizado(accesoSincronizado);
	}

	public FabricaObjPersistencia(Class requestor, boolean accesoSincronizado) {
		this.solicitante = requestor;
		this.setAccesoSincronizado(accesoSincronizado);
	}

	public FabricaObjPersistencia(Class requestor) {
		this.solicitante = requestor;
	}

	/* ================================== Atributos ======================= */
	protected boolean debug = false;

	protected Metadatos metadatos = null;
	/* Map de las columnas de las tablas de la base de datos a campos del objeto */
	protected Map asignaciones = null;
	/* Objeto a persistencia */
	protected Class solicitante = null;
	/* objectoId */
	protected int objectoId = 0;
	/* Coleccion de Objetos (datos) */
	protected List listaObjetos = new ArrayList();
	/* Coleccion de Objetos a eliminar de la base de datos */
	protected List eliminarObjetos = new ArrayList();
	/*
	 * Se usa para optimizar el sistema enviando solo los objetos que hayan sido
	 * modificados
	 */
	protected boolean descartarObjetosNoModificados;
	/* Define las secuencias de operaciones update/insert/delete */
	protected char[] secuenciaModificacionDatos;
	public static final char SECUENCIA_MODIFICACION_DATOS_DELETE = 'D';
	public static final char SECUENCIA_MODIFICACION_DATOS_INSERT = 'I';
	public static final char SECUENCIA_MODIFICACION_DATOS_UPDATE = 'U';
	/* Desactivar modificaciones de la base de datos */
	protected boolean soloLectura = false;

	/* Configuracion de bloqueo optimista */
	/* Generar sentencias WHERE con campos clave unicamente */
	public static final char KEY_COLUMNAS = 'K';
	/*
	 * Generar sentencias WHERE con campos clave unicamente y columnas
	 * modificadas
	 */
	public static final char KEY_Y_COLUMNA_MODIFICADA = 'M';
	/*
	 * Generar sentencias WHERE con campos clave unicamente y todas las columnas
	 * actualizables. Se mantiene un estricto control de concurrencia
	 */
	public static final char KEY_Y_COLUMNA_ACTUALIZADA = 'U';
	/*
	 * Configura el actual bloqueo optimista. por defecto es KEY_COLUMNAS (Slo
	 * con con columnas de clave primaria.
	 */
	protected char bloqueoOptimista = KEY_COLUMNAS;
	/*
	 * Lanza una excepcin si PreparedStatement.executeUpdate () no devuelve 1
	 * para cada ObjPersistencia. Esto significa que lo ms probable sea que la
	 * fila en la tabla de base de datos fue Modificada o eliminada por otro
	 * usuario / proceso.
	 */
	protected boolean lanzarBloqueoOptimistaDeleteExcepcion = true;
	protected boolean lanzarBloqueoOptimistaUpdateExcepcion = true;
	protected boolean lanzarFalloInsertExcepcion = true;

	/* FIN configuracion bloqueo optimista */
	/* Catalogo de la base de datos */
	protected String catalogo = null;
	/*
	 * Si reemplazaNombreCatalogo es true y catalogo desde this.metaData es nulo
	 * entonces usaremos un catalogo de miembros variables
	 */
	protected boolean reemplazaNombreCatalogo = false;
	/* Esquema de la base de datos */
	protected String esquema = null;
	/* Tabla de la base de datos */
	protected String tabla = null;
	/* Este objeto genera el SQL para la modificacin de datos */
	protected GrabarDatos grabarDatos = null;
	/* Utilidad de manejo JDBC */
	protected DBServicios dbServicios = null;
	/*
	 * Internamente se usa el argumento en el mtodo invoke(obj,
	 * VACIO_LECTURA_ARG)
	 */
	protected Object[] VACIO_LECTURA_ARG = new Object[0];
	/* Uso de colecciones sincronizadas */
	protected boolean accesoSincronizado = false;
	/*
	 * Si cargaDiferida == cierto establece slo los valores originales vs
	 * originales y actuales
	 */
	protected boolean cargaDiferida = false;
	/* Registros por pagina */
	protected int tamaoPagina = 0;
	/*
	 * No ejecuta las operaciones INSERTS/UPDATES/DELETES - se limita a generar
	 * el SQL y los parametros
	 */
	protected boolean generarSQLSolo = false;
	/* Supervisa si cargar() aade objetos a la lista de objetos */
	protected boolean aadir = false;
	/*
	 * Algunas bases de datos (como hsqdb 1.7.1 ) fallan cuando las instruccin
	 * UPDATE utilizan TABLA.COLUMNA en la clusula SET. INSERT tambin falla.
	 */
	protected boolean noNombresColumnasCompletos = false;
	/*
	 * Lanza excepcin o ignora si ObjPersistencia tiene menos campos que el
	 * mapeo
	 */
	protected boolean lanzaCampoInexistenteExcepcion = false;
	/*
	 * Utilizada para determinar si se crean/borran/actualizan ObjPersistencia
	 */
	protected long fabricaPersistenteId = System.currentTimeMillis();
	/* Si el objeto es no null debera manejar excepciones */
	protected ManejadorExcepciones manejadorExcepciones;
	/*
	 * Si es falso suprimir excepcin cuando se llame a grabarDatos con la lista
	 * de objetos vaca
	 */
	protected boolean excepcionListaObjetosVacia = false;
	/* Solo un miembro/metodo */
	protected boolean valido = true;
	/* Crear y rellenar automticamente Metadatos */
	protected boolean crearDefectoMetadato = true;
	/* Obtenga el valor de Resultset tal como se especifica datatype */
	protected Transformador transformador = null;

	public int cargar(ResultSet[] rs) throws java.sql.SQLException {
		boolean cargaOriginal = this.isAadir();
		this.setAadir(true);
		int numeroRegistros = 0;
		if (debug)
			System.out.println("slgfp @@@@@@@@@@@@@@ rs.length: " + rs.length);
		if (debug)
			System.out.println("slgfp @@@@@@@@@@@@@@ rs: " + Arrays.asList(rs));

		for (int i = 0; i < rs.length; i++) {
			if (debug)
				System.out.println("slgfp @@@@@@@@@@@@@@ resultSet[" + i
						+ "]: " + rs[i]);
			numeroRegistros = numeroRegistros + cargar(rs[i]);
			if (debug)
				System.out.println("slgfp @@@@@@@@@@@@@@ numeroRegistros_" + i
						+ ": " + numeroRegistros);
		}
		this.setAadir(cargaOriginal);
		return numeroRegistros;
	}

	public int cargar(ResultSet rs) throws java.sql.SQLException {
		long ts1 = -1;
		long ts2 = -1;
		String nombreColumna = null;
		String nombreCampo = null;
		ResultSetMetaData rsmd = rs.getMetaData();
		int rsColCotador = rsmd.getColumnCount();
		List rsColumnas = new ArrayList(rsColCotador);
		for (int i = 1; i <= rsColCotador; i++) {
			rsColumnas.add(rsmd.getColumnName(i));
		}
		int numeroColumnas = 0;
		if (debug)
			System.out
					.println("slgfp ============= " + new java.util.Date()
							+ " " + this.getClass() + "::" + this
							+ "::cargar()::solicitante clase::"
							+ solicitante.getName());
		if (!isAadir()) {
			this.listaObjetos.clear();
			if (debug)
				System.out.println("slgfp ============= "
						+ new java.util.Date() + " " + this.getClass() + "::"
						+ this + "::cargar()::objectoId 1: " + objectoId);
			this.objectoId = 0;
			if (debug)
				System.out.println("slgfp ============= "
						+ new java.util.Date() + " " + this.getClass()
						+ "::cargar()::objectoId 2: " + objectoId);
		}
		try {
			if (debug)
				ts1 = System.currentTimeMillis();
			if (this.metadatos == null) {
				poblarMetaDatos();
				if (debug)
					ts2 = System.currentTimeMillis();
				if (debug)
					System.out.println("slgfp ============= "
							+ new java.util.Date() + " " + this.getClass()
							+ "::" + this + "::poblarMetaDatos: " + (ts2 - ts1)
							+ " ms");
			}
			numeroColumnas = this.metadatos.getNumeroColumnas();
			if (debug)
				System.out.println("slgfp ============= "
						+ new java.util.Date() + " " + this.getClass() + "::"
						+ this + "::load()::numeroColumnas: " + numeroColumnas);
			if (transformador == null) {
				transformador = new TransformadorDefecto(rs);
			} else {
				transformador.setResultSet(rs);
			}
			while (rs.next()) {
				ContenedorDatos valoresOriginales = null;
				if (!isSoloLectura()) {
					valoresOriginales = new ContenedorDatos(numeroColumnas);
				}
				Object objPersistencia = solicitante.newInstance();
				if (!isCargaDiferida()) {
					((ObjPersistencia) objPersistencia).setCargando(true);
				}
				for (int iC = 1; iC <= numeroColumnas; iC++) {
					nombreCampo = this.metadatos.getNombreCampo(iC);
					if (debug)
						System.out.println("slgfp ============= "
								+ new java.util.Date() + " " + this.getClass()
								+ "::" + this + "::load()::nombreCampos: "
								+ nombreCampo);
					if (nombreCampo != null) { // ResultSet puede tener mas
												// columnas que campos, con lo
												// que omitiremos las columnas
												// innecesarias
						nombreColumna = this.metadatos.getNombreColumna(iC);
						if (!rsColumnas.contains(nombreColumna)) {
							// ResultSet tiene menos columnas que campos
							// ObjPersistencia
							this.metadatos.setModificable(iC, false);
							continue;
						}
						String nombreCampoCalse = this.metadatos
								.getNombreCampoCalse(iC);
						Object[] valorConvertido = transformador.transformar(
								nombreCampoCalse, nombreColumna);
						if (!isSoloLectura()) {
							valoresOriginales.setNombreCampoYValor(iC,
									nombreCampo, valorConvertido[0]);
						}
						if (!isCargaDiferida() || isSoloLectura()) {
							this.metadatos.getEscribirMetodos(iC).invoke(
									objPersistencia, valorConvertido);
						}
					}
					if (debug)
						System.out.print("\n");
				}
				if (!isSoloLectura()) {
					((ObjPersistencia) objPersistencia)
							.setValoresOriginales(valoresOriginales);
				}
				if (!isCargaDiferida()) {
					((ObjPersistencia) objPersistencia).setCargando(false);
					((ObjPersistencia) objPersistencia).setCargado(true);
				}
				if (isAccesoSincronizado()) {
					((ObjPersistencia) objPersistencia).setObjectoId(this
							.getSincronizarProximoObjetoId());
				} else {
					((ObjPersistencia) objPersistencia).setObjectoId(this
							.getProximoObjetoId());
				}
				((ObjPersistencia) objPersistencia)
						.setEstadoPersistencia(ObjPersistencia.ORIGINAL);
				((ObjPersistencia) objPersistencia)
						.setFabricaPersitenciaId(fabricaPersistenteId);
				this.listaObjetos.add(objPersistencia);
			}
		} catch (InstantiationException ie) {
			ie.printStackTrace();
			throw new java.sql.SQLException("No puedo inicializar: "
					+ solicitante);
		} catch (IllegalAccessException iae) {
			iae.printStackTrace();
			throw new java.sql.SQLException(iae.getMessage());
		} catch (InvocationTargetException ite) {
			ite.printStackTrace();
			throw new java.sql.SQLException(ite.getTargetException()
					.getMessage());
		} catch (Exception e) {
			e.printStackTrace();
			throw new java.sql.SQLException(e.getMessage());
		}
		ts2 = System.currentTimeMillis();
		if (debug)
			System.out.println("slgfp ============= " + new java.util.Date()
					+ " " + this.getClass() + "::" + this
					+ "::cargar() total: " + (ts2 - ts1 + " ms"));
		return this.listaObjetos.size();
	}

	public Map getMapping() {
		return asignaciones;
	}

	public void setAsignaciones(Map asignaciones) {
		this.asignaciones = asignaciones;
	}

	public void setAsignaciones(String asignaciones) {
		StringTokenizer t = new StringTokenizer(asignaciones, ",", false);
		if (this.asignaciones == null) {
			this.asignaciones = new Properties();
		} else {
			this.asignaciones.clear();
		}
		while (t.hasMoreTokens()) {
			String token = t.nextToken();
			int idx = token.indexOf("=");
			if (idx == -1) {
				throw new IllegalArgumentException(
						"FabricaObjPersistencia::setAsignaciones(String asignaciones): "
								+ asignaciones
								+ "(Formato de argumento: <NOMBRE_COLUMNA>=<nombreCampos>)");
			}
			this.asignaciones.put(token.substring(0, idx).trim(), token
					.substring(idx + 1).trim());
		}
	}

	public java.lang.Class getSolicitante() {
		return solicitante;
	}

	public void setSolicitante(java.lang.Class solicitante) {
		this.solicitante = solicitante;
	}

	public List getClavePrimaria(String catalogo, String esquema, String tabla)
			throws SQLException {
		Object cp = PKCache.getInstance().getClavePrimaria(catalogo, esquema,
				tabla);
		if (cp == null) {
			cp = PKCache.getInstance().getClavePrimaria(
					this.dbServicios.getConexion(), catalogo, esquema, tabla);
		}
		return (List) cp;
	}

	public void setClavePrimaria(String catalogo, String esquema, String tabla,
			List cp) {
		PKCache.getInstance().setClavePrimaria(catalogo, esquema, tabla, cp);
	}

	public int setObjeto(ObjPersistencia objPersistencia) {
		int idx = -1;
		int id = 0;
		if (objPersistencia.getFabricaPersistenciaId() != fabricaPersistenteId) {
			objPersistencia.setFabricaPersitenciaId(fabricaPersistenteId);
			if (isAccesoSincronizado()) {
				id = this.getSincronizarProximoObjetoId();
				objPersistencia.setObjectoId(id);
			} else {
				id = this.getProximoObjetoId();
				objPersistencia.setObjectoId(id);
			}
			this.listaObjetos.add(objPersistencia);
			idx = this.listaObjetos.size() + 1;
		} else {
			id = objPersistencia.getObjectoId();
			if (id == 0) { // Nuevo objeto
				if (isAccesoSincronizado()) {
					id = this.getSincronizarProximoObjetoId();
					objPersistencia.setObjectoId(id);
				} else {
					id = this.getProximoObjetoId();
					objPersistencia.setObjectoId(id);
				}
				this.listaObjetos.add(objPersistencia);
				idx = this.listaObjetos.size() + 1;
			} else {
				idx = buscaObjetoIdPorIndice(id);
				if (idx > 0) {
					this.listaObjetos.set(idx - 1, objPersistencia);
				} else {
					this.listaObjetos.add(objPersistencia);
				}
			}
		}
		return idx;
	}

	/**
	 * Este no es un ndice basado en cero: idx del primer objeto es 1, el
	 * segundo es 2, etc...
	 */
	public void setObjecto(int indice, ObjPersistencia objPersistencia) {
		int id = 0;
		if (objPersistencia.getFabricaPersistenciaId() != fabricaPersistenteId) {
			objPersistencia.setFabricaPersitenciaId(fabricaPersistenteId);
			if (isAccesoSincronizado()) {
				id = this.getSincronizarProximoObjetoId();
				objPersistencia.setObjectoId(id);
			} else {
				id = this.getProximoObjetoId();
				objPersistencia.setObjectoId(id);
			}
		} else {
			if (objPersistencia.getObjectoId() == 0) {
				if (isAccesoSincronizado()) {
					id = this.getSincronizarProximoObjetoId();
					objPersistencia.setObjectoId(id);
				} else {
					id = this.getProximoObjetoId();
					objPersistencia.setObjectoId(id);
				}
			}
		}
		this.listaObjetos.set(indice - 1, objPersistencia);
	}

	/**
	 * Este no es un ndice basado en cero: idx del primer objeto es 1, el
	 * segundo es 2, etc...
	 */
	public void addObjeto(int index, ObjPersistencia objPersistencia) {
		int id = 0;
		if (objPersistencia.getFabricaPersistenciaId() != fabricaPersistenteId) {
			objPersistencia.setFabricaPersitenciaId(fabricaPersistenteId);
			if (isAccesoSincronizado()) {
				id = this.getSincronizarProximoObjetoId();
				objPersistencia.setObjectoId(id);
			} else {
				id = this.getProximoObjetoId();
				objPersistencia.setObjectoId(id);
			}
		} else {
			if (objPersistencia.getObjectoId() == 0) {
				if (isAccesoSincronizado()) {
					id = this.getSincronizarProximoObjetoId();
					objPersistencia.setObjectoId(id);
				} else {
					id = this.getProximoObjetoId();
					objPersistencia.setObjectoId(id);
				}
			}
		}
		this.listaObjetos.add(index - 1, objPersistencia);
	}

	public int addObjeto(ObjPersistencia objPersistencia) {
		int id = 0;
		if (objPersistencia.getFabricaPersistenciaId() != fabricaPersistenteId) {
			objPersistencia.setFabricaPersitenciaId(fabricaPersistenteId);
			if (isAccesoSincronizado()) {
				id = this.getSincronizarProximoObjetoId();
				objPersistencia.setObjectoId(id);
			} else {
				id = this.getProximoObjetoId();
				objPersistencia.setObjectoId(id);
			}
		} else {
			if (objPersistencia.getObjectoId() == 0) {
				if (isAccesoSincronizado()) {
					id = this.getSincronizarProximoObjetoId();
					objPersistencia.setObjectoId(id);
				} else {
					id = this.getProximoObjetoId();
					objPersistencia.setObjectoId(id);
				}
			}
		}
		this.listaObjetos.add(objPersistencia);
		return this.listaObjetos.size() - 1;
	}

	public ObjPersistencia buscarObjetoIdPorIndice(int objectId) {
		int idx = buscaObjetoIdPorIndice(objectId);
		return (ObjPersistencia) this.listaObjetos.get(idx - 1);
	}

	/**
	 * Este no es un ndice basado en cero: idx del primer objeto es 1, el
	 * segundo es 2, etc...
	 */
	public int buscaObjetoIdPorIndice(int objectId) {
		int id = 0;
		int idx = 0;
		Iterator it = this.listaObjetos.iterator();
		while (it.hasNext()) {
			idx++;
			ObjPersistencia objPersistencia = (ObjPersistencia) it.next();
			id = objPersistencia.getObjectoId();
			if (objectId == id) {
				break;
			}
		}
		return idx;
	}

	protected synchronized int getSincronizarProximoObjetoId() {
		this.objectoId++;
		return this.objectoId;
	}

	protected int getProximoObjetoId() {
		this.objectoId++;
		return this.objectoId;
	}

	public List getListaObjetos() {
		return listaObjetos;
	}

	public void setListaObjetos(List listaObjetos) {
		this.listaObjetos = listaObjetos;
	}

	/* No elimina de la base de datos, sino simplemente de la lista de objetos */
	public boolean descartar(ObjPersistencia objPersistencia) {
		int idx = buscaObjetoIdPorIndice(objPersistencia.getObjectoId());
		if (idx > 0) {
			this.listaObjetos.remove(idx - 1);
			return true;
		}
		return false;
	}

	public boolean eliminar(ObjPersistencia objPersistencia) {
		boolean exito = false;
		int eliminarIdx = -1;
		if (objPersistencia.getFabricaPersistenciaId() == fabricaPersistenteId) {
			int idx = buscaObjetoIdPorIndice(objPersistencia.getObjectoId());
			if (!antesEliminar(objPersistencia)) {
				return exito;
			}
			if (idx > 0) {
				if (objPersistencia.getEstadoPersistencia() == ObjPersistencia.CREADO) {
					this.listaObjetos.remove(idx - 1); /*
														 * No elimina de la base
														 * de datos, sino
														 * simplemente de la
														 * lista de objetos
														 */
					exito = false;
				} else {
					if (objPersistencia.eliminar()) {
						this.eliminarObjetos.add(objPersistencia);
						eliminarIdx = this.eliminarObjetos.size() + 1;
						this.listaObjetos.remove(idx - 1);
						exito = true;
					} else {
						exito = false;
					}
				}
			} else {
				if (isAccesoSincronizado()) {
					objPersistencia.setObjectoId(this
							.getSincronizarProximoObjetoId());
				} else {
					objPersistencia.setObjectoId(this.getProximoObjetoId());
				}
				if (objPersistencia.getEstadoPersistencia() == ObjPersistencia.CREADO) {
					/*
					 * No elimina de la base de datos, sino simplemente de la
					 * lista de objetos
					 */
					exito = false;
				} else {
					if (objPersistencia.eliminar()) {
						this.eliminarObjetos.add(objPersistencia);
						exito = true;
					}
				}
			}
			if (!despuesQuitar(eliminarIdx)) {
				return false;
			}
		} else {
			objPersistencia.setFabricaPersitenciaId(fabricaPersistenteId);
			if (!antesEliminar(objPersistencia)) {
				return false;
			}
			if (isAccesoSincronizado()) {
				objPersistencia.setObjectoId(this
						.getSincronizarProximoObjetoId());
			} else {
				objPersistencia.setObjectoId(this.getProximoObjetoId());
			}

			if (objPersistencia.getEstadoPersistencia() == ObjPersistencia.CREADO) {
				this.eliminarObjetos.add(objPersistencia); /*
															 * No elimina de la
															 * base de datos,
															 * sino simplemente
															 * de la lista de
															 * objetos
															 */
				exito = false;
			} else {
				if (objPersistencia.eliminar()) {
					this.eliminarObjetos.add(objPersistencia);
					exito = true;
				}
			}
			if (!despuesQuitar(eliminarIdx)) {
				return false;
			}
		}
		return exito;
	}

	// Sobreescritura necesaria
	public boolean antesEliminar(ObjPersistencia objPersistencia) {
		return true;
	}

	/**
	 * Este no es un ndice basado en cero: idx del primer objeto es 1, el
	 * segundo es 2, etc...
	 */
	public boolean despuesQuitar(int idx) {
		return true;
	}

	public boolean almacenar(ObjPersistencia objPersistencia) {
		boolean exito = false;
		int idx = -1;
		if (objPersistencia.getFabricaPersistenciaId() == fabricaPersistenteId) { // ObjPersistencia
																					// podra
																					// ser
																					// de
																					// otro
																					// FabricaObjPersistencia
																					// o
																					// creado
																					// manualmente
																					// ...
			idx = buscaObjetoIdPorIndice(objPersistencia.getObjectoId());
			if (!antesAlmacenar(objPersistencia)) {
				return false;
			}
			if (idx > 0) {
				if (objPersistencia.almacenar()) {
					this.listaObjetos.set(idx - 1, objPersistencia);
					exito = true;
				}
			} else {
				if (isAccesoSincronizado()) {
					objPersistencia.setObjectoId(this
							.getSincronizarProximoObjetoId());
				} else {
					objPersistencia.setObjectoId(this.getProximoObjetoId());
				}
				if (objPersistencia.almacenar()) {
					this.listaObjetos.add(objPersistencia);
					idx = this.listaObjetos.size() - 1;
					exito = true;
				} else {
					exito = false;
				}
			}
			if (!despuesAlmacenar(idx)) {
				return false;
			}
		} else {
			objPersistencia.setFabricaPersitenciaId(fabricaPersistenteId);
			if (isAccesoSincronizado()) {
				objPersistencia.setObjectoId(this
						.getSincronizarProximoObjetoId());
			} else {
				objPersistencia.setObjectoId(this.getProximoObjetoId());
			}
			if (!antesAlmacenar(objPersistencia)) {
				return false;
			}
			if (objPersistencia.almacenar()) {
				this.listaObjetos.add(objPersistencia);
				idx = this.listaObjetos.size() - 1;
				exito = true;
			} else {
				exito = false;
			}
			if (!despuesAlmacenar(idx)) {
				return false;
			}
		}
		return exito;
	}

	// Sobreescritura necesaria
	public boolean antesAlmacenar(ObjPersistencia o) {
		return true;
	}

	/**
	 * Este no es un ndice basado en cero: idx del primer objeto es 1, el
	 * segundo es 2, etc...
	 */
	public boolean despuesAlmacenar(int idx) {
		return true;
	}

	public boolean crear(ObjPersistencia objPersistencia) {
		if (objPersistencia.getFabricaPersistenciaId() != fabricaPersistenteId) {
			objPersistencia.setFabricaPersitenciaId(fabricaPersistenteId);
		}
		int idx = -1;
		boolean exito = false;
		if (!antesCrear(objPersistencia)) {
			return false;
		}
		if (objPersistencia.crear()) {
			idx = this.setObjeto(objPersistencia);
			exito = true;
		}
		if (!despuesCrear(idx)) {
			return false;
		}
		return exito;
	}

	// Sobreescritura necesaria
	public boolean antesCrear(ObjPersistencia objPersistencia) {
		return true;
	}

	/**
	 * Este no es un ndice basado en cero: idx del primer objeto es 1, el
	 * segundo es 2, etc...
	 */
	public boolean despuesCrear(int idx) {
		return true;
	}

	public List getEliminarObjetos() {
		return eliminarObjetos;
	}

	public void setObjetosEliminados(List objetosEliminados) {
		this.eliminarObjetos = objetosEliminados;
	}

	public char getSecuenciaModificacionDatos(int i) {
		return this.secuenciaModificacionDatos[i];
	}

	public char[] getSecuenciaModificacionDatos() {
		return this.secuenciaModificacionDatos;
	}

	public void setSecuenciaModificacionDatos(char[] secuenciaModificacionDatos) {
		this.secuenciaModificacionDatos = secuenciaModificacionDatos;
	}

	public boolean isDescartarObjetosNoModificados() {
		return descartarObjetosNoModificados;
	}

	public void setDescartarObjetosNoModificados(
			boolean descartarObjetosNoModificados) {
		ListIterator li = this.listaObjetos.listIterator();
		while (li.hasNext()) {
			ObjPersistencia objPersistencia = (ObjPersistencia) li.next();
			if (objPersistencia.getEstadoPersistencia() == objPersistencia.ORIGINAL) {
				li.remove();
			}
		}
		this.descartarObjetosNoModificados = descartarObjetosNoModificados;
	}

	public char getBloqueoOptimista() {
		return bloqueoOptimista;
	}

	public void setBloqueoOptimista(char bloqueoOptimista) {
		this.bloqueoOptimista = bloqueoOptimista;
	}

	public boolean isSoloLectura() {
		return soloLectura;
	}

	public void setSoloLectura(boolean soloLectura) {
		this.soloLectura = soloLectura;
	}

	public void setOriginal() {
		if (isSoloLectura()) {
			throw new IllegalStateException(
					"No puedo usar setOriginal() para objetos de solo lectura");
		}
		listaObjetos.add(this.eliminarObjetos);
		ListIterator li = listaObjetos.listIterator();
		while (li.hasNext()) {
			ObjPersistencia objPersistencia = (ObjPersistencia) li.next();
			if (objPersistencia.getEstadoPersistencia() == ObjPersistencia.ELIMINADO
					&& objPersistencia.getValoresOriginales() != null
					&& objPersistencia.getValoresOriginales()
							.getContadorCampos() > 0) {
				objPersistencia.setEstadoPersistencia(ObjPersistencia.ORIGINAL);
			} else if (objPersistencia.getEstadoPersistencia() == ObjPersistencia.CREADO
					&& objPersistencia.getValoresOriginales() == null
					|| objPersistencia.getValoresOriginales()
							.getContadorCampos() < 1) {
				li.remove();
			} else if (objPersistencia.getEstadoPersistencia() == ObjPersistencia.ALMACENADO
					&& objPersistencia.getValoresOriginales() != null
					&& objPersistencia.getValoresOriginales()
							.getContadorCampos() > 0) {
				objPersistencia.setEstadoPersistencia(ObjPersistencia.ORIGINAL);
			} else {
				throw new IllegalArgumentException("Estado invlido de SQL");
			}
		}
		eliminarObjetos.clear();
	}

	public java.lang.String getEsquema() {
		return esquema;
	}

	public void setEsquema(java.lang.String esquema) {
		this.esquema = esquema;
	}

	public java.lang.String getTabla() {
		return tabla;
	}

	public void setTabla(java.lang.String tabla) {
		this.tabla = tabla;
	}

	public boolean escribirDatos() {
		if (!isValido()) {
			return false;
		}
		if (!antesGrabarDatos()) {
			return false;
		}
		if (this.grabarDatos == null) {
			this.grabarDatos = new DBGrabarDatos();
		}
		this.grabarDatos
				.setExcepcionListaObjetosVacia(excepcionListaObjetosVacia);
		boolean exito = this.grabarDatos.grabarDatos(this);
		if (!despuesGrabarDatos()) {
			return false;
		}
		return exito;
	}

	// Sobreescritura necesaria
	public boolean antesGrabarDatos() {
		return true;
	}

	// Sobreescritura necesaria
	public boolean despuesGrabarDatos() {
		return true;
	}

	public Metadatos getMetaDatos() {
		if (crearDefectoMetadato && this.metadatos == null) {
			poblarMetaDatos();
		}
		return this.metadatos;
	}

	public void poblarMetaDatos() {
		try {
			this.metadatos = new Metadatos();
			this.metadatos
					.setLanzaCampoInexistenteExcepcion(lanzaCampoInexistenteExcepcion);
			this.metadatos.poblar(getMapping(), this.solicitante);
			Collection tablas = (this.metadatos.getTablas()).values();
			if (debug)
				System.out.println("slgfp ============= "
						+ new java.util.Date() + " " + this.getClass() + "::"
						+ this + "::metadatos.getTablas(): "
						+ this.metadatos.getTablas());
			if (!this.isSoloLectura()) { // No es necesario PK si solo es de
											// lectura
				/*
				 * Todos los nombre de tablas distintos para el solicitante
				 * (ObjPersistencia)
				 */
				Iterator i = tablas.iterator();
				while (i.hasNext()) {
					String[] tablaId = (String[]) i.next();
					if (tablaId[0] == null && reemplazaNombreCatalogo) {
						tablaId[0] = this.catalogo;
					}
					List pk = getClavePrimaria(tablaId[0], tablaId[1],
							tablaId[2]);
					if (debug)
						System.out.println("slgfp ============= "
								+ new java.util.Date() + " " + this.getClass()
								+ "::" + this + "::pk: " + pk);
				}
			}
			if (tablas.size() == 1) {
				// Slo hay una tabla para este objeto, de lo contrario, se debe
				// fijar el catlogo y/o un esquema y tabla para
				// actualizar/borrar/insertar cada tabla
				if (this.metadatos.getNombreCatalogo(1) == null
						&& !reemplazaNombreCatalogo) {
					setCatalogo(this.catalogo);
				}
				setEsquema(this.metadatos.getNombreEsquema(1));
				setTabla(this.metadatos.getNombreTabla(1));
				if (debug)
					System.out
							.println("slgfp ============= "
									+ new java.util.Date() + " "
									+ this.getClass() + "::" + this
									+ "::this.catalogo: " + this.catalogo);
				if (debug)
					System.out.println("slgfp ============= "
							+ new java.util.Date() + " " + this.getClass()
							+ "::" + this + "::esquema: " + getEsquema());
				if (debug)
					System.out.println("slgfp ============= "
							+ new java.util.Date() + " " + this.getClass()
							+ "::" + this + "::tablas: " + getTabla());
			}
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		}
	}

	public void setMetaData(Metadatos metadatos) {
		this.metadatos = metadatos;
	}

	public DBServicios getDBServicios() {
		if (this.dbServicios == null) {
			this.dbServicios = new DBServicios();
		}
		return this.dbServicios;
	}

	public void setDBServicios(DBServicios dbServicios) {
		this.dbServicios = dbServicios;
	}

	public boolean isLanzarBloqueoOptimistaDeleteExcepcion() {
		return lanzarBloqueoOptimistaDeleteExcepcion;
	}

	public void setLanzarBloqueoOptimistaDeleteExcepcion(
			boolean lanzarBloqueoOptimistaDeleteExcepcion) {
		this.lanzarBloqueoOptimistaDeleteExcepcion = lanzarBloqueoOptimistaDeleteExcepcion;
	}

	public boolean isLanzarBloqueoOptimistaUpdateExcepcion() {
		return lanzarBloqueoOptimistaUpdateExcepcion;
	}

	public void setLanzarBloqueoOptimistaUpdateExcepcion(
			boolean lanzarBloqueoOptimistaUpdateExcepcion) {
		this.lanzarBloqueoOptimistaUpdateExcepcion = lanzarBloqueoOptimistaUpdateExcepcion;
	}

	public void setLanzarFalloInsercionExcepcion(
			boolean lanzarFalloInsertExcepcion) {
		this.lanzarFalloInsertExcepcion = lanzarFalloInsertExcepcion;
	}

	public boolean isLanzarFalloInsertExcepcion() {
		return this.lanzarFalloInsertExcepcion;
	}

	/*
	 * Llama a este mtodo despus de sincronizacin con xito completa
	 * COMMITpara sincronizar valores originales y actuales: copia los valores
	 * actuales al original y fija estadoPersistencia ORIGINAL y descarta
	 * objetos suprimidos de la base de datos
	 */
	public void sincronizandoEstadoDePersistencia() {
		if (isSoloLectura()) {
			throw new IllegalStateException(
					"No es posible llamar a sincronizacionEstadoPersistente al tratarse de objetos de solo lectura");
		}
		try {
			eliminarObjetos.clear();
			ListIterator li = listaObjetos.listIterator();
			while (li.hasNext()) {
				boolean copiarValores = true;
				boolean creado = false;
				ObjPersistencia objPersistencia = (ObjPersistencia) li.next();
				if (debug)
					System.out.println("slgfp ============= "
							+ new java.util.Date() + " " + this.getClass()
							+ "::" + this
							+ "::syncSqlEstado()::ObjPersistencia: "
							+ objPersistencia + " \n");
				if (objPersistencia.getEstadoPersistencia() == ObjPersistencia.ELIMINADO
						&& objPersistencia.getValoresOriginales() != null
						&& objPersistencia.getValoresOriginales()
								.getContadorCampos() > 0) {
					li.remove();
					copiarValores = false;
				} else if (objPersistencia.getEstadoPersistencia() == ObjPersistencia.CREADO
						&& (objPersistencia.getValoresOriginales() == null || objPersistencia
								.getValoresOriginales().getContadorCampos() < 1)) {
					objPersistencia
							.setEstadoPersistencia(ObjPersistencia.ORIGINAL);
					creado = true;
				} else if (objPersistencia.getEstadoPersistencia() == ObjPersistencia.ALMACENADO
						&& objPersistencia.getValoresOriginales() != null
						&& objPersistencia.getValoresOriginales()
								.getContadorCampos() > 0) {
					objPersistencia
							.setEstadoPersistencia(ObjPersistencia.ORIGINAL);
				} else if (objPersistencia.getEstadoPersistencia() == ObjPersistencia.ORIGINAL) {
					// No hace nada
					continue;
				} else {
					throw new IllegalArgumentException(
							"No valido el EstadoPersistencia");
				}
				if (copiarValores) { // Sincroniza los valores en los objetos
					ContenedorDatos originalValues = null;
					if (creado) {
						originalValues = new ContenedorDatos(
								asignaciones.size());
					} else {
						originalValues = objPersistencia.getValoresOriginales();
					}
					Iterator i = asignaciones.values().iterator();
					int indiceCampo = 0;
					if (debug)
						System.out.println("slgfp ============= "
								+ new java.util.Date() + " " + this.getClass()
								+ "::" + this + "::syncSqlEstado()::creado: "
								+ creado + " \n");
					while (i.hasNext()) {
						indiceCampo++;
						String nombreCampo = (String) i.next();
						if (creado) {
							Method metodoLectura = this.metadatos
									.getMetodoLectura(this.metadatos
											.getIndiceColumnaPorNombreCampo(nombreCampo));
							Object value = metodoLectura.invoke(
									objPersistencia, VACIO_LECTURA_ARG);
							originalValues.setNombreCampoYValor(indiceCampo,
									nombreCampo, value);
						} else {
							if (originalValues.buscaIndiceCampo(nombreCampo) != -1) {
								Method metodoLectura = this.metadatos
										.getMetodoLectura(this.metadatos
												.getIndiceColumnaPorNombreCampo(nombreCampo));
								Object value = metodoLectura.invoke(
										objPersistencia, VACIO_LECTURA_ARG);
								objPersistencia.setValoresOriginales(
										nombreCampo, value);
							}
						}
					}
					if (creado) {
						objPersistencia.setValoresOriginales(originalValues);
						creado = false;
					}
					objPersistencia.setModificado(false);
				}
			}
			if (this.grabarDatos != null) {
				this.grabarDatos.resetear();
			}
			valido = true;
		} catch (Throwable t) {
			if (t instanceof InvocationTargetException) {
				throw new RuntimeException(
						((InvocationTargetException) t).getTargetException());
			} else {
				throw new RuntimeException(t);
			}
		}
	}

	public int getNumeroEliminados() {
		if (grabarDatos == null) {
			throw new NullPointerException("GrabarDatos no inicializado");
		}
		return grabarDatos.getNumeroEliminados();
	}

	public int getContadorCreado() {
		if (grabarDatos == null) {
			throw new NullPointerException("GrabarDatos no inicializado");
		}
		return grabarDatos.getNumeroCreados();
	}

	public int getNumeroDeModificados() {
		if (grabarDatos == null) {
			throw new NullPointerException("GrabarDatos no inicializado");
		}
		return grabarDatos.getNumeroDeModificados();
	}

	public GrabarDatos getGrabarDatos() {
		if (grabarDatos == null) {
			grabarDatos = new DBGrabarDatos();
		}
		return grabarDatos;
	}

	public void setGrabarDatos(GrabarDatos grabarDatos) {
		this.grabarDatos = grabarDatos;
	}

	public boolean isAccesoSincronizado() {
		return accesoSincronizado;
	}

	public void setAccesoSincronizado(boolean accesoSincronizado) {
		if (accesoSincronizado) {
			this.eliminarObjetos = Collections
					.synchronizedList(eliminarObjetos);
			this.listaObjetos = Collections.synchronizedList(listaObjetos);
			this.accesoSincronizado = accesoSincronizado;
		}
	}

	public boolean isCargaDiferida() {
		return cargaDiferida;
	}

	public void setCargaDiferida(boolean cargaDiferida) {
		this.cargaDiferida = cargaDiferida;
	}

	public int getTamaoPagina() {
		return tamaoPagina;
	}

	public void setTamaoPagina(int tamaoPagina) {
		if (tamaoPagina < 0) {
			throw new IllegalArgumentException("Tamao de pagina no valido: "
					+ tamaoPagina);
		}
		this.tamaoPagina = tamaoPagina;
	}

	/**
	 * Este no es un ndice basado en cero: idx del primer objeto es 1, el
	 * segundo es 2, etc...
	 */
	public MarcadorPagina getPagina(int numeroPagina) {
		MarcadorPagina marcadorPagina = new MarcadorPagina();
		marcadorPagina.setObjectosTotal(this.listaObjetos.size());
		if (getTamaoPagina() == 0) {
			marcadorPagina.setNumeroPagina(1);
			marcadorPagina.setPaginasTotal(1);
			marcadorPagina.setPagina(this.listaObjetos);
		} else {
			List page = new ArrayList();
			if (this.listaObjetos.isEmpty()) {
				marcadorPagina.setNumeroPagina(0);
				marcadorPagina.setPaginasTotal(0);
				marcadorPagina.setPagina(page);
			} else {
				double d1 = new Integer(this.listaObjetos.size()).doubleValue();
				double d2 = new Integer(this.tamaoPagina).doubleValue();
				double d3 = java.lang.Math.ceil(d1 / d2);
				int paginasTotal = (new Double(d3)).intValue();
				marcadorPagina.setPaginasTotal(paginasTotal);
				if (numeroPagina > paginasTotal) {
					numeroPagina = paginasTotal;
				} else if (numeroPagina < 1) {
					numeroPagina = 1;
				} else {
					//
				}
				marcadorPagina.setNumeroPagina(numeroPagina);
				ListIterator li = this.listaObjetos
						.listIterator((numeroPagina - 1) * getTamaoPagina());
				int count = 0;
				while (li.hasNext() && count < getTamaoPagina()) {
					Object obj = li.next();
					page.add(obj);
					count++;
				}
				marcadorPagina.setPagina(page);
			}
		}
		return marcadorPagina;
	}

	public boolean isSoloGenerarSQL() {
		return generarSQLSolo;
	}

	public void setGenerarSoloSQL(boolean generarSQLSolo) {
		this.generarSQLSolo = generarSQLSolo;
	}

	public java.util.List getGeneratedSQL() {
		if (grabarDatos == null) {
			throw new NullPointerException("GrabarDatos no inicializado");
		}
		return this.grabarDatos.getGeneradosSQL();
	}

	public java.lang.String getGeneratedSQLasXML() {
		if (grabarDatos == null) {
			throw new NullPointerException("GrabarDatos no inicializado");
		}
		return this.grabarDatos.getGeneradosSQLComoXML();
	}

	public boolean isAadir() {
		return aadir;
	}

	public void setAadir(boolean aadir) {
		this.aadir = aadir;
	}

	public java.lang.Throwable getQueEstaMal() {
		if (this.grabarDatos == null) {
			return null;
		}
		return this.grabarDatos.getQueEstaMal();
	}

	public java.lang.String getCatalogo() {
		return catalogo;
	}

	public void setCatalogo(java.lang.String catalogo) {
		this.catalogo = catalogo;
	}

	public boolean isNoNombreColumnaCompleto() {
		return noNombresColumnasCompletos;
	}

	public void setNoNombreDeColumnaCompleta(boolean noNombresColumnasCompletos) {
		this.noNombresColumnasCompletos = noNombresColumnasCompletos;
	}

	public boolean isDebug() {
		return debug;
	}

	public void limpiarDatos() {
		if (this.listaObjetos != null)
			this.listaObjetos.clear();
		if (this.eliminarObjetos != null)
			this.eliminarObjetos.clear();
		valido = true;
	}

	public void setDebug(boolean debug) {
		this.debug = debug;
	}

	public boolean isReemplazaNombreCatalogo() {
		return reemplazaNombreCatalogo;
	}

	public void setReemplazaNombreCatalogo(boolean reemplazaNombreCatalogo) {
		this.reemplazaNombreCatalogo = reemplazaNombreCatalogo;
	}

	public void setManejadorExcepciones(
			ManejadorExcepciones manejadorExcepciones) {
		this.manejadorExcepciones = manejadorExcepciones;
		manejadorExcepciones.setFabricaObjPersistente(this);
	}

	public ManejadorExcepciones getManejadorExcepciones() {
		return this.manejadorExcepciones;
	}

	// Sobreescritura necesarioa
	public boolean isValido() {
		return valido;
	}

	// Crea un ContenedorDatos vacio desde Metadatos
	public ContenedorDatos crearValoresOriginales() throws SQLException {
		ContenedorDatos contenedorDatos = new ContenedorDatos(
				this.metadatos.getNumeroColumnas());
		for (int i = 1; i <= this.metadatos.getNumeroColumnas(); i++) {
			contenedorDatos.setNombreCampo(i, this.metadatos.getNombreCampo(i));
		}
		return contenedorDatos;
	}

	public void setOrigenDatos(DataSource ds) {
		getDBServicios().setDataSource(ds);
	}

	public DataSource getDataSource() throws SQLException {
		return getDBServicios().getDataSource();
	}

	public void setDataSourceName(String dataSourceNombre) {
		getDBServicios().setNombreDataSource(dataSourceNombre);
	}

	public boolean isExcepcionListaObjetosVacia() {
		return excepcionListaObjetosVacia;
	}

	public void setExcepcionSiListaObjetosVacia(
			boolean excepcionListaObjetosVacia) {
		this.excepcionListaObjetosVacia = excepcionListaObjetosVacia;
	}

	public boolean isCrearDefectoMetadato() {
		return crearDefectoMetadato;
	}

	public void setCreaMetadatosDefecto(boolean crearDefectoMetadato) {
		this.crearDefectoMetadato = crearDefectoMetadato;
	}

	public boolean isLanzaCampoInexistenteExcepcion() {
		return lanzaCampoInexistenteExcepcion;
	}

	public void setLanzarExcepcionCampoInexistente(
			boolean lanzaCampoInexistenteExcepcion) {
		this.lanzaCampoInexistenteExcepcion = lanzaCampoInexistenteExcepcion;
	}

	public Transformador getTransformador() {
		return transformador;
	}

	public void setTransformador(Transformador transformador) {
		this.transformador = transformador;
	}

	public String toString() {
		StringBuffer sb = new StringBuffer();
		Object value = null;
		Method[] metodos = getClass().getMethods();
		for (int i = 0; i < metodos.length; i++) {
			String nombreMetodo = metodos[i].getName();
			if (nombreMetodo.equals("getMetaDatos")
					|| nombreMetodo.equals("getClass")) {
				continue;
			}
			if (nombreMetodo.equals("getSecuenciaModificacionDatos")
					&& metodos[i].getParameterTypes().length == 0) {
				if (secuenciaModificacionDatos != null
						&& secuenciaModificacionDatos.length > 0) {
					sb.append("dataModificationSequence=")
							.append(new String(secuenciaModificacionDatos))
							.append("&");
				} else {
					sb.append("secuenciaModificacionDatos=").append("&");
				}

			} else if ((nombreMetodo.startsWith("get") || nombreMetodo
					.startsWith("is"))
					&& metodos[i].getParameterTypes().length == 0) {
				try {
					value = metodos[i].invoke(this, VACIO_LECTURA_ARG);
				} catch (Throwable t) {
					continue;
				}
				String c = "";
				if (nombreMetodo.startsWith("is")) {
					c = nombreMetodo.substring(2, 3).toLowerCase();
					sb.append(c);
					sb.append(nombreMetodo.substring(3));
				} else if (nombreMetodo.startsWith("get")) {
					c = nombreMetodo.substring(3, 4).toLowerCase();
					sb.append(c);
					sb.append(nombreMetodo.substring(4));
				}
				sb.append("=");
				sb.append((value == null) ? "" : value);
				sb.append("&");
			}
		}
		return sb.toString();
	}
}
