package slgfp;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.math.BigInteger;
import java.math.BigDecimal;
import java.util.Collections;
import java.util.Set;
import java.util.HashSet;

/**
 * La clase nos sirve para el control de las secuencias de la base de datos que
 * sern usadas como claves de los registros/objetos
 * 
 * @version 1.0, 03-05-2011
 * @author Sergio Lovillo
 */

@SuppressWarnings({ "rawtypes", "unchecked", "null" })
public class GeneradorClaves {

	protected String tabla; // Obligatorio
	protected String idColumna; // Obligatorio
	protected String sequenciaNombreColumna; // Obligatorio para secuencias no
												// externas
	protected int incremento = 10; // El incremental de la secuencia desde 0
	// No use la misma conexion que para el resto de transacciones puesto que
	// esta operacion genera un commit siempre
	protected DBServicios dbServicios;
	protected int reintentosMax = 3; // Cualquier numero mayor que 0
	protected long timeout = 250; // Cualquier numero mayor que 0
	protected String sqlEstado = "23000"; // Duplica clave
	protected int vErrorCodigo = Integer.MIN_VALUE;
	protected int nivelAislamiento = -1;
	protected boolean fijarNivelAislamiento = false;
	protected Map valorBajo = Collections.synchronizedMap(new HashMap());
	protected Map valorAlto = Collections.synchronizedMap(new HashMap());
	protected Class claseClave = Integer.class; // Clase de la que extiende
												// java.lang.Number
	protected boolean secuenciaExterna = false; // Usado para secuencias reales:
												// Oracle, PostgreSQL, etc...
	protected String select;
	protected String update;
	protected String insert; // Si la secuencia no existe la crea
	protected boolean crearSecuenciaBajoDemanda = true;
	protected Number valorInicial = new Integer(0); // No se usa con secuancias
													// reales
	protected Set secuencias;
	protected boolean cacheSentencias;

	public GeneradorClaves() {
	}

	public GeneradorClaves(int nivelAislamiento) {
		this.nivelAislamiento = nivelAislamiento;
	}

	public GeneradorClaves(String tabla, String sequenciaNombreColumna,
			String idColumna, int nivelAislamiento) {
		this.tabla = tabla;
		this.sequenciaNombreColumna = sequenciaNombreColumna;
		this.idColumna = idColumna;
		this.nivelAislamiento = nivelAislamiento;
	}

	public GeneradorClaves(String tabla, String sequenciaNombreColumna,
			String idColumna) {
		this.tabla = tabla;
		this.sequenciaNombreColumna = sequenciaNombreColumna;
		this.idColumna = idColumna;
	}

	public GeneradorClaves(String tabla, String sequenciaNombreColumna,
			String idColumna, int nivelAislamiento, boolean secuenciaExterna) {
		this.tabla = tabla;
		this.sequenciaNombreColumna = sequenciaNombreColumna;
		this.idColumna = idColumna;
		this.nivelAislamiento = nivelAislamiento;
		this.secuenciaExterna = secuenciaExterna;
	}

	public GeneradorClaves(String tabla, String sequenciaNombreColumna,
			String idColumna, boolean secuenciaExterna) {
		this.tabla = tabla;
		this.sequenciaNombreColumna = sequenciaNombreColumna;
		this.idColumna = idColumna;
		this.secuenciaExterna = secuenciaExterna;
	}

	public void iniciar() {
		if (!secuenciaExterna && this.tabla == null) {
			throw new IllegalStateException("El nombre de la tabla no existe");
		}
		if (!secuenciaExterna && this.sequenciaNombreColumna == null) {
			throw new IllegalStateException(
					"La sequenciaNombreColumna de la secuencia no existe");
		}
		if (this.idColumna == null) {
			throw new IllegalStateException("idColumna no existe");
		}
		if (dbServicios == null) {
			throw new IllegalStateException("DBServicios no definido");
		}
		if (!secuenciaExterna) {
			// SELECT ID FROM SECUENCIAS WHERE SECUENCIA_NOMBRE = ? ${secuencia}
			select = "SELECT " + idColumna + " FROM " + tabla + " WHERE "
					+ sequenciaNombreColumna + " = ?";
			// UPDATE SECUENCIAS SET ID = ID + 10 WHERE SECUENCIA_NOMBRE = ? AND
			// ID
			// = ? ${actual ID}
			update = "UPDATE " + tabla + " SET " + idColumna + " = "
					+ idColumna + " + " + String.valueOf(incremento)
					+ " WHERE " + sequenciaNombreColumna + " = ? AND "
					+ idColumna + " = ?";
			if (crearSecuenciaBajoDemanda) {
				// INSERT INTO SECUENCIAS (SECUENCIA_NOMBRE, ID) VALUES (?, 0)
				// ${secuencia}
				insert = "INSERT INTO " + tabla + " (" + sequenciaNombreColumna
						+ ", " + idColumna + ") VALUES (?, "
						+ valorInicial.toString() + ")";
			}
			String todasSecuencias = "SELECT " + sequenciaNombreColumna
					+ " FROM " + tabla;
			try {
				secuencias = Collections
						.synchronizedSet(new HashSet(
								dbServicios
										.getResultadoUnicoColumnaComoLista(todasSecuencias)));
			} catch (SQLException sqle) {
				throw new RuntimeException(sqle);
			}
		}
	}

	public synchronized Number getProximoValor(String s) throws SQLException {
		if (!secuenciaExterna && this.select == null) {
			throw new IllegalStateException("GeneradorClaves not initialized");
		}
		if (!secuenciaExterna && !secuencias.contains(s)
				&& crearSecuenciaBajoDemanda) { // new sequence
			Collection param = new ArrayList(1);
			param.add(s);
			dbServicios.ejecutar(insert, param);
			secuencias.add(s);
		}
		Object valor = valorBajo.get(sequenciaNombreColumna);
		Number altoValor = null;
		if (valor == null) {
			valor = getSemilla(s);
			valorBajo.put(sequenciaNombreColumna, valor);
			if (claseClave.getName().equals("java.lang.Integer")) {
				altoValor = ((Number) valor).intValue() + incremento;
			} else if (claseClave.getName().equals("java.lang.Long")) {
				altoValor = ((Number) valor).longValue() + incremento;
			} else if (claseClave.getName().equals("java.math.BigInteger")) {
				altoValor = new BigInteger(valor.toString())
						.add(new BigInteger(String.valueOf(incremento)));
			} else if (claseClave.getName().equals("java.math.BigDecimal")) {
				altoValor = new BigDecimal(valor.toString())
						.add(new BigDecimal(String.valueOf(incremento)));
			} else {
				try {
					altoValor = new BigDecimal(valor.toString())
							.add(new BigDecimal(String.valueOf(incremento)));
					Constructor c = claseClave.getConstructor(String.class);
					altoValor = (Number) c.newInstance(altoValor.toString());
				} catch (InvocationTargetException ite) {
					throw new SQLException(ite.getTargetException().toString());
				} catch (Exception e) {
					throw new SQLException(e.toString());
				}
			}
			valorAlto.put(sequenciaNombreColumna, altoValor);
		} else {
			if (claseClave.getName().equals("java.lang.Integer")) {
				if (((Number) valor).intValue() < ((Number) valorAlto
						.get(sequenciaNombreColumna)).intValue() - 1) {
					valor = new Integer(((Number) valor).intValue() + 1);
					valorBajo.put(sequenciaNombreColumna, valor);
				} else {
					valor = getSemilla(s);
					valorBajo.put(sequenciaNombreColumna, valor);
					valorAlto.put(sequenciaNombreColumna,
							((Number) valor).intValue() + incremento);
				}
			} else if (claseClave.getName().equals("java.lang.Long")) {
				if (((Number) valor).longValue() < ((Number) valorAlto
						.get(sequenciaNombreColumna)).longValue() - 1) {
					valor = new Long(((Number) valor).longValue() + 1);
					valorBajo.put(sequenciaNombreColumna, valor);
				} else {
					valor = getSemilla(s);
					valorBajo.put(sequenciaNombreColumna, valor);
					valorAlto.put(sequenciaNombreColumna,
							((Number) valor).longValue() + incremento);
				}
			} else if (claseClave.getName().equals("java.math.BigInteger")) {
				if (new BigInteger(valor.toString()).compareTo(new BigInteger(
						valorAlto.get(sequenciaNombreColumna).toString())) == -1) {
					valor = new BigInteger(valor.toString())
							.add(BigInteger.ONE);
					valorBajo.put(sequenciaNombreColumna, valor);
				} else {
					valor = getSemilla(s);
					valorBajo.put(sequenciaNombreColumna, valor);
					valorAlto.put(sequenciaNombreColumna,
							new BigInteger(valor.toString())
									.add(new BigInteger(String
											.valueOf(incremento))));
				}
			} else if (claseClave.getName().equals("java.math.BigDecimal")) {
				if (new BigDecimal(valor.toString()).compareTo(new BigDecimal(
						valorAlto.get(sequenciaNombreColumna).toString())) == -1) {
					valor = new BigDecimal(valor.toString())
							.add(BigDecimal.ONE);
					valorBajo.put(sequenciaNombreColumna, valor);
				} else {
					valor = getSemilla(s);
					valorBajo.put(sequenciaNombreColumna, valor);
					valorAlto.put(sequenciaNombreColumna,
							new BigDecimal(valor.toString())
									.add(new BigDecimal(String
											.valueOf(incremento))));
				}
			} else {
				if (new BigDecimal(valor.toString()).compareTo(new BigDecimal(
						valorAlto.get(sequenciaNombreColumna).toString())) == -1) {
					valor = new BigDecimal(valor.toString())
							.add(BigDecimal.ONE);
					try {
						valor = new BigDecimal(valor.toString())
								.add(BigDecimal.ONE);
						Constructor c = claseClave.getConstructor(String.class);
						valorBajo.put(sequenciaNombreColumna,
								c.newInstance(valor.toString()));
					} catch (InvocationTargetException ite) {
						throw new SQLException(ite.getTargetException()
								.toString());
					} catch (Exception e) {
						throw new SQLException(e.toString());
					}
				} else {
					valor = getSemilla(s);
					valorBajo.put(sequenciaNombreColumna, valor);
					try {
						Constructor c = claseClave.getConstructor(String.class);
						BigDecimal temp = new BigDecimal(valor.toString())
								.add(new BigDecimal(String.valueOf(incremento)));
						valorAlto.put(sequenciaNombreColumna,
								c.newInstance(temp.toString()));
					} catch (InvocationTargetException ite) {
						throw new SQLException(ite.getTargetException()
								.toString());
					} catch (Exception e) {
						throw new SQLException(e.toString());
					}
				}
			}
		}
		return (Number) valor;
	}

	public synchronized Number getValorActual(String s) throws SQLException {
		if (!secuenciaExterna && this.select == null) {
			throw new IllegalStateException("GeneradorClaves no inicializado");
		}
		if (!secuenciaExterna && !secuencias.contains(s)
				&& crearSecuenciaBajoDemanda) { // Nueva secuencia
			Collection p = new ArrayList(1);
			p.add(s);
			dbServicios.ejecutar(insert, p);
			secuencias.add(s);
		}
		Object valorActual;
		try {
			if (secuenciaExterna) {
				if (tabla == null || tabla.trim().length() == 0) {
					valorActual = dbServicios.getValorUnico("SELECT "
							+ idColumna);
				} else {
					valorActual = dbServicios.getValorUnico("SELECT "
							+ idColumna + " FROM " + tabla);
				}
			} else {
				Collection p = new ArrayList(1);
				p.add(s);
				valorActual = dbServicios.getValorUnico(select, p);
			}
		} catch (SQLException sqle) {
			throw sqle;
		} finally {
			dbServicios.liberacion(false);
		}
		return (Number) valorActual;
	}

	protected synchronized Number getSemilla(String s) throws SQLException {
		if (!secuenciaExterna && this.select == null) {
			throw new IllegalStateException("GeneradorClaves no inicializado");
		}
		Object id = null;
		boolean exito = false;
		if (secuenciaExterna) {
			try {
				id = dbServicios.getValorUnico("SELECT " + idColumna + " FROM "
						+ tabla);
				exito = true;
			} finally {
				dbServicios.liberacion(false);
			}
		} else {
			try {
				int contadorReintentos = 0;
				if (nivelAislamiento != -1 && !fijarNivelAislamiento) { // Fija
																		// solo
																		// uno
					dbServicios.getConexion().setTransactionIsolation(
							nivelAislamiento);
					fijarNivelAislamiento = true;
				}
				dbServicios.comenzandoTransaccion();
				while (!exito && contadorReintentos <= reintentosMax) {
					try {
						Collection p = new ArrayList(1);
						p.add(s);
						System.out.println("========== param: " + p);
						id = dbServicios.getValorUnico(select, p);
						p.add(id);
						dbServicios.ejecutar(update, p);
						exito = true;
						if (claseClave.getName().equals("java.lang.Integer")) {
							id = new Integer(((Number) id).intValue()
									+ incremento);
						} else if (claseClave.getName()
								.equals("java.lang.Long")) {
							id = new Long(((Number) id).intValue() + incremento);
						} else if (claseClave.getName().equals(
								"java.math.BigInteger")) {
							id = new BigInteger(id.toString())
									.add(new BigInteger(String
											.valueOf(incremento)));
						} else if (claseClave.getName().equals(
								"java.math.BigDecimal")) {
							id = new BigDecimal(id.toString())
									.add(new BigDecimal(String
											.valueOf(incremento)));
						} else {
							Constructor c = claseClave
									.getConstructor(String.class);
							id = c.newInstance(String.valueOf(incremento));
						}
						break;
					} catch (InstantiationException ie) {
						throw new SQLException(ie.toString());
					} catch (IllegalAccessException iae) {
						throw new SQLException(iae.toString());
					} catch (InvocationTargetException ite) {
						throw new SQLException(ite.getTargetException()
								.toString());
					} catch (SQLException sqle) {
						if ((sqle.getSQLState() != null && sqle.getSQLState()
								.equals(sqlEstado))
								|| sqle.getErrorCode() == vErrorCodigo) {
							// Clave duplicada, reintentar
						} else {
							throw sqle; // Se ha encontrado algn problema
						}
					}
					contadorReintentos++;
					try {
						wait(timeout);
					} catch (InterruptedException ie) {
						throw new SQLException("No puedo conseguir un nico ID");
					}
				}
			} catch (NoSuchMethodException nsme) {
				throw new SQLException(nsme.getMessage()); // No debe ocurrir
			} catch (SQLException sqle) {
				throw sqle;
			} finally {
				dbServicios.validandoTransaccion(); // Damos los cambios por
													// buenos
				dbServicios.liberacion(false);
			}
		}
		if (!exito) {
			throw new SQLException("No puedo conseguir un nico ID");
		}
		return (Number) id;
	}

	public String getTabla() {
		return tabla;
	}

	public void setTabla(String tabla) {
		this.tabla = tabla;
	}

	public String getIdColumna() {
		return idColumna;
	}

	public void setIdColumna(String idColumna) {
		this.idColumna = idColumna;
	}

	public int getIncremento() {
		return incremento;
	}

	public void setIncremento(int incremento) {
		this.incremento = incremento;
	}

	public DBServicios getDBServicios() {
		return dbServicios;
	}

	public void setDBServicios(DBServicios dbServicios) {
		this.dbServicios = dbServicios;
		this.dbServicios.setCacheSentencias(cacheSentencias);
	}

	public int getReintentosMax() {
		return reintentosMax;
	}

	public void setReintentosMax(int reintentosMax) {
		this.reintentosMax = reintentosMax;
	}

	public String getSqlEstado() {
		return sqlEstado;
	}

	public void setSqlEstado(String sqlEstado) {
		this.sqlEstado = sqlEstado;
	}

	public long getTimeout() {
		return timeout;
	}

	public void setTimeout(long timeout) {
		if (timeout < 0) {
			throw new IllegalArgumentException(
					"El valor de timeout debe ser mayor de 0");
		}
		this.timeout = timeout;
	}

	public int getNivelAislamiento() {
		return nivelAislamiento;
	}

	public void setNivelAislamiento(int nivelAislamiento) {
		this.nivelAislamiento = nivelAislamiento;
	}

	public String getSequenciaNombreColumna() {
		return sequenciaNombreColumna;
	}

	public void setSequenciaNombreColumna(String sequenciaNombreColumna) {
		this.sequenciaNombreColumna = sequenciaNombreColumna;
	}

	public Class getClaseClave() {
		return claseClave;
	}

	public void setClaseClave(Class claseClave) {
		if (Number.class.isAssignableFrom(claseClave)) {
			this.claseClave = claseClave;
		} else {
			throw new IllegalArgumentException(
					"ClaseClave debe extenderse de java.lang.Number");
		}
		try {
			Constructor c = claseClave.getConstructor(String.class);
			Object obj = c.newInstance("0");
			String valorCad = obj.toString();
			if (valorCad == null
					&& !(valorCad.equals("0") && !(valorCad.startsWith("0.")))) {
				throw new IllegalArgumentException(claseClave.getName()
						+ ".toString() debe devolver un numero como cadena");
			}
		} catch (InvocationTargetException ite) {
			throw new IllegalArgumentException(ite.getTargetException()
					.toString());
		} catch (Exception e) {
			throw new IllegalArgumentException(e.toString());
		}
	}

	public boolean isSecuenciaExterna() {
		return secuenciaExterna;
	}

	public void setSecuenciaExterna(boolean secuenciaExterna) {
		this.secuenciaExterna = secuenciaExterna;
	}

	public Number getValorInicial() {
		return valorInicial;
	}

	public void setValorInicial(Number valorInicial) {
		this.valorInicial = valorInicial;
	}

	public boolean isCacheSentencias() {
		return cacheSentencias;
	}

	public void setCacheSentencias(boolean cacheSentencias) {
		this.cacheSentencias = cacheSentencias;
	}

	public boolean isCrearSecuenciaBajoDemanda() {
		return crearSecuenciaBajoDemanda;
	}

	public void setCrearSecuenciaBajoDemanda(boolean crearSecuenciaBajoDemanda) {
		this.crearSecuenciaBajoDemanda = crearSecuenciaBajoDemanda;
	}

	public void reiniciar() throws SQLException {
		tabla = null;
		idColumna = null;
		sequenciaNombreColumna = null;
		incremento = 10;
		reintentosMax = 3;
		timeout = 250;
		vErrorCodigo = Integer.MIN_VALUE;
		nivelAislamiento = -1;
		fijarNivelAislamiento = false;
		valorBajo.clear();
		valorAlto.clear();
		claseClave = Integer.class;
		secuenciaExterna = false;
		select = null;
		update = null;
		insert = null;
		valorInicial = new Integer(0);
		cacheSentencias = false;
		secuencias.clear();
		dbServicios.liberacion(true);
	}
}
