package slgfp;

import javax.sql.*;
import java.util.*;
import java.sql.SQLException;
import java.io.Serializable;
import java.lang.reflect.*;
import java.beans.*;

/**
 * La clase nos sirve para poblar y cachear columnas, tablas, campos, etc.
 * 
 * @version 1.0, 03-05-2011
 * @author Sergio Lovillo
 */

@SuppressWarnings({ "serial", "rawtypes", "unchecked", "unused" })
public class Metadatos implements Serializable, Cloneable {

	public Metadatos() {
	}

	public String toString() {
		StringBuffer sb = new StringBuffer(
				"slgfp ============= org.slgfp.MetaDatos: ");
		sb.append("\n contadorColumna: " + contadorColumna);
		sb.append("\n nombreCatalogo: " + Arrays.asList(nombreCatalogo));
		sb.append("\n nombreEsquema: " + Arrays.asList(nombreEsquema));
		sb.append("\n nombreTabla: " + Arrays.asList(nombreTabla));
		sb.append("\n etiquetaColumna: " + Arrays.asList(etiquetaColumna));
		sb.append("\n nombreColumna: " + Arrays.asList(nombreColumna));
		sb.append("\n nombreColumnaCompleto: "
				+ Arrays.asList(nombreColumnaCompleto));
		sb.append("\n nombreCampo: " + Arrays.asList(nombreCampo));
		sb.append("\n metodoLectura: " + Arrays.asList(metodoLectura));
		sb.append("\n escribirMetodos: " + Arrays.asList(escribirMetodos));
		sb.append("\n nombreClaseCampo: " + Arrays.asList(nombreClaseCampo));
		sb.append("\n solicitante: " + solicitante);
		return sb.toString();
	}

	protected boolean[] autoIncremento = null;
	protected boolean[] sensibleMayusculas = null;
	protected boolean[] circulacion = null;
	protected boolean[] grabacionSinDuda = null;
	protected boolean[] soloLectura = null;
	protected boolean[] buscable = null;
	protected boolean[] firmado = null;
	protected boolean[] escrito = null;
	protected int[] tamaoColumnaMostrado = null;
	protected int[] tipoColumna = null;
	protected int[] puedeSerNulo = null;
	protected int[] precision = null;
	protected int[] escala = null;
	protected String[] nombreCatalogo = null;
	protected String[] nombreEsquema = null;
	protected String[] nombreTabla = null;
	protected String[] nombreClaseColumna = null;
	protected String[] etiquetaColumna = null;
	protected String[] nombreColumna = null;
	protected String[] nombreTipoColumna = null;
	protected String[] nombreColumnaCompleto = null;
	protected String[] nombreCampo = null;
	// java.lang.reflect.Method no es serializable, asi que despues de enviar
	// el objeto a otra JVM en modo escritura modo lectura se fijar a nulo
	protected transient Method[] metodoLectura = null;
	protected transient Method[] escribirMetodos = null;
	protected String[] nombreClaseCampo = null;
	protected Class[] claseCampo = null;
	protected int contadorColumna = 0;
	protected Class solicitante = null; // ObjPersistencia
	protected Map tablas = new HashMap();
	// Lanzar excepcin o ignorar si ObjPersistencia tiene menos campos que
	// asignaciones
	protected boolean lanzarCampoInexistenteExcepcion = false;

	/**
	 * Devuelve el nombre del catalogo de la tabla de la columna
	 * 
	 * @param columnaIn
	 *            Primera columna el 1, segunda el 2, ...
	 * @return devuelve el nombre del catalogo de la tabla de la columna
	 * @exception SQLException
	 *                si ocurriese algun error de acceso a la base de datos
	 * 
	 */
	public String getNombreCatalogo(int columnaIn) throws SQLException {
		return this.nombreCatalogo[columnaIn - 1];
	}

	public String getNombreClaseColumna(int indiceColumna) throws SQLException {
		return this.nombreClaseColumna[indiceColumna - 1];
	}

	public int getNumeroColumnas() throws SQLException {
		return this.contadorColumna;
	}

	public int getTamaoColumnaMostrado(int cI) throws SQLException {
		return this.tamaoColumnaMostrado[cI - 1];
	}

	public String getEtiquetaColumna(int cI) throws SQLException {
		return this.etiquetaColumna[cI - 1];
	}

	public String getNombreColumna(int cI) throws SQLException {
		return this.nombreColumna[cI - 1];
	}

	public int getTipoColumna(int cI) throws SQLException {
		return this.tipoColumna[cI - 1];
	}

	public String getNombreTipoColumna(int cI) throws SQLException {
		return this.nombreTipoColumna[cI - 1];
	}

	public int getPrecision(int cI) throws SQLException {
		return this.precision[cI - 1];
	}

	public int getEscale(int cI) throws SQLException {
		return this.escala[cI - 1];
	}

	public String getNombreEsquema(int cI) throws SQLException {
		return this.nombreEsquema[cI - 1];
	}

	public String getNombreTabla(int cI) throws SQLException {
		return this.nombreTabla[cI - 1];
	}

	public boolean isAutoIncremento(int cI) throws SQLException {
		return this.autoIncremento[cI - 1];
	}

	public boolean isSensibleMayusculas(int cI) throws SQLException {
		return this.sensibleMayusculas[cI - 1];
	}

	public boolean isCirculacion(int cI) throws SQLException {
		return this.circulacion[cI - 1];
	}

	public boolean isGrabacionSinDuda(int cI) throws SQLException {
		return this.grabacionSinDuda[cI - 1];
	}

	public int isPuedeSerNulo(int cI) throws SQLException {
		return this.puedeSerNulo[cI - 1];
	}

	public boolean isSoloLectura(int cI) throws SQLException {
		return this.soloLectura[cI - 1];
	}

	public boolean isBuscable(int cI) throws SQLException {
		return this.buscable[cI - 1];
	}

	public boolean isFirmado(int cI) throws SQLException {
		return this.firmado[cI - 1];
	}

	public boolean isEscrito(int cI) throws SQLException {
		return this.escrito[cI - 1];
	}

	public void setAutoIncremento(int cI, boolean propiedad)
			throws SQLException {
		this.autoIncremento[cI - 1] = propiedad;
	}

	public void setSensibleMayusculas(int cI, boolean propiedad)
			throws SQLException {
		this.sensibleMayusculas[cI - 1] = propiedad;
	}

	public void setCombreCatalogo(int cI, String nombreCatalogo)
			throws SQLException {
		this.nombreCatalogo[cI - 1] = nombreCatalogo;
	}

	public void setContadorColumna(int contadorColumna) throws SQLException {
		this.contadorColumna = contadorColumna;
	}

	public void setTamaoColumnaMostrado(int cI, int tamao)
			throws SQLException {
		this.tamaoColumnaMostrado[cI - 1] = tamao;
	}

	public void setEtiquetaColumna(int cI, String etiqueta) throws SQLException {
		this.etiquetaColumna[cI - 1] = etiqueta;
	}

	public void setNombreColumna(int cI, String nombreColumna)
			throws SQLException {
		this.nombreColumna[cI - 1] = nombreColumna;
	}

	public void setTipoColumna(int cI, int SQLTipo) throws SQLException {
		this.tipoColumna[cI - 1] = SQLTipo;
	}

	public void setNombreTipoColumna(int cI, String nombreTipo)
			throws SQLException {
		this.nombreTipoColumna[cI - 1] = nombreTipo;
	}

	public void setCirculacion(int cI, boolean propiedad) throws SQLException {
		this.circulacion[cI - 1] = propiedad;
	}

	public void sePuedeSerNulo(int cI, int propiedad) throws SQLException {
		this.puedeSerNulo[cI - 1] = propiedad;
	}

	public void setPrecision(int cI, int precision) throws SQLException {
		this.precision[cI - 1] = precision;
	}

	public void setEscala(int cI, int escala) throws SQLException {
		this.escala[cI - 1] = escala;
	}

	public void setNombreEsquema(int cI, String nombreEsquema)
			throws SQLException {
		this.nombreEsquema[cI - 1] = nombreEsquema;
	}

	public void setBuscable(int cI, boolean propiedad) throws SQLException {
		this.buscable[cI - 1] = propiedad;
	}

	public void setFirmado(int cI, boolean propiedad) throws SQLException {
		this.firmado[cI - 1] = propiedad;
	}

	public void setNombreTabla(int cI, String nombreTabla) throws SQLException {
		this.nombreTabla[cI - 1] = nombreTabla;
	}

	public void setModificable(int cI, boolean escrito) {
		this.escrito[cI - 1] = escrito;
		this.soloLectura[cI - 1] = !escrito;
	}

	public void setGrabacionSinDuda(int cI, boolean grabacionSinDuda) {
		this.grabacionSinDuda[cI - 1] = grabacionSinDuda;
	}

	public void setNombreClaseColumna(int cI,
			java.lang.String nombreClaseColumna) {
		this.nombreClaseColumna[cI - 1] = nombreClaseColumna;
	}

	public void setSoloLectura(int cI, boolean soloLectura) throws SQLException {
		this.soloLectura[cI - 1] = soloLectura;
		this.escrito[cI - 1] = !soloLectura;
	}

	public java.util.Map getTablas() {
		return tablas;
	}

	public void setTablas(java.util.Map tablas) {
		this.tablas = tablas;
	}

	public String getNombreColumnaCompleto(int cI) throws SQLException {
		return this.nombreColumnaCompleto[cI - 1];
	}

	public void setNombreColumnaCompleto(int cI, String nombreColumna)
			throws SQLException {
		this.nombreColumnaCompleto[cI - 1] = nombreColumna;
	}

	public String getNombreCampo(int cI) throws SQLException {
		return this.nombreCampo[cI - 1];
	}

	public void setNombreCampo(int cI, String nombreCampo) throws SQLException {
		this.nombreCampo[cI - 1] = nombreCampo;
	}

	public Method getEscribirMetodos(int cI) throws SQLException {
		if (this.escribirMetodos == null
				|| this.escribirMetodos[cI - 1] == null) {
			poblarEscrituraMetodo(cI, nombreCampo[cI - 1]);
		}
		return this.escribirMetodos[cI - 1];
	}

	public void setEscribirMetodos(int cI, Method escribirMetodos) {
		if (this.escribirMetodos == null) {
			this.escribirMetodos = new Method[contadorColumna];
		}
		this.escribirMetodos[cI - 1] = escribirMetodos;
	}

	public Method getMetodoLectura(int cI) throws SQLException {
		if (this.metodoLectura == null || this.metodoLectura[cI - 1] == null) {
			poblarLecturaMetodo(cI, nombreCampo[cI - 1]);
		}
		return this.metodoLectura[cI - 1];
	}

	public void setMetodoLectura(int columnIndex, Method metodoLectura) {
		if (this.metodoLectura == null) {
			this.metodoLectura = new Method[contadorColumna];
		}
		this.metodoLectura[columnIndex - 1] = metodoLectura;
	}

	public String getNombreCampoCalse(int cI) throws SQLException {
		return this.nombreClaseCampo[cI - 1];
	}

	public void setNombreClaseCampo(int cI, String nombreClaseCampo)
			throws SQLException {
		this.nombreClaseCampo[cI - 1] = nombreClaseCampo;
	}

	public java.lang.Class getClaseCampo(int cI) {
		return this.claseCampo[cI - 1];
	}

	public void setClaseCampo(int cI, Class claseCampo) {
		this.claseCampo[cI - 1] = claseCampo;
	}

	protected void poblarLecturaMetodo(int cI, String nombreCampo)
			throws java.sql.SQLException {
		try {
			Method l = (new PropertyDescriptor(nombreCampo, this.solicitante))
					.getReadMethod();
			this.setMetodoLectura(cI, l);
		} catch (IntrospectionException e) {
			if (lanzarCampoInexistenteExcepcion) {
				e.printStackTrace();
				throw new RuntimeException(e);
			}
		}
	}

	protected void poblarEscrituraMetodo(int cI, String nombreCampo)
			throws java.sql.SQLException {
		try {
			Method e = (new PropertyDescriptor(nombreCampo, this.solicitante))
					.getWriteMethod();
			this.setEscribirMetodos(cI, e);
			Class[] tipoP = e.getParameterTypes();
			Class paraC = tipoP[0];
			this.setClaseCampo(cI, paraC);
			this.setNombreClaseCampo(cI, paraC.getName());
		} catch (IntrospectionException e) {
			if (lanzarCampoInexistenteExcepcion) {
				e.printStackTrace();
				throw new RuntimeException(e);
			}
		}
	}

	public void poblar(Map mapping, Class solicitante)
			throws java.sql.SQLException {
		if (mapping == null || mapping.isEmpty()) {
			throw new SQLException("Metadatos: perdida/invalida asignaciones");
		}
		this.solicitante = solicitante;
		int contadorColumna = mapping.size();
		this.setContadorColumna(contadorColumna);
		this.nombreCatalogo = new String[contadorColumna];
		this.nombreEsquema = new String[contadorColumna];
		this.nombreTabla = new String[contadorColumna];
		this.nombreColumna = new String[contadorColumna];
		this.etiquetaColumna = new String[contadorColumna];
		this.soloLectura = new boolean[contadorColumna];
		this.escrito = new boolean[contadorColumna];
		this.tipoColumna = new int[contadorColumna];
		this.nombreColumnaCompleto = new String[contadorColumna];
		this.nombreCampo = new String[contadorColumna];
		this.metodoLectura = new Method[contadorColumna];
		this.escribirMetodos = new Method[contadorColumna];
		this.claseCampo = new Class[contadorColumna];
		this.nombreClaseCampo = new String[contadorColumna];

		Iterator iter = mapping.keySet().iterator();
		int indiceColumna = 1;
		while (iter.hasNext()) {
			String catalogo = null;
			String esquema = null;
			String tabla = null;
			String tablaId = null;
			String columna = null;
			String nombreColumnaCompleto = (String) iter.next();
			StringTokenizer st = new StringTokenizer(nombreColumnaCompleto,
					".", false);
			int tokens = st.countTokens();
			if (tokens == 2) { // solo los nombres de tabla y columnas, no
								// esquema ni catalogo
				int idx1 = nombreColumnaCompleto.indexOf(".");
				tablaId = nombreColumnaCompleto.substring(0, idx1).trim();
				tabla = tablaId;
			} else if (tokens == 3) { // no catalogo
				int idx1 = nombreColumnaCompleto.indexOf(".");
				int idx2 = nombreColumnaCompleto.lastIndexOf(".");
				tablaId = nombreColumnaCompleto.substring(0, idx2).trim();
				tabla = nombreColumnaCompleto.substring(idx1 + 1, idx2).trim();
				esquema = nombreColumnaCompleto.substring(0, idx1).trim();
			} else if (tokens == 4) { // catalogo, esquema, tabla and columna
				tablaId = nombreColumnaCompleto.substring(0,
						nombreColumnaCompleto.lastIndexOf(".")).trim();
				int i = 0;
				while (st.hasMoreTokens() && i < 3) {
					String token = st.nextToken().trim();
					if (i == 0) {
						catalogo = token;
					} else if (i == 1) {
						esquema = token;
					} else if (i == 2) {
						tabla = token;
					}
					i++;
				}
			} else {
				throw new IllegalArgumentException(
						"Invalidas asignaciones de la base de datos");
			}
			this.setCombreCatalogo(indiceColumna, catalogo);
			this.setNombreEsquema(indiceColumna, esquema);
			this.setNombreTabla(indiceColumna, tabla);
			this.setNombreColumna(indiceColumna, nombreColumnaCompleto
					.substring(nombreColumnaCompleto.lastIndexOf(".") + 1));
			this.setNombreColumnaCompleto(indiceColumna, nombreColumnaCompleto);
			String nombreCampo = (String) mapping.get(nombreColumnaCompleto);
			this.setNombreCampo(indiceColumna, nombreCampo);
			String[] tablasDistintas = new String[3];
			tablasDistintas[0] = catalogo;
			tablasDistintas[1] = esquema;
			tablasDistintas[2] = tabla;
			this.tablas.put(tablaId, tablasDistintas);
			this.setSoloLectura(indiceColumna, false);
			this.poblarLecturaMetodo(indiceColumna, nombreCampo);
			this.poblarEscrituraMetodo(indiceColumna, nombreCampo);
			this.setEtiquetaColumna(indiceColumna,
					etiqueta(getNombreColumna(indiceColumna)));
			indiceColumna++;
		}
	}

	public int getIndiceColumnaPorNombreCampo(String nombreCamposBuscable) {
		if (nombreCamposBuscable == null)
			return -1;
		for (int i = 0; i < this.nombreCampo.length; i++) {
			if (nombreCampo[i] == null)
				return -1;
			if (nombreCamposBuscable.equalsIgnoreCase(nombreCampo[i])) {
				return i + 1;
			}
		}
		return -1;
	}

	public String etiqueta(String nombreColumna) {
		String label = ((String) nombreColumna).toLowerCase();
		StringBuffer sb = new StringBuffer();
		boolean aMayuscula = false;
		for (int i = 0; i < label.length(); i++) {
			char c = label.charAt(i);
			if (c != '_') {
				if (i == 0) {
					aMayuscula = true;
				}
				if (aMayuscula) {
					c = Character.toUpperCase(c);
				} else {
				}
				sb.append(c);
				aMayuscula = false;
			} else {
				if (i > 0) {
					sb.append(' ');
				}
				aMayuscula = true;
			}
		}
		return sb.toString();
	}

	public boolean isLanzarCampoInexistenteExcepcion() {
		return lanzarCampoInexistenteExcepcion;
	}

	public void setLanzaCampoInexistenteExcepcion(
			boolean lanzarCampoInexistenteExcepcion) {
		this.lanzarCampoInexistenteExcepcion = lanzarCampoInexistenteExcepcion;
	}
}
