package slgfp;

import java.beans.*;
import java.util.*;
import java.lang.reflect.*;

/**
 * La clase nos sirve para poblar y cachear columnas, tablas, campos, etc. En el
 * caso de que por necesidades de implementacin no se pueda heredar de esta
 * clase, deber implementar la interfaz slgfp.ObjetoPersistente y copiar todos
 * los elementos y mtodos desde slgfp.ObjetoPersistenteAbstracto en su propia
 * clase.
 * 
 * @version 1.0, 03-05-2011
 * @author Sergio Lovillo
 */

@SuppressWarnings({ "serial", "rawtypes", "unchecked" })
public abstract class ObjPersistenciaAbstracto implements ObjPersistencia,
		java.io.Serializable, Cloneable {

	protected ContenedorDatos valoresOriginales = null;
	protected int objectoId = 0;
	protected char estadoPersistencia = UNDEFINED;
	protected boolean cargado = false;
	protected boolean cargando = false;
	protected boolean modificado = false;
	protected Map valoresMostrados;
	protected static Object[] VACIO_LECTURA_ARG = new Object[0];
	protected long fabricaPersistenteId = -1;
	protected String juegoCaracteres = "UTF-8";

	public String toString() {
		StringBuffer sb = new StringBuffer();
		Object valor = null;
		Method[] metodos = getClass().getMethods();
		for (int i = 0; i < metodos.length; i++) {
			String nombreMetodo = metodos[i].getName();
			if (nombreMetodo.equals("getValoresComoHtml")
					|| nombreMetodo.equals("getClass")) {
				continue;
			}
			if ((nombreMetodo.startsWith("get") || nombreMetodo
					.startsWith("is"))
					&& metodos[i].getParameterTypes().length == 0) {
				try {
					valor = leerValor(metodos[i]);
				} catch (Throwable t) {
					continue;
				}
				String primerCampoChar = "";
				if (nombreMetodo.startsWith("is")) {
					primerCampoChar = nombreMetodo.substring(2, 3)
							.toLowerCase();
					sb.append(primerCampoChar);
					sb.append(nombreMetodo.substring(3));
				} else if (nombreMetodo.startsWith("get")) {
					primerCampoChar = nombreMetodo.substring(3, 4)
							.toLowerCase();
					sb.append(primerCampoChar);
					sb.append(nombreMetodo.substring(4));
				}
				sb.append("=");
				sb.append((valor == null) ? "" : valor);
				sb.append("&");
			}
		}
		sb.append("valoresMostrados="
				+ (valoresMostrados == null ? "" : valoresMostrados.toString()));
		return sb.toString();
	}

	public void setValoresOriginales(ContenedorDatos valoresOriginales) {
		this.valoresOriginales = valoresOriginales;
	}

	public ContenedorDatos getValoresOriginales() {
		return this.valoresOriginales;
	}

	public void setValoresOriginales(String nombreCampo, Object valor) {
		if (valoresOriginales == null) {
			throw new IllegalArgumentException(
					"setValoresOriginales: ContenedorDatos no actualizable");
		}
		valoresOriginales.setObjecto(nombreCampo, valor);
	}

	public Object getValoresOriginales(String nombreCampo) {
		Object valor = (valoresOriginales == null) ? null : valoresOriginales
				.getObject(nombreCampo);
		return valor;
	}

	public char getEstadoPersistencia() {
		return estadoPersistencia;
	}

	public void setEstadoPersistencia(char estadoPersistencia) {
		this.estadoPersistencia = estadoPersistencia;
	}

	public int getObjectoId() {
		return objectoId;
	}

	public void setObjectoId(int objectoId) {
		this.objectoId = objectoId;
	}

	public boolean almacenar() {
		if (valoresOriginales != null
				&& valoresOriginales.getContadorCampos() > 0) {
			setEstadoPersistencia(ALMACENADO);
			return true;
		}
		return false;
	}

	public boolean eliminar() {
		if (valoresOriginales != null
				&& valoresOriginales.getContadorCampos() > 0) {
			setEstadoPersistencia(ELIMINADO);
			return true;
		}
		return false;
	}

	public boolean crear() {
		if (valoresOriginales == null
				|| valoresOriginales.getContadorCampos() < 1) {
			setEstadoPersistencia(CREADO);
			return true;
		}
		return false;
	}

	public boolean syncOriginal() {
		boolean sync = false;
		if (valoresOriginales != null
				&& valoresOriginales.getContadorCampos() > 0) {
			try {
				setEstadoPersistencia(ORIGINAL);
				for (int indiceCampos = 1; indiceCampos <= valoresOriginales
						.getContadorCampos(); indiceCampos++) {
					String nombreCampo = valoresOriginales
							.getNombreCampo(indiceCampos);
					PropertyDescriptor pd = new PropertyDescriptor(nombreCampo,
							this.getClass());
					Object value = leerValor(pd.getReadMethod());
					valoresOriginales.setObjecto(indiceCampos, value);
				}
				sync = true;
				setModificado(false);
			} catch (Throwable t) {
				throw new RuntimeException(t);
			}
		}
		return sync;
	}

	// puebla ObjPersistencia con valores desde valoresOriginales.
	public boolean cargar() {
		setCargando(true);
		if (valoresOriginales != null
				&& valoresOriginales.getContadorCampos() > 0) {
			try {
				List p = new ArrayList(1);
				for (int indiceCampo = 1; indiceCampo <= valoresOriginales
						.getContadorCampos(); indiceCampo++) {
					p.clear();
					String nombreCampo = valoresOriginales
							.getNombreCampo(indiceCampo);
					Object valor = valoresOriginales.getObject(indiceCampo);
					PropertyDescriptor pd = new PropertyDescriptor(nombreCampo,
							this.getClass());
					p.add(valor);
					escribirValor(pd.getWriteMethod(), p.toArray());
				}
			} catch (Throwable t) {
				throw new RuntimeException(t);
			}
			setCargando(false);
			setCargado(true);
			setModificado(false);
			return true;
		}
		setCargando(false);
		return false;
	}

	public boolean isCargado() {
		return cargado;
	}

	public void setCargado(boolean cargado) {
		this.setEstadoPersistencia(ORIGINAL);
		this.cargado = cargado;
	}

	/**
	 * Este mtodo es adecuado para uso con JSP.
	 */

	public java.lang.String getValoresComoHtml() {
		StringBuffer sb = new StringBuffer();
		Object valor = null;
		Method[] metodos = getClass().getMethods();
		for (int i = 0; i < metodos.length; i++) {
			String nombreMetodos = metodos[i].getName();
			if (nombreMetodos.equals("getValoresComoHtml")
					|| nombreMetodos.equals("getClass")) {
				continue;
			}
			if ((nombreMetodos.startsWith("get") || nombreMetodos
					.startsWith("is"))
					&& metodos[i].getParameterTypes().length == 0) {
				try {
					valor = leerValor(metodos[i]);
				} catch (Throwable t) {
					continue;
				}
				String primerCampoChar = "";
				if (nombreMetodos.startsWith("is")) {
					primerCampoChar = nombreMetodos.substring(2, 3)
							.toLowerCase();
					sb.append(primerCampoChar);
					sb.append(nombreMetodos.substring(3));
				} else if (nombreMetodos.startsWith("get")) {
					primerCampoChar = nombreMetodos.substring(3, 4)
							.toLowerCase();
					sb.append(primerCampoChar);
					sb.append(nombreMetodos.substring(4));
				}
				sb.append("=");
				if (valor == null) {
					sb.append("");
				} else {
					try {
						sb.append(java.net.URLEncoder.encode(valor.toString(),
								juegoCaracteres));
					} catch (java.io.UnsupportedEncodingException uee) {
						throw new RuntimeException(uee);
					}
				}
				sb.append("&");
			}
		}
		sb.deleteCharAt(sb.length() - 1);
		return sb.toString();
	}

	/**
	 * Indicador de que el objeto esta realizando el proceso de carga con valore
	 * de la base de datos
	 */

	public boolean isCargando() {
		return this.cargando;
	}

	public void setCargando(boolean cargando) {
		this.cargando = cargando;
	}

	/**
	 * Indicador de que el objeto se ha modificado. Debe ser llamado a si mismo
	 */
	public boolean isModificado() {
		return this.modificado;
	}

	public void setModificado(boolean modificado) {
		this.modificado = modificado;
	}

	/**
	 * Muestra el valor para el campo dado. Ejemplo: el valor del campo es "123"
	 * pero lo que debe ser es "XYZ"
	 */
	public void setValoresMostrados(String campo, Object valorMostrado) {
		if (valoresMostrados == null)
			valoresMostrados = new HashMap();
		valoresMostrados.put(campo, valorMostrado);
	}

	public Object getMostrarValor(String nombreCampo) {
		Object mostrado;
		try {
			if (valoresMostrados == null) {
				PropertyDescriptor pDesc = new PropertyDescriptor(nombreCampo,
						this.getClass());
				mostrado = leerValor(pDesc.getReadMethod());
			} else {
				mostrado = valoresMostrados.get(nombreCampo);
			}
		} catch (Throwable t) {
			throw new RuntimeException(t);
		}
		return mostrado;
	}

	public void setFabricaPersitenciaId(long fabricaPersistenciaId) {
		this.fabricaPersistenteId = fabricaPersistenciaId;
	}

	public long getFabricaPersistenciaId() {
		return this.fabricaPersistenteId;
	}

	protected void escribirValor(Method metodo, Object[] valor)
			throws Throwable {
		try {
			metodo.invoke(this, valor);
		} catch (Throwable t) {
			if (t instanceof InvocationTargetException) {
				throw ((InvocationTargetException) t).getTargetException();
			} else {
				throw t;
			}
		}
	}

	protected Object leerValor(Method metodo) throws Throwable {
		Object valor = null;
		try {
			valor = metodo.invoke(this, VACIO_LECTURA_ARG);
		} catch (Throwable t) {
			if (t instanceof InvocationTargetException) {
				throw ((InvocationTargetException) t).getTargetException();
			} else {
				throw t;
			}
		}
		return valor;
	}
}
