package slgfp;

import java.util.*;
import java.sql.Connection;

/**
 * La clase nos sirve para recuperar y almacenar claves primarias desde los
 * Metadatos de la base de datos o entrada de usuario.
 * 
 * @version 1.0, 03-05-2011
 * @author Sergio Lovillo
 */

@SuppressWarnings({ "serial", "rawtypes", "unchecked" })
public class PKCache implements java.io.Serializable, Cloneable {

	private PKCache() {
		if (Boolean.getBoolean("debug-slgfp")
				|| Boolean.getBoolean("debug-" + getClass().getName())) {
			setDebug(true);
		}
	}

	private static PKCache instancia = null;
	/* Clave para cada tabla */
	protected Map pkCache = new HashMap();
	protected boolean debug = false;
	private static final String DOT = ".";

	public static PKCache getInstance() {
		if (instancia == null) {
			instancia = new PKCache();
		}
		return instancia;
	}

	/*
	 * Este metodo permite definir columnas para una clave primaria de tabla sin
	 * conexion con la base de datos, etc.
	 * 
	 * EL formato correcto es: catalogo + DOT + esquema + DOT + tabla + DOT +
	 * columna or esquema + DOT + tabla + DOT + columna
	 */
	public synchronized void setClavePrimaria(String catalogo, String esquema,
			String tabla, List pk) {
		String tablaId = null;
		if (catalogo != null && catalogo.trim().length() != 0) {
			tablaId = catalogo + DOT + esquema + DOT + tabla;
		} else if (esquema != null && esquema.trim().length() != 0) {
			tablaId = esquema + DOT + tabla;
		} else if (tabla != null && tabla.trim().length() != 0) {
			tablaId = tabla;
		} else {
			throw new IllegalArgumentException(
					"setClavePrimaria(): perdidos todos los identificadores de tabla");
		}
		if (pk == null || pk.size() == 0) {
			throw new IllegalArgumentException(
					"setClavePrimaria(): Lista PK vacia");
		}
		if (debug)
			System.out.println("slgfp ============= " + new java.util.Date()
					+ " " + this.getClass() + "::" + this
					+ "::setClavePrimaria()::tablaId: " + tablaId + " pk: "
					+ pk);
		pkCache.put(tablaId, pk);
	}

	public synchronized List getClavePrimaria(String catalogo, String esquema,
			String tabla) {
		if (debug)
			System.out.println("slgfp ============= " + new java.util.Date()
					+ " " + this.getClass() + "::" + this
					+ "::getClavePrimaria()::catalogo: " + catalogo
					+ "::esquema: " + esquema + "::tabla: " + tabla);
		String tablaId = null;
		// Construye id tabla
		if (catalogo != null && catalogo.trim().length() != 0) {
			tablaId = catalogo.trim() + DOT + esquema.trim() + DOT
					+ tabla.trim();
		} else if (esquema != null && esquema.trim().length() != 0) {
			tablaId = esquema.trim() + DOT + tabla.trim();
		} else if (tabla != null && tabla.trim().length() != 0) {
			tablaId = tabla.trim();
		} else {
			throw new IllegalArgumentException(
					"getClavePrimaria(): perdidos todos los identificadores de tabla");
		}
		if (this.pkCache.containsKey(tablaId)) {
			return (List) this.pkCache.get(tablaId); // Ya ha sido cacheada
		} else {
			return null;
		}
	}

	public synchronized List getClavePrimaria(Connection conexion,
			String catalogo, String esquema, String tabla) {
		if (debug)
			System.out.println("slgfp ============= " + new java.util.Date()
					+ " " + this.getClass() + "::" + this
					+ "::getClavePrimaria()::catalogo: " + catalogo
					+ "::esquema: " + esquema + "::tabla: " + tabla);
		String tablaId = null;
		// Construye id tabla
		if (catalogo != null && catalogo.trim().length() != 0) {
			tablaId = catalogo.trim() + DOT + esquema.trim() + DOT
					+ tabla.trim();
		} else if (esquema != null && esquema.trim().length() != 0) {
			tablaId = esquema.trim() + DOT + tabla.trim();
		} else if (tabla != null && tabla.trim().length() != 0) {
			tablaId = tabla.trim();
		} else {
			throw new IllegalArgumentException(
					"getClavePrimaria(): perdidos todos los identificadores de tabla");
		}
		List camposPK = (List) this.pkCache.get(tablaId);
		if (debug)
			System.out.println("slgfp ============= " + new java.util.Date()
					+ " " + this.getClass() + "::" + this
					+ "::getClavePrimaria()::tablaId: " + tablaId
					+ " camposPK: " + camposPK);
		if (camposPK != null) {
			return camposPK; // it's already cached
		} else {
			camposPK = new ArrayList();
		}
		java.sql.ResultSet pkInfo = null;
		try {
			java.sql.DatabaseMetaData dbmd = conexion.getMetaData();
			if (debug)
				System.out.println("slgfp ============= "
						+ new java.util.Date() + " " + this.getClass() + "::"
						+ this + "::getClavePrimaria()::DatabaseMetaDatos: "
						+ dbmd.getDatabaseProductName() + "::catalogo: "
						+ catalogo + "::esquema: " + esquema + "::tabla: "
						+ tabla);
			pkInfo = dbmd.getPrimaryKeys(catalogo, esquema, tabla);
			if (debug)
				System.out.println("slgfp ============= "
						+ new java.util.Date() + " " + this.getClass() + "::"
						+ this + "::getClavePrimaria()::pkInfo: " + pkInfo);
			while (pkInfo.next()) {
				if (debug)
					System.out.println("slgfp ============= "
							+ new java.util.Date() + " " + this.getClass()
							+ "::" + this
							+ "::getClavePrimaria()::pkInfo.next()");
				String nombreColumna = pkInfo.getString(4);
				camposPK.add(tablaId + DOT + nombreColumna);
			}
			if (debug)
				System.out.println("slgfp ============= "
						+ new java.util.Date() + " " + this.getClass() + "::"
						+ this + "::getClavePrimaria()::camposPK: " + camposPK);
			if (camposPK.isEmpty()) {
				throw new java.sql.SQLException(
						"No puedo recibir la informacion de las PK desde los metadatos de la base de datos");
			}
		} catch (java.sql.SQLException sqle) {
			throw new RuntimeException(sqle);
		} catch (Exception e) {
			throw new RuntimeException(e);
		} finally {
			try {
				if (pkInfo != null)
					pkInfo.close();
			} catch (java.sql.SQLException sqle) {
				sqle.printStackTrace();
				throw new RuntimeException(sqle);
			}
		}
		pkCache.put(tablaId, camposPK);
		return camposPK;
	}

	public boolean isDebug() {
		return debug;
	}

	public void setDebug(boolean debug) {
		this.debug = debug;
	}
}
