package slgfp;

import java.sql.*;

/**
 * La clase nos sirve para definir los parmetros del frameworks de manera
 * general
 * 
 * @version 1.0, 03-05-2011
 * @author Sergio Lovillo
 */

@SuppressWarnings({ "serial", "rawtypes", "unchecked" })
public class Parametros implements ParameterMetaData, java.io.Serializable {

	// Parametros - El primer parametro es el 1, el segundo el 2 y as
	// sucesivamente
	protected Object[] parametros;
	protected int[] parametroModo;
	protected int[] parametroTipo;
	protected String[] parametroNombreTipo;
	protected int[] precision;
	protected int[] escala;
	protected int[] nulo;
	protected boolean[] signo;
	protected String[] parametroNombre;
	protected int parametroContador;

	public Parametros() {
	}

	public boolean isWrapperFor(Class iface) {
		return false;
	}

	public Class unwrap(Class iface) {
		return null;
	}

	public Parametros(int contadorParametros) {
		setParametros(new Object[contadorParametros]);
	}

	protected void iniciar() {
		this.parametroContador = this.parametros.length;
		this.parametroModo = new int[parametroContador];
		this.parametroTipo = new int[parametroContador];
		this.parametroNombreTipo = new String[parametroContador];
		this.precision = new int[parametroContador];
		this.escala = new int[parametroContador];
		this.nulo = new int[parametroContador];
		this.signo = new boolean[parametroContador];
		this.parametroNombre = new String[parametroContador];
	}

	public Parametros(Object[] parametros) {
		setParametros(parametros);
	}

	public String getParameterClassName(int i) throws SQLException {
		return parametros[i - 1].getClass().getName();
	}

	public int getParameterCount() throws SQLException {
		return parametroContador;
	}

	public int getParameterMode(int i) throws SQLException {
		return parametroModo[i - 1];
	}

	public void setParameterMode(int i, int parameterModo) throws SQLException {
		this.parametroModo[i - 1] = parameterModo;
	}

	public int getParameterType(int i) throws SQLException {
		return parametroTipo[i - 1];
	}

	public void setParametroTipo(int i, int parametroTipo) throws SQLException {
		this.parametroTipo[i - 1] = parametroTipo;
	}

	public String getParameterTypeName(int i) throws SQLException {
		return parametroNombreTipo[i - 1];
	}

	public void setParameterTypeName(int i, String parametroNombreTipo)
			throws SQLException {
		this.parametroNombreTipo[i - 1] = parametroNombreTipo;
	}

	public int getPrecision(int i) throws SQLException {
		return precision[i - 1];
	}

	public void setPrecision(int i, int precision) throws SQLException {
		this.precision[i - 1] = precision;
	}

	public int getScale(int i) throws SQLException {
		return escala[i - 1];
	}

	public void setScale(int i, int escala) throws SQLException {
		this.escala[i - 1] = escala;
	}

	public int isNullable(int i) throws SQLException {
		return nulo[i - 1];
	}

	public void setNullable(int i, int nulo) throws SQLException {
		this.nulo[i - 1] = nulo;
	}

	public boolean isSigned(int i) throws SQLException {
		return signo[i - 1];
	}

	public void setSigned(int i, boolean s) throws SQLException {
		this.signo[i - 1] = s;
	}

	public Object[] getParametros() {
		return parametros;
	}

	public void setParametros(Object[] p) {
		this.parametros = p;
		iniciar();
	}

	public Object getParametros(int i) {
		return parametros[i - 1];
	}

	public Object getParametros(String nombreParametros) throws SQLException {
		return parametros[getIndiceParametro(nombreParametros) - 1];
	}

	public void setParametros(int i, Object valor) {
		this.parametros[i - 1] = valor;
	}

	public String getParametroNombre(int i) throws SQLException {
		return parametroNombre[i - 1];
	}

	public void setParametroNombre(int id, String parametroNombre)
			throws SQLException {
		if (parametroNombre == null || parametroNombre.trim().length() == 0) {
			throw new SQLException("Nombre del parametro no valido");
		}
		parametroNombre = parametroNombre.trim();
		for (int i = 0; i < parametroContador; i++) {
			String nombre = this.parametroNombre[i];
			if (nombre != null && nombre.equalsIgnoreCase(parametroNombre)) {
				throw new SQLException("Nombre de parametro duplicado: "
						+ parametroNombre);
			}
		}
		this.parametroNombre[id - 1] = parametroNombre;
	}

	public int getIndiceParametro(String parametroNombre) throws SQLException {
		if (parametroNombre == null || parametroNombre.trim().length() == 0) {
			return -1;
		}
		for (int i = 0; i < parametroContador; i++) {
			String nombre = this.parametroNombre[i];
			if (nombre != null && nombre.equalsIgnoreCase(parametroNombre)) {
				return i + 1;
			}
		}
		return -1;
	}

	public String toString() {
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < parametroContador; i++) {
			sb.append("\nparametro").append(i + 1).append(": ");
			sb.append("parametroNombre=").append(parametroNombre[i]);
			sb.append("&valor=").append(parametros[i]);
			sb.append("&parametroModo=").append(parametroModo[i]);
			sb.append("&parametroTipo=").append(parametroTipo[i]);
			sb.append("&parametroNombreTipo=").append(parametroNombreTipo[i]);
			sb.append("&precision=").append(precision[i]);
			sb.append("&escala=").append(escala[i]);
			sb.append("&nulo=").append(nulo[i]);
			sb.append("&signo=").append(signo[i]);
		}
		return sb.toString();
	}
}
