package slgfp;

import java.io.Serializable;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * La clase nos sirve para recuperar y transformar los DataTypes. Se considera
 * que en una ampliacion posterior se puedan incorporar ms tipos nuevos.
 * 
 * @version 1.0, 03-05-2011
 * @author Sergio Lovillo
 */

@SuppressWarnings({ "serial" })
public class TransformadorDefecto implements Serializable, Transformador {

	protected transient ResultSet resultSet;

	public TransformadorDefecto() {
	}

	public TransformadorDefecto(ResultSet resultSet) {
		setResultSet(resultSet);
	}

	public Object[] transformar(String nombreClaseCampo, String nombreColumna)
			throws SQLException {
		Object valor = null;
		Object[] valorTransformado = null;
		// Tenemos definidos ciertos tipos de datos. Este punto presenta una
		// posibilidad de expansion.
		if (nombreClaseCampo == null) {
			valorTransformado = new Object[1];
			valor = resultSet.getObject(nombreColumna);
			valorTransformado[0] = valor;
		} else if (nombreClaseCampo.equals("java.lang.String")) {
			valorTransformado = new String[1];
			valor = resultSet.getString(nombreColumna);
			if (resultSet.wasNull()) {
				valorTransformado[0] = (String) null;
			} else {
				valorTransformado[0] = valor;
			}
		} else if (nombreClaseCampo.equals("java.lang.Integer")) {
			valorTransformado = new Integer[1];
			valor = new Integer(resultSet.getInt(nombreColumna));
			if (resultSet.wasNull()) {
				valorTransformado[0] = (Integer) null;
			} else {
				valorTransformado[0] = valor;
			}
		} else if (nombreClaseCampo.equals("java.lang.Long")) {
			valorTransformado = new Long[1];
			valor = new Long(resultSet.getLong(nombreColumna));
			if (resultSet.wasNull()) {
				valorTransformado[0] = (Long) null;
			} else {
				valorTransformado[0] = valor;
			}
		} else if (nombreClaseCampo.equals("java.lang.Double")) {
			valorTransformado = new Double[1];
			valor = new Double(resultSet.getDouble(nombreColumna));
			if (resultSet.wasNull()) {
				valorTransformado[0] = (Double) null;
			} else {
				valorTransformado[0] = valor;
			}
		} else if (nombreClaseCampo.equals("java.lang.Float")) {
			valorTransformado = new Float[1];
			valor = new Float(resultSet.getFloat(nombreColumna));
			if (resultSet.wasNull()) {
				valorTransformado[0] = (Float) null;
			} else {
				valorTransformado[0] = valor;
			}
		} else if (nombreClaseCampo.equals("java.lang.Short")) {
			valorTransformado = new Short[1];
			valor = new Short(resultSet.getShort(nombreColumna));
			if (resultSet.wasNull()) {
				valorTransformado[0] = (Short) null;
			} else {
				valorTransformado[0] = valor;
			}
		} else if (nombreClaseCampo.equals("java.sql.Timestamp")) {
			valorTransformado = new java.sql.Timestamp[1];
			valor = (java.sql.Timestamp) resultSet.getTimestamp(nombreColumna);
			if (resultSet.wasNull()) {
				valorTransformado[0] = (java.sql.Timestamp) null;
			} else {
				valorTransformado[0] = valor;
			}
		} else if (nombreClaseCampo.equals("java.sql.Time")) {
			valorTransformado = new java.sql.Time[1];
			valor = (java.sql.Time) resultSet.getTime(nombreColumna);
			if (resultSet.wasNull()) {
				valorTransformado[0] = (java.sql.Time) null;
			} else {
				valorTransformado[0] = valor;
			}
		} else if (nombreClaseCampo.equals("java.sql.Date")) {
			valorTransformado = new java.sql.Date[1];
			valor = (java.sql.Date) resultSet.getDate(nombreColumna);
			if (resultSet.wasNull()) {
				valorTransformado[0] = (java.sql.Date) null;
			} else {
				valorTransformado[0] = valor;
			}
		} else if (nombreClaseCampo.equals("java.util.Date")) {
			valorTransformado = new java.util.Date[1];
			java.sql.Date d = resultSet.getDate(nombreColumna);
			if (resultSet.wasNull()) {
				valorTransformado[0] = (java.util.Date) null;
			} else {
				valor = new java.util.Date(d.getTime());
				valorTransformado[0] = valor;
			}
		} else if (nombreClaseCampo.equals("java.math.BigDecimal")) {
			valorTransformado = new java.math.BigDecimal[1];
			valor = resultSet.getBigDecimal(nombreColumna);
			if (resultSet.wasNull()) {
				valorTransformado[0] = (java.math.BigDecimal) null;
			} else {
				valorTransformado[0] = valor;
			}
		} else if (nombreClaseCampo.equals("java.math.BigInteger")) {
			valorTransformado = new java.math.BigInteger[1];
			long longValor = 0;
			longValor = resultSet.getLong(nombreColumna);
			if (resultSet.wasNull()) {
				valorTransformado[0] = (java.math.BigInteger) null;
			} else {
				valor = java.math.BigInteger.valueOf(longValor);
				valorTransformado[0] = valor;
			}
		} else if (nombreClaseCampo.equals("boolean")) {
			valorTransformado = new Boolean[1];
			boolean booleanValor = false;
			booleanValor = resultSet.getBoolean(nombreColumna);
			if (resultSet.wasNull()) {
				valorTransformado[0] = new Boolean(false);
			} else {
				valor = new Boolean(booleanValor);
				valorTransformado[0] = valor;
			}
		} else if (nombreClaseCampo.equals("byte")) {
			valorTransformado = new Byte[1];
			byte byteValor = 0;
			byteValor = resultSet.getByte(nombreColumna);
			if (resultSet.wasNull()) {
				byte vacioByte = 0;
				valorTransformado[0] = new Byte(vacioByte);
			} else {
				valor = new Byte(byteValor);
				valorTransformado[0] = valor;
			}
		} else if (nombreClaseCampo.equals("char")) {
			valorTransformado = new Character[1];
			String stringValor = null;
			stringValor = resultSet.getString(nombreColumna);
			if (resultSet.wasNull()) {
				char vacioChar = '\u0000';
				valorTransformado[0] = new Character(vacioChar);
			} else {
				valor = new Character(stringValor.charAt(0));
				valorTransformado[0] = valor;
			}
		} else if (nombreClaseCampo.equals("double")) {
			valorTransformado = new Double[1];
			double doubleValor = 0.0;
			doubleValor = resultSet.getDouble(nombreColumna);
			if (resultSet.wasNull()) {
				valorTransformado[0] = new Double(0.0);
			} else {
				valor = new Double(doubleValor);
				valorTransformado[0] = valor;
			}
		} else if (nombreClaseCampo.equals("float")) {
			valorTransformado = new Float[1];
			float floatValor = 0;
			floatValor = resultSet.getFloat(nombreColumna);
			if (resultSet.wasNull()) {
				valorTransformado[0] = new Float(0.0);
			} else {
				valor = new Float(floatValor);
				valorTransformado[0] = valor;
			}
		} else if (nombreClaseCampo.equals("int")) {
			valorTransformado = new Integer[1];
			int intValor = 0;
			intValor = resultSet.getInt(nombreColumna);
			if (resultSet.wasNull()) {
				valorTransformado[0] = new Integer(0);
			} else {
				valor = new Integer(intValor);
				valorTransformado[0] = valor;
			}
		} else if (nombreClaseCampo.equals("long")) {
			valorTransformado = new Long[1];
			long longValor = 0;
			longValor = resultSet.getLong(nombreColumna);
			if (resultSet.wasNull()) {
				valorTransformado[0] = new Long(0);
			} else {
				valor = new Long(longValor);
				valorTransformado[0] = valor;
			}
		} else if (nombreClaseCampo.equals("short")) {
			valorTransformado = new Short[1];
			short shortValor = 0;
			shortValor = resultSet.getShort(nombreColumna);
			if (resultSet.wasNull()) {
				short s = 0;
				valorTransformado[0] = new Short(s);
			} else {
				valor = new Short(shortValor);
				valorTransformado[0] = valor;
			}
		} else if (nombreClaseCampo.equals("java.lang.Boolean")) {
			valorTransformado = new Boolean[1];
			boolean booleanValor = false;
			booleanValor = resultSet.getBoolean(nombreColumna);
			if (resultSet.wasNull()) {
				valorTransformado[0] = new Boolean(false);
			} else {
				valor = new Boolean(booleanValor);
				valorTransformado[0] = valor;
			}
		} else if (nombreClaseCampo.equals("java.lang.Character")) {
			valorTransformado = new Character[1];
			String stringValor = null;
			stringValor = resultSet.getString(nombreColumna);
			if (resultSet.wasNull()) {
				char vacioChar = '\u0000';
				valorTransformado[0] = new Character(vacioChar);
			} else {
				valor = new Character(stringValor.charAt(0));
				valorTransformado[0] = valor;
			}
		} else {
			valorTransformado = new Object[1];
			valor = resultSet.getObject(nombreColumna);
			valorTransformado[0] = valor;
		}
		return valorTransformado;
	}

	public ResultSet getResultSet() {
		return resultSet;
	}

	public void setResultSet(ResultSet resultSet) {
		this.resultSet = resultSet;
	}
}
