package slgfp;

/**
 * La clase nos sirve para realizar la gestin de los valores de los objetos.
 * Con esta clase es ms cmodo su manejo.
 * 
 * @version 1.0, 03-05-2011
 * @author Sergio Lovillo
 */

@SuppressWarnings({ "serial", "rawtypes", "unchecked" })
public class ValorObjeto implements Cloneable, java.io.Serializable, Comparable {

	protected Object valor;
	protected String etiquetaValor;
	protected boolean compararPorValor = false;
	protected boolean errorComparacionNula;

	public ValorObjeto() {
	}

	public ValorObjeto(Object valor, String etiquetaValor) {
		this.valor = valor;
		this.etiquetaValor = etiquetaValor;
	}

	public ValorObjeto(Object valor, String etiquetaValor,
			boolean compararPorValor) {
		this.valor = valor;
		this.etiquetaValor = etiquetaValor;
		this.compararPorValor = compararPorValor;
	}

	public java.lang.Object getValor() {
		return valor;
	}

	public void setValor(java.lang.Object valor) {
		this.valor = valor;
	}

	public java.lang.String getEtiquetaValor() {
		if (etiquetaValor == null) {
			if (valor != null) {
				etiquetaValor = valor.toString();
			} else {
				etiquetaValor = "";
			}
		}
		return etiquetaValor;
	}

	public void setEtiquetaValor(java.lang.String etiquetaValor) {
		this.etiquetaValor = etiquetaValor;
	}

	public boolean isCompararPorValor() {
		return compararPorValor;
	}

	public void setCompararPorValor(boolean compararPorValor) {
		this.compararPorValor = compararPorValor;
	}

	public String toString() {
		return getEtiquetaValor();
	}

	public int compareTo(Object o) {
		if (o == null) {
			if (errorComparacionNula) {
				throw new NullPointerException("ValorObjeto es nulo");
			} else {
				return -1;
			}
		}
		if (this.equals(o)) {
			return 0;
		}
		ValorObjeto other = (ValorObjeto) o;
		int i1 = this.etiquetaValor.compareTo(other.etiquetaValor);
		int i2 = ((Comparable) this.valor).compareTo((Comparable) other.valor);
		if (i1 > 0 && i2 > 0) {
			return 1;
		} else if (i1 < 0 && i2 < 0) {
			return -1;
		} else {
			if (compararPorValor) {
				return i2;
			} else {
				return i1;
			}
		}
	}

	public boolean equals(java.lang.Object o) {
		if (!(o instanceof ValorObjeto)) {
			return false;
		}
		ValorObjeto aux = (ValorObjeto) o;
		return ((valor == null ? aux.valor == null : valor.equals(aux.valor)) && (etiquetaValor == null ? aux.etiquetaValor == null
				: etiquetaValor.equals(aux.etiquetaValor)));
	}

	public int hashCode() {
		return ((valor == null ? -1 : valor.hashCode()) ^ (etiquetaValor == null ? -1
				: etiquetaValor.hashCode()));
	}

	public boolean isErrorComparacionNula() {
		return errorComparacionNula;
	}

	public void setErrorComparacionNula(boolean errorComparacionNula) {
		this.errorComparacionNula = errorComparacionNula;
	}

}
