package slgfp.buscar;

import java.util.*;
import slgfp.*;
import java.lang.reflect.Method;

/**
 * La clase nos sirve para realizar lo necesario para construir consultas a la
 * base de datos
 * 
 * @version 1.0, 03-05-2011
 * @author Sergio Lovillo
 */

@SuppressWarnings({ "rawtypes", "unchecked" })
public abstract class GeneradorCriteriosBusqueda {

	/**
	 * Esta clase se encargar de realizar lo necesario para construir consultas
	 * a la base de datos
	 */

	public GeneradorCriteriosBusqueda() {
	}

	protected List fields;
	protected Metadatos metadatos;
	protected List operatorsList;
	protected Map fieldOperators;
	protected Map fieldValues;
	protected List searchCriteriaHolders = new ArrayList();
	protected List arguments = new ArrayList();
	protected String select = "";
	protected String from = "";
	protected String joins = "";
	protected String where = "";
	protected String groupBy = "";
	protected String having = "";
	protected String orderBy = "";
	protected String query = "";
	protected String dynamicWhere = "";
	protected String ejecutable;
	protected Map likeHint = new HashMap(3);
	protected static final String SPACE = " ";
	protected Set adhocColumnas = new HashSet();

	public void beforeBuildCriteria() {
	}

	public void resetear() {
		if (searchCriteriaHolders != null)
			searchCriteriaHolders.clear();
		if (arguments != null)
			arguments.clear();
		if (adhocColumnas != null)
			adhocColumnas.clear();
		select = "";
		from = "";
		joins = "";
		where = "";
		groupBy = "";
		having = "";
		orderBy = "";
		query = "";
		dynamicWhere = "";
	}

	public String construirCriterio() {
		beforeBuildCriteria();
		StringBuffer sb = new StringBuffer();
		int size = searchCriteriaHolders.size();
		Iterator it = searchCriteriaHolders.iterator();
		int count = 0;
		while (it.hasNext()) {
			count++;
			ManejadorCriterioBusqueda holder = (ManejadorCriterioBusqueda) it
					.next();
			String fieldName = holder.getNombreCampo();
			String functions = holder.getFunciones();
			if (fieldName == null || fieldName.trim().equals("")) {
				throw new IllegalArgumentException(
						"Busqueda: no se encuentra el campo name");
			}
			String operator = holder.getOperator();
			Object searchValue = holder.getValorBuscado();
			String booleanCondition = holder.getCondicionBoolean();
			String columnName;
			try {
				columnName = metadatos.getNombreColumnaCompleto(metadatos
						.getIndiceColumnaPorNombreCampo(fieldName));
				holder.setNombreCampo(columnName);
			} catch (java.sql.SQLException sqle) {
				throw new RuntimeException(sqle);
			}
			if (functions != null) {
				sb.append(replace(functions, columnName));
			} else {
				sb.append(columnName);
			}
			if (searchValue == null || searchValue.toString().trim().equals("")) {
				if (operator.equals("=")) {
					sb.append(SPACE).append("IS NULL").append(SPACE);
				} else if (operator.equals("<>") || operator.equals("!=")) {
					sb.append(SPACE).append("IS NOT NULL").append(SPACE);
				} else {
					throw new IllegalArgumentException(
							"Operador invalido para el valor a buscar");
				}
			} else {
				sb.append(SPACE).append(operator).append(SPACE);
				sb.append("?");
				this.arguments.add(searchValue);
			}
			if (count < size) {
				sb.append(SPACE).append(booleanCondition).append("\n")
						.append(SPACE);
			} else {
				if (booleanCondition
						.equals(ManejadorCriterioBusqueda.OR_NESTED_LOGIC)
						|| booleanCondition
								.equals(ManejadorCriterioBusqueda.AND_NESTED_LOGIC)
						|| booleanCondition
								.equals(ManejadorCriterioBusqueda.AND_NESTED_LOGIC_END)
						|| booleanCondition
								.equals(ManejadorCriterioBusqueda.OR_NESTED_LOGIC_END)
						|| booleanCondition
								.equals(ManejadorCriterioBusqueda.NESTED_LOGIC_END)) {
					sb.append(SPACE).append(") ");
				} else {
				}
			}
		}
		if (searchCriteriaHolders.size() != 0) {
		} else {
			throw new RuntimeException("No search criteria");
		}
		setDynamicWhere(sb.toString());
		return dynamicWhere;
	}

	public String setSort(List sort) {
		return setSort(sort, true, true);
	}

	public String setSort(List sort, boolean findMapping) {
		return setSort(sort, findMapping, true);
	}

	public String setSort(List sort, boolean findMapping, boolean replace) {
		if (sort.size() == 0) {
			return orderBy;
		}
		StringBuffer sb = new StringBuffer(" ORDER BY ");
		if (!replace) {
			if (orderBy != null && !orderBy.trim().equals("")) {
				sb.append(orderBy).append(", ");
			}
		}
		Iterator iter = sort.iterator();
		while (iter.hasNext()) {
			String[] fieldNameAndSortDir = (String[]) iter.next();
			String columnName;
			if (findMapping) {
				try {
					columnName = metadatos
							.getNombreColumnaCompleto(metadatos
									.getIndiceColumnaPorNombreCampo(fieldNameAndSortDir[0]));
				} catch (java.sql.SQLException sqle) {
					throw new RuntimeException(sqle);
				}
			} else {
				columnName = fieldNameAndSortDir[0];
			}
			sb.append(columnName);
			if (fieldNameAndSortDir[1] != null) {
				sb.append(SPACE).append(fieldNameAndSortDir[1]);
			}
			sb.append(", ");
		}
		int len = sb.length();
		sb.delete(len - 2, len - 1);
		orderBy = sb.toString();
		return orderBy;
	}

	public void addSearch() {
		this.searchCriteriaHolders.add(new ManejadorCriterioBusqueda());
	}

	public void removeSearch(int idx) {
		try {
			searchCriteriaHolders.remove(idx);
		} catch (IndexOutOfBoundsException e) {
			throw new IllegalArgumentException(e.getMessage());
		}
	}

	public int size() {
		return searchCriteriaHolders.size();
	}

	public java.util.List getFields() {
		return fields;
	}

	public void setFields(java.util.List fields) {
		this.fields = fields;
	}

	public java.util.List getOperatorsList() {
		return operatorsList;
	}

	public void setOperatorsList(java.util.List operatorsList) {
		this.operatorsList = operatorsList;
	}

	public java.util.Map getFieldOperators() {
		return fieldOperators;
	}

	public void setFieldOperators(java.util.Map fieldOperators) {
		this.fieldOperators = fieldOperators;
	}

	public java.util.Map getFieldValues() {
		return fieldValues;
	}

	public void setFieldValues(java.util.Map fieldValues) {
		this.fieldValues = fieldValues;
	}

	public java.util.List getSearchCriteriaHolders() {
		return searchCriteriaHolders;
	}

	public void setManejadorCriterioBusqueda(
			java.util.List searchCriteriaHolders) {
		this.searchCriteriaHolders = searchCriteriaHolders;
	}

	public java.util.List getArgumentos() {
		return arguments;
	}

	public void setArguments(java.util.List arguments) {
		this.arguments = arguments;
	}

	public slgfp.Metadatos getMetaData() {
		return metadatos;
	}

	public void setMetaData(slgfp.Metadatos metadatos) {
		this.metadatos = metadatos;
	}

	public java.lang.String getSelect() {
		return select;
	}

	public void setSelect(java.lang.String select) {
		if (select != null) {
			this.select = select.trim();
		}
	}

	public java.lang.String getFrom() {
		return from;
	}

	public void setFrom(java.lang.String from) {
		if (from != null) {
			this.from = from.trim();
		}
	}

	public java.lang.String getJoins() {
		return joins;
	}

	public void setJoins(java.lang.String joins) {
		if (joins != null) {
			this.joins = joins.trim();
		}
	}

	public java.lang.String getWhere() {
		return where;
	}

	public void setWhere(java.lang.String where) {
		if (where != null) {
			this.where = where.trim();
		}
	}

	public java.lang.String getGroupBy() {
		return groupBy;
	}

	public void setGroupBy(java.lang.String groupBy) {
		if (groupBy != null) {
			this.groupBy = groupBy.trim();
		}
	}

	public java.lang.String getHaving() {
		return having;
	}

	public void setHaving(java.lang.String having) {
		if (having != null) {
			this.having = having.trim();
		}
	}

	public java.lang.String getOrderBy() {
		return orderBy;
	}

	public void setOrderBy(java.lang.String orderBy) {
		if (orderBy != null) {
			this.orderBy = orderBy.trim();
		}
	}

	public java.lang.String getConsulta() {
		StringBuffer sb = new StringBuffer();
		if (select.trim().toUpperCase().startsWith("SELECT")) {
			sb.append(select);
		} else {
			sb.append("SELECT ").append(select);
		}
		if (!from.trim().equals("")) {
			if (!from.toUpperCase().startsWith("FROM")) {
				sb.append(" \nFROM ").append(from.trim());
			} else {
				sb.append(SPACE).append(from.trim());
			}
		}
		if (!joins.trim().equals("")) {
			sb.append(SPACE).append(joins.trim());
		}
		if (!where.equals("")) {
			if (!where.trim().toUpperCase().startsWith("WHERE")) {
				sb.append(" \nWHERE ").append(where.trim());
			} else {
				sb.append(SPACE).append(where.trim());
			}
		}

		if (!dynamicWhere.trim().equals("")) {
			if (where != null && !where.trim().equals("")) {
				sb.append(" AND\n (").append(dynamicWhere.trim());
			} else {
				sb.append(" \nWHERE (").append(dynamicWhere.trim());
			}
			sb.append(")");
		}

		if (!groupBy.equals("")) {
			if (!groupBy.trim().toUpperCase().startsWith("GROUP ")) {
				sb.append(" \nGROUP BY ").append(groupBy.trim());
			} else {
				sb.append(SPACE).append(groupBy.trim());
			}
		}
		if (!having.equals("")) {
			if (!having.trim().toUpperCase().startsWith("HAVING")) {
				sb.append(" \nHAVING ").append(having.trim());
			} else {
				sb.append(SPACE).append(having.trim());
			}
		}
		if (!orderBy.equals("")) {
			if (!orderBy.trim().toUpperCase().startsWith("ORDER ")) {
				sb.append(" \nORDER BY ").append(orderBy.trim());
			} else {
				sb.append(SPACE).append(orderBy.trim());
			}
		}
		query = sb.toString();
		return query;
	}

	public void setQuery(java.lang.String query) {
		this.query = query;
	}

	public java.util.Map getLikeHint() {
		return likeHint;
	}

	public int getLikeHint(String operatorName) {
		return ((Integer) likeHint.get(operatorName)).intValue();
	}

	public void setLikeHint(String operatorName, int likeHint) {
		this.likeHint.put(operatorName, new Integer(likeHint));
	}

	public void setLikeHint(java.util.Map likeHint) {
		this.likeHint = likeHint;
	}

	public java.lang.String getDynamicWhere() {
		return dynamicWhere;
	}

	public void setDynamicWhere(java.lang.String dynamicWhere) {
		if (where != null) {
			this.dynamicWhere = dynamicWhere.trim();
		}
	}

	protected String replace(String functions, String columnName) {
		int start = functions.indexOf("${");
		int end = functions.indexOf("}", start);
		String result = functions.substring(0, start);
		result = result + columnName + functions.substring(end + 1);
		return result;
	}

	public String getEjecutable() {
		return ejecutable;
	}

	public void setEjecutable(String executable) {
		this.ejecutable = executable;
	}

	public String toString() {
		Object[] EMPTY_READ_ARG = new Object[0];
		StringBuffer sb = new StringBuffer();
		Object value = null;
		Method[] methods = getClass().getMethods();
		for (int i = 0; i < methods.length; i++) {
			String methodName = methods[i].getName();
			if (methodName.equals("") || methodName.equals("getClass")) {
				continue;
			}
			if ((methodName.startsWith("get") || methodName.startsWith("is"))
					&& methods[i].getParameterTypes().length == 0) {
				try {
					value = methods[i].invoke(this, EMPTY_READ_ARG);
				} catch (Throwable t) {
					continue;
				}
				String fieldFirstChar = "";
				if (methodName.startsWith("is")) {
					fieldFirstChar = methodName.substring(2, 3).toLowerCase();
					sb.append(fieldFirstChar);
					sb.append(methodName.substring(3));
				} else if (methodName.startsWith("get")) {
					fieldFirstChar = methodName.substring(3, 4).toLowerCase();
					sb.append(fieldFirstChar);
					sb.append(methodName.substring(4));
				}
				sb.append("=");
				sb.append((value == null) ? "" : value);
				sb.append("&");
			}
		}
		return sb.toString();
	}
}
