package slgfp.buscar;

import java.io.Reader;
import java.net.URL;

/**
 * La clase nos sirve para extender la interfaz GeneradorCriteriosBusqueda y,
 * as, realizar lo necesario para construir consultas a la base de datos
 * 
 * @version 1.0, 03-05-2011
 * @author Sergio Lovillo
 */

public class GeneradorCriteriosBusquedaBasica extends
		GeneradorCriteriosBusqueda {

	protected XRutaSQLLectorConsultas lectorConsultas = new XRutaSQLLectorConsultas();

	public GeneradorCriteriosBusquedaBasica() {
	}

	public GeneradorCriteriosBusquedaBasica(URL consultaURL) {
		lectorConsultas.setConsultaURL(consultaURL);
	}

	public GeneradorCriteriosBusquedaBasica(String consultaArchivoRuta) {
		lectorConsultas.setConsultaArchivoRuta(consultaArchivoRuta);
	}

	public GeneradorCriteriosBusquedaBasica(Reader lector) {
		lectorConsultas.setLector(lector);
	}

	public void cargarConsulta(String consultaId, boolean recargaDocumento) {
		resetear();
		lectorConsultas.cargarConsulta(consultaId, recargaDocumento);
	}

	public void cargarConsulta(String consultaId) {
		this.cargarConsulta(consultaId, true);
	}

	public void cargarEjecutable(String consultaId) {
		this.cargarEjecutable(consultaId, true);
	}

	public void cargarEjecutable(String consultaId, boolean recargarDocumento) {
		resetear();
		lectorConsultas.cargarEjecutable(consultaId, recargarDocumento);
	}

	public String getEjecutable() {
		this.ejecutable = lectorConsultas.getEjecutable();
		return ejecutable;
	}

	public void setEjecutable(String ejecutable) {
		this.ejecutable = ejecutable;
	}

	public XRutaSQLLectorConsultas getLectorConsulta() {
		return lectorConsultas;
	}

	public void setLectorConsulta(XRutaSQLLectorConsultas lectorConsulta) {
		this.lectorConsultas = lectorConsulta;
	}

	public String getConsultaArchivoRuta() {
		return lectorConsultas.getConsultaArchivoRuta();
	}

	public void setConsultaArchivoRuta(String consultaArchivoRuta) {
		lectorConsultas.setConsultaArchivoRuta(consultaArchivoRuta);
	}

	public URL getConsultaURL() {
		return lectorConsultas.getConsultaURL();
	}

	public void setQueryURL(URL queryURL) {
		lectorConsultas.setConsultaURL(queryURL);
	}

	public Reader getLector() {
		return lectorConsultas.getLector();
	}

	public void setLector(Reader lector) {
		lectorConsultas.setLector(lector);
	}

	public String getConsulta() {
		try {
			lectorConsultas.setAdhocColumnas(adhocColumnas);
			setSelect(lectorConsultas.getSelect());
			setFrom(lectorConsultas.getFrom());
			setJoins(lectorConsultas.getJoins());
			setWhere(lectorConsultas.getWhere());
			setGroupBy(lectorConsultas.getGroupBy());
			setHaving(lectorConsultas.getHaving());
			if (this.orderBy == null || this.orderBy.trim().length() == 0) {
				if (lectorConsultas.getOrderBy() != null
						&& lectorConsultas.getOrderBy().trim().length() > 0) {
					setOrderBy(lectorConsultas.getOrderBy());
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		}
		return super.getConsulta();
	}

}
