package slgfp.buscar;

import java.util.*;

/**
 * La clase nos sirve para realizar lo necesario para gestionar las consultas a
 * la base de datos
 * 
 * @version 1.0, 03-05-2011
 * @author Sergio Lovillo
 */

@SuppressWarnings({ "serial", "rawtypes" })
public class ManejadorCriterioBusqueda implements java.io.Serializable,
		Cloneable {

	public ManejadorCriterioBusqueda() {
	}

	protected String etiquetaCampo = null;
	protected String nombreCampo = null;
	protected String operador = "=";
	protected String funciones = null;
	protected Object valorBuscado = null;
	protected String condicionBoolean = AND;

	public static final String LESS = "<";
	public static final String LESS_OR_EQUALS = "<=";
	public static final String EQUALS = "=";
	public static final String LIKE = "LIKE";
	public static final String NOT_LIKE = "NOT LIKE";
	public static final String IN = "IN";
	public static final String NOT_IN = "NOT IN";
	public static final String NOT_EQUALS = "<>";
	public static final String NOT_EXISTS = "NOT EXISTS";
	public static final String EXISTS = "EXISTS";
	public static final String BETWEEN = "BETWEEN";
	public static final String NOT_BETWEEN = "NOT BETWEEN";
	public static final String IS = "IS";
	public static final String IS_NOT = "IS NOT";
	public static final String GREATER_OR_EQUALS = ">=";
	public static final String GREATER = ">";

	public static final String AND = "AND";
	public static final String OR = "OR";
	public static final String AND_NESTED_LOGIC_START = "AND (";
	public static final String OR_NESTED_LOGIC_START = "OR (";
	public static final String AND_NESTED_LOGIC_END = ") AND";
	public static final String OR_NESTED_LOGIC_END = ") OR";
	public static final String AND_NESTED_LOGIC = ") AND (";
	public static final String OR_NESTED_LOGIC = ") OR (";
	public static final String NESTED_LOGIC_END = ")";

	// Parmetros de apoyo al comando LIKE
	public static final int EMPEZAR_CON = 0;
	public static final int CONTENEDOR = 1;
	public static final int TERMINA_CON = 2;
	// Hint por defecto
	protected int comoSugerencia = EMPEZAR_CON;

	protected String operadores = "<=NOT LIKE!=NOT IN<>NOT EXISTSNOT BETWEEN IS NOT >=";
	protected Map mapBuscadorValores = null;
	protected List listaBuscadorValores = null;

	public java.lang.String getNombreCampo() {
		return nombreCampo;
	}

	public void setNombreCampo(java.lang.String nombreCampo) {
		this.nombreCampo = nombreCampo;
	}

	public java.lang.String getOperator() {
		return operador;
	}

	public void setOperator(java.lang.String operator) {
		if (operadores.indexOf(operator.toUpperCase()) == -1) {
			throw new IllegalArgumentException("Operador no valido: "
					+ operator);
		}
		this.operador = operator;
	}

	public java.lang.Object getValorBuscado() {
		return valorBuscado;
	}

	public void setValorBuscado(java.lang.Object valorBuscado) {
		this.valorBuscado = valorBuscado;
	}

	public java.lang.String getCondicionBoolean() {
		return condicionBoolean;
	}

	public void setCondicionBoolean(java.lang.String condicionBoolean) {
		if (condicionBoolean == null) {
			throw new IllegalArgumentException(
					"Condicin buleana no valida. (AND/OR)");
		}
		String tmp = condicionBoolean.toUpperCase();
		if (condicionBoolean.equalsIgnoreCase(AND) || tmp.equals(OR)
				|| tmp.equals(AND_NESTED_LOGIC_START)
				|| tmp.equals(AND_NESTED_LOGIC_END)
				|| tmp.equals(OR_NESTED_LOGIC_START)
				|| tmp.equals(NESTED_LOGIC_END)
				|| tmp.equals(OR_NESTED_LOGIC_END)
				|| tmp.equals(OR_NESTED_LOGIC) || tmp.equals(AND_NESTED_LOGIC)) {
		} else {
			throw new IllegalArgumentException(
					"Condicion buleana no valida: \n\n" + condicionBoolean
							+ " \n\nSolo se puede usar AND/OR");
		}
		this.condicionBoolean = condicionBoolean;
	}

	public String toString() {
		StringBuffer sb = new StringBuffer();
		String espacio = " ";
		if (funciones != null) {
			sb.append(funciones.replaceFirst("${}", nombreCampo));
		} else {
			sb.append(nombreCampo);
		}
		sb.append(espacio).append(operador).append(espacio);
		sb.append(valorBuscado);
		sb.append(espacio).append(condicionBoolean).append(espacio);

		return sb.toString();
	}

	public java.util.Map getMapBuscadorValores() {
		return mapBuscadorValores;
	}

	public void setMapBuscadorValores(java.util.Map mapBuscadorValores) {
		this.mapBuscadorValores = mapBuscadorValores;
	}

	public java.lang.String getEtiquetaCampo() {
		return etiquetaCampo;
	}

	public void setEtiquetaCampo(java.lang.String etiquetaCampo) {
		this.etiquetaCampo = etiquetaCampo;
	}

	public java.util.List getListaBuscadorValores() {
		return listaBuscadorValores;
	}

	public void setListaBuscadorValores(java.util.List listaBuscadorValores) {
		this.listaBuscadorValores = listaBuscadorValores;
	}

	public int getComoSugerencia() {
		return comoSugerencia;
	}

	public void setComoSugerencia(int comoSugerencia) {
		this.comoSugerencia = comoSugerencia;
	}

	public java.lang.String getFunciones() {
		return this.funciones;
	}

	/**
	 * Un ejemplo de uso podra ser: ManejadorCriterioBusqueda manejador = new
	 * ManejadorCriterioBusqueda(); ...
	 * manejador.setFunctions("CONCAT(UPPER(SUBSTR(${}, 10)), 'XYZ')" ); ... el
	 * resultado sera: TESTXYZ
	 */
	public void setFunciones(java.lang.String funciones) {
		this.funciones = funciones;
	}
}
