package slgfp.buscar;

import java.io.*;
import java.lang.reflect.Method;
import java.util.*;
import org.jdom.*;
import org.jdom.input.*;
import org.jdom.xpath.*;
import java.net.URL;

/**
 * La clase nos sirve para realizar lo necesario para construir consultas a la
 * base de datos usando objetox SAX.
 * 
 * @version 1.0, 03-05-2011
 * @author Sergio Lovillo
 */

@SuppressWarnings({ "serial", "rawtypes", "unchecked" })
public class XRutaSQLLectorConsultas implements java.io.Serializable, Cloneable {

	protected String select;
	protected String from;
	protected String where;
	protected String joins;
	protected String groupBy;
	protected String having;
	protected String orderBy;
	protected String consulta;
	protected String queryId = "";
	protected String consultaArchivoRuta;
	protected String ejecutable;
	protected URL consultaURL;
	protected Reader lector;
	protected Document doc;
	protected boolean recargarDocumento = false;
	protected boolean distinct = false;
	protected boolean ansiJoin = false;
	// Si se usa una JOIN que requera aadir tablas
	protected Map columnaCondicionMapeo;
	protected Map columnaTablasMapeo;
	// Para listar las columnas adhoc
	protected Set adhocColumnas;
	protected Element ansiJoinElement;
	protected Element selectElement;
	protected Element fromElement;
	protected Element joinsElement;
	protected Element whereElement;
	protected Element groupByElement;
	protected Element havingElement;
	protected Element orderByElement;
	protected Element ejecutableElemento;
	protected boolean debug;

	public XRutaSQLLectorConsultas() {
		if (Boolean.getBoolean("debug-slgfp")
				|| Boolean.getBoolean("debug-" + getClass().getName())) {
			debug = true;
		}
	}

	public void resetear() {
		if (ansiJoinElement != null) {
			ansiJoin = true;
		} else {
			ansiJoin = false;
		}
		String a = selectElement.getAttributeValue("distinct");
		if (a != null && a.equals("true")) {
			distinct = true;
		} else {
			distinct = false;
		}
		select = selectElement.getText();
		from = fromElement.getText();
		joins = joinsElement.getText();
		where = whereElement.getText();
		groupBy = groupByElement.getText();
		having = havingElement.getText();
		orderBy = orderByElement.getText();
		if (columnaTablasMapeo != null)
			columnaTablasMapeo.clear();
		if (columnaCondicionMapeo != null)
			columnaCondicionMapeo.clear();
		adhocColumnas = null;
	}

	public void resetearTodo() {
		from = null;
		where = null;
		joins = null;
		groupBy = null;
		having = null;
		orderBy = null;
		queryId = "";
		distinct = false;
		ansiJoin = false;
		if (columnaCondicionMapeo != null)
			columnaCondicionMapeo.clear();
		if (columnaTablasMapeo != null)
			columnaTablasMapeo.clear();
		adhocColumnas = null;
		ejecutable = null;
	}

	public void cargarEjecutable(String q) {
		if (debug)
			System.out.println("slgfp ============= " + new java.util.Date()
					+ " " + this.getClass() + "::loadExecutable()::queryId: "
					+ q + " \n");
		if (!this.queryId.equals(q)) {
			recargarDocumento = true;
		}
		if (recargarDocumento) {
			resetearTodo();
		}
		if (this.queryId.equals(q)) {
			return;
		}

		this.queryId = q;
		try {
			SAXBuilder b = new SAXBuilder();
			if (doc == null || recargarDocumento) {
				if (consultaArchivoRuta != null) {
					doc = b.build(new File(consultaArchivoRuta));
				} else if (consultaURL != null) {
					doc = b.build(consultaURL);
				} else if (lector != null) {
					doc = b.build(lector);
				} else {
					throw new IOException("Falta la consulta");
				}
			}
			ejecutableElemento = (Element) XPath.newInstance(
					"//ejecutable[@id='" + q + "']").selectSingleNode(doc);
			ejecutable = ejecutableElemento.getText();
		} catch (IOException ioe) {
			throw new RuntimeException(ioe);
		} catch (JDOMException jdome) {
			throw new RuntimeException(jdome);
		}
	}

	public void cargarEjecutable(String consultaId, boolean recargarDocumento) {
		this.recargarDocumento = recargarDocumento;
		this.cargarEjecutable(consultaId);
	}

	public void cargarConsulta(String consultaId, boolean recargarDocumento) {
		this.recargarDocumento = recargarDocumento;
		this.cargarConsulta(consultaId);
	}

	public void cargarConsulta(String consultaId) {
		if (debug)
			System.out.println("slgfp ============= " + new java.util.Date()
					+ " " + this.getClass()
					+ "::cargarConsulta()::consultaId: " + consultaId + " \n");
		if (!this.queryId.equals(consultaId)) {
			resetearTodo();
		}
		if (recargarDocumento) {
			resetearTodo();
		}
		if (this.queryId.equals(consultaId)) {
			resetear();
			return;
		}

		this.queryId = consultaId;
		try {
			SAXBuilder b = new SAXBuilder();
			if (doc == null || recargarDocumento) {
				if (consultaArchivoRuta != null) {
					doc = b.build(new File(consultaArchivoRuta));
				} else if (consultaURL != null) {
					doc = b.build(consultaURL);
				} else if (lector != null) {
					doc = b.build(lector);
				} else {
					throw new IOException("Falta la consulta");
				}
			}
			Element consultaElemento = (Element) XPath.newInstance(
					"//consulta[@id='" + consultaId + "']").selectSingleNode(
					doc);
			if (consultaElemento == null) {
				throw new IllegalArgumentException("Consulta Id: \""
						+ consultaId + "\" no existe");
			}

			ansiJoinElement = (Element) XPath.newInstance(
					"//consulta[@id='" + consultaId + "']/ansi_join")
					.selectSingleNode(doc);
			if (ansiJoinElement != null) {
				ansiJoin = true;
			} else {
				ansiJoin = false;
			}
			selectElement = (Element) XPath.newInstance(
					"//consulta[@id='" + consultaId + "']/select")
					.selectSingleNode(doc);
			String a = selectElement.getAttributeValue("distinct");
			if (a != null && a.equals("true")) {
				distinct = true;
			} else {
				distinct = false;
			}
			select = selectElement.getText();

			fromElement = (Element) XPath.newInstance(
					"//consulta[@id='" + consultaId + "']/from")
					.selectSingleNode(doc);
			if (fromElement != null) {
				from = fromElement.getText();
			}

			if (isAnsiJoin()) {
				joinsElement = (Element) XPath.newInstance(
						"//consulta[@id='" + consultaId + "']/joins")
						.selectSingleNode(doc);
				joins = joinsElement.getText();
			}

			whereElement = (Element) XPath.newInstance(
					"//consulta[@id='" + consultaId + "']/where")
					.selectSingleNode(doc);
			if (whereElement != null) {
				where = whereElement.getText();
			}

			groupByElement = (Element) XPath.newInstance(
					"//consulta[@id='" + consultaId + "']/group_by")
					.selectSingleNode(doc);
			if (groupByElement != null) {
				groupBy = groupByElement.getText();
			}

			havingElement = (Element) XPath.newInstance(
					"//consulta[@id='" + consultaId + "']/having")
					.selectSingleNode(doc);
			if (havingElement != null) {
				having = havingElement.getText();
			}
			orderByElement = (Element) XPath.newInstance(
					"//consulta[@id='" + consultaId + "']/order_by")
					.selectSingleNode(doc);
			if (orderByElement != null) {
				orderBy = orderByElement.getText();
			}
			List adhocMapeo = XPath.newInstance(
					"//consulta[@id='" + consultaId + "']/adhoc_mappings/*")
					.selectNodes(doc);
			if (adhocMapeo != null && !adhocMapeo.isEmpty()) {
				columnaCondicionMapeo = new HashMap();
				columnaTablasMapeo = new HashMap();
				Iterator it = adhocMapeo.iterator();
				while (it.hasNext()) {
					Element adhocElement = (Element) it.next();
					Element condicionesElement = adhocElement
							.getChild("conditions");
					String conditions = condicionesElement.getText();
					Element columnasElement = adhocElement.getChild("columns");
					List columnas = columnasElement.getChildren();
					Iterator columnsIter = columnas.iterator();
					while (columnsIter.hasNext()) {
						String columna = ((Element) columnsIter.next())
								.getText();
						columnaCondicionMapeo.put(columna, conditions);
						if (!isAnsiJoin()) {
							Element adhocTablesElement = adhocElement
									.getChild("tablas");
							if (adhocTablesElement != null) {
								String tables = adhocTablesElement.getText();
								columnaTablasMapeo.put(columna, tables);
							}
						}
					}
				}
			}
		} catch (IOException ioe) {
			ioe.printStackTrace();
			throw new RuntimeException(ioe);
		} catch (JDOMException jdome) {
			jdome.printStackTrace();
			throw new RuntimeException(jdome);
		}
	}

	public java.net.URL getConsultaURL() {
		return consultaURL;
	}

	public void setConsultaURL(java.net.URL queryURL) {
		if (debug)
			System.out.println("slgfp ============= " + new java.util.Date()
					+ " " + this.getClass()
					+ "::setConsultaURL()::consultaURL: " + queryURL + " \n");
		if (this.consultaURL == null || !this.consultaURL.equals(queryURL)) {
			setRecargaDocumento(true);
		}
		this.consultaArchivoRuta = null;
		this.lector = null;
		this.consultaURL = queryURL;
	}

	public java.lang.String getConsultaArchivoRuta() {
		return consultaArchivoRuta;
	}

	public void setConsultaArchivoRuta(java.lang.String consultaArchivoRuta) {
		if (debug)
			System.out.println("slgfp ============= " + new java.util.Date()
					+ " " + this.getClass()
					+ "::setConsultaArchivoRuta()::consultaArchivoRuta: "
					+ consultaArchivoRuta + " \n");
		if (this.consultaArchivoRuta == null
				|| !this.consultaArchivoRuta.equals(consultaArchivoRuta)) {
			setRecargaDocumento(true);
		}
		this.consultaURL = null;
		this.lector = null;
		this.consultaArchivoRuta = consultaArchivoRuta;
	}

	public Reader getLector() {
		return lector;
	}

	public void setLector(Reader lector) {
		if (debug)
			System.out.println("slgfp ============= " + new java.util.Date()
					+ " " + this.getClass() + "::setLector()::lector: "
					+ lector + " \n");
		if (this.lector == null || !this.lector.equals(lector)) {
			setRecargaDocumento(true);
		}
		this.consultaURL = null;
		this.consultaArchivoRuta = null;
		this.lector = lector;
	}

	public String getGroupBy() {
		return (groupBy == null ? "" : groupBy);
	}

	public void setGroupBy(String groupBy) {
		this.groupBy = groupBy;
	}

	public String getOrderBy() {
		return (orderBy == null ? "" : orderBy);
	}

	public void setOrderBy(String orderBy) {
		this.orderBy = orderBy;
	}

	public String getWhere() {
		if (adhocColumnas != null) {
			if (!isAnsiJoin()) {
				Set columnasCondicion = new HashSet();
				Iterator it = adhocColumnas.iterator();
				while (it.hasNext()) {
					String columna = (String) it.next();
					String codiciones = getColumnaCondicionMapeo(columna);
					if (codiciones != null && !codiciones.trim().equals("")) {
						columnasCondicion.add(codiciones.trim());
					}
				}
				StringBuffer sb3 = new StringBuffer();
				Iterator it2 = columnasCondicion.iterator();
				while (it2.hasNext()) {
					String value = (String) it2.next();
					if (value != null && !value.trim().equals("")) {
						sb3.append(value).append(" AND\n ");
					}
				}
				if (sb3.length() > 0) {
					sb3.setLength(sb3.length() - 5);
				}
				if (where == null || where.trim().equals("")) {
					if (sb3.length() > 0) {
						where = sb3.toString().trim();
					}
				} else {
					if (sb3.length() > 0) {
						where = where.trim() + " AND\n "
								+ sb3.toString().trim();
					} else {
					}
				}
			}
		}
		return (where == null ? "" : where.trim());
	}

	public void setWhere(String where) {
		this.where = where;
	}

	public String getHaving() {
		return (having == null ? "" : having);
	}

	public void setHaving(String having) {
		this.having = having;
	}

	public String getFrom() {
		if (adhocColumnas != null) {
			StringBuffer sb1 = new StringBuffer();
			if (from != null) {
				sb1.append(from);
			}
			Iterator it = adhocColumnas.iterator();
			while (it.hasNext()) {
				Object adhocTables = getColumnaTablasMapeo((String) it.next());
				if (adhocTables != null) {
					sb1.append(", ").append(adhocTables);
				}
			}
			StringTokenizer st = new StringTokenizer(sb1.toString(), ",", false);
			Set tablaNombres = new HashSet();
			while (st.hasMoreTokens()) {
				String tablaNombre = st.nextToken();
				tablaNombres.add(tablaNombre.trim());
			}
			StringBuffer sb2 = new StringBuffer();
			Iterator it3 = tablaNombres.iterator();
			while (it3.hasNext()) {
				sb2.append(it3.next()).append(", ");
			}
			int idx = sb2.lastIndexOf(",");
			if (idx > -1) {
				sb2.deleteCharAt(idx);
				from = sb2.toString();
			}
		}
		return (from == null ? "" : from);
	}

	public void setFrom(String from) {
		this.from = from;
	}

	public boolean isRecargarDocumento() {
		return recargarDocumento;
	}

	public void setRecargaDocumento(boolean recargarDocumento) {
		this.recargarDocumento = recargarDocumento;
	}

	public boolean isDistinct() {
		return distinct;
	}

	public void setDistinct(boolean distinct) {
		this.distinct = distinct;
	}

	public String getJoins() {
		if (adhocColumnas != null) {
			if (isAnsiJoin()) {
				Set columnasCondicion = new HashSet();
				Iterator it = adhocColumnas.iterator();
				while (it.hasNext()) {
					String columna = (String) it.next();
					String condicion = getColumnaCondicionMapeo(columna);
					if (condicion != null && !condicion.trim().equals("")) {
						columnasCondicion.add(condicion.trim());
					}
				}
				StringBuffer sb3 = new StringBuffer();
				Iterator it2 = columnasCondicion.iterator();
				while (it2.hasNext()) {
					String value = (String) it2.next();
					if (value != null && !value.trim().equals("")) {
						sb3.append(value.trim()).append(" AND\n ");
					}
				}
				if (sb3.length() > 0) {
					sb3.setLength(sb3.length() - 5);
				}
				if (joins == null || joins.trim().equals("")) {
					joins = sb3.toString();
				} else {
					joins = joins.trim() + " " + sb3.toString();
				}
			}
		}
		return (joins == null ? "" : joins);
	}

	public void setJoins(String joins) {
		this.joins = joins;
	}

	public void setConsulta(java.lang.String consulta) {
		this.consulta = consulta;
	}

	public String getConsulta() {
		return this.consulta;
	}

	public java.lang.String getSelect() {
		if (isDistinct()) {
			return " DISTINCT " + select;
		}
		return select;
	}

	public void setSelect(java.lang.String select) {
		this.select = select;
	}

	public boolean isAnsiJoin() {
		return ansiJoin;
	}

	public void setAnsiJoin(boolean ansiJoin) {
		this.ansiJoin = ansiJoin;
	}

	public java.util.Map getColumnaCondicionMapeo() {
		return columnaCondicionMapeo;
	}

	public String getColumnaCondicionMapeo(String nombreColumna) {
		return (columnaCondicionMapeo.containsKey(nombreColumna) ? (String) columnaCondicionMapeo
				.get(nombreColumna) : "");
	}

	public void setColumnaCondicionMapeo(java.util.Map columnaCondicionMapeo) {
		this.columnaCondicionMapeo = columnaCondicionMapeo;
	}

	public void setColumnaCondicionMapeo(String nombreColumna, String condicion) {
		this.columnaCondicionMapeo.put(nombreColumna, condicion);
	}

	public java.util.Map getColumnaTablasMapeo() {
		return columnaTablasMapeo;
	}

	public String getColumnaTablasMapeo(String nombreColumna) {
		return (String) columnaTablasMapeo.get(nombreColumna);
	}

	public void setColumnaTablasMapeo(java.util.Map columnaTablasMapeo) {
		this.columnaTablasMapeo = columnaTablasMapeo;
	}

	public void setColumnaTablasMapeo(String nombreColumna, String tablas) {
		this.columnaTablasMapeo.put(nombreColumna, tablas);
	}

	public java.util.Set getAdhocColumnas() {
		return adhocColumnas;
	}

	public void setAdhocColumnas(java.util.Set adhocColumnas) {
		this.adhocColumnas = adhocColumnas;
	}

	public Element getEjecutableElemento() {
		return ejecutableElemento;
	}

	public void setEjecutableElemento(Element ejecutableElemento) {
		this.ejecutableElemento = ejecutableElemento;
	}

	public String getEjecutable() {
		return ejecutable;
	}

	public void setEjecutable(String ejecutable) {
		this.ejecutable = ejecutable;
	}

	public boolean isDebug() {
		return debug;
	}

	public void setDebug(boolean debug) {
		this.debug = debug;
	}

	public String toString() {
		Object[] VACIO_LECTURA_ARG = new Object[0];
		StringBuffer sb = new StringBuffer();
		Object valor = null;
		Method[] metodos = getClass().getMethods();
		for (int i = 0; i < metodos.length; i++) {
			String nombreMetodo = metodos[i].getName();
			if (nombreMetodo.equals("") || nombreMetodo.equals("getClass")) {
				continue;
			}
			if ((nombreMetodo.startsWith("get") || nombreMetodo
					.startsWith("is"))
					&& metodos[i].getParameterTypes().length == 0) {
				try {
					valor = metodos[i].invoke(this, VACIO_LECTURA_ARG);
				} catch (Throwable t) {
					continue;
				}
				String primerCaracterCampo = "";
				if (nombreMetodo.startsWith("is")) {
					primerCaracterCampo = nombreMetodo.substring(2, 3)
							.toLowerCase();
					sb.append(primerCaracterCampo);
					sb.append(nombreMetodo.substring(3));
				} else if (nombreMetodo.startsWith("get")) {
					primerCaracterCampo = nombreMetodo.substring(3, 4)
							.toLowerCase();
					sb.append(primerCaracterCampo);
					sb.append(nombreMetodo.substring(4));
				}
				sb.append("=");
				sb.append((valor == null) ? "" : valor);
				sb.append("&");
			}
		}
		return sb.toString();
	}
}
