package slgfp.pruebas;

import java.util.*;
import slgfp.*;
import slgfp.buscar.*;
import java.sql.*;

/**
 * Nos sirve de base para recuperar los registros de la tabla CUSTOMER de la
 * base de datos preparndonos para definir las estructuras necesarias.
 * 
 * @version 1.0, 03-05-2011
 * @author Sergio Lovillo
 */

@SuppressWarnings({ "serial", "rawtypes", "unchecked" })
public class FabricaDeClientes extends slgfp.FabricaObjPersistencia implements
		java.io.Serializable, Cloneable {

	// El objeto principal conexion las asignaciones de CAMPO_OBJETO y
	// CAMPO_TABLA
	protected Properties sqlMap = null;

	// Constructor principal sin parmetros
	public FabricaDeClientes() {
		/*
		 * No es necesario cargar las asignaciones de esta forma, podra ser
		 * cualquier cosa: XML, JNDI, bases de datos, etc..
		 */
		sqlMap = cargaAsignaciones("Clientes.sql");
		iniciaFabrica();
		setSolicitante(Clientes.class);
	}

	// Inicializamos el acceso a la base de datos, el interfaz
	protected void iniciaFabrica() {
		try {
			DataSourceBasicoImpl ds = new DataSourceBasicoImpl();
			ds.setNombreControlador("org.hsqldb.jdbcDriver");
			ds.setDbURL("jdbc:hsqldb:hsql://localhost");
			ds.setNombreUsuario("sa");
			ds.setPassword("");
			this.dbServicios = new DBServicios();
			dbServicios.setDataSource(ds);
			// Fichero conexion las asignaciones de tipo PROPERTIES
			setAsignaciones(cargaAsignaciones("Clientes.propiedades"));
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public Properties cargaAsignaciones(String path) {
		java.io.InputStream inStream = null;
		Properties props = new Properties();
		try {
			inStream = this.getClass().getResourceAsStream(path);
			props.load(inStream);
		} catch (java.io.IOException ioe) {
			throw new RuntimeException(ioe);
		} finally {
			try {
				inStream.close();
			} catch (java.io.IOException ioe) {
				throw new RuntimeException(ioe);
			}
		}
		return props;
	}

	public void seleccionarComonarComo() {
		System.out.println("\n======================= Este es un test de u"
				+ "so de ciertos mtodos para rellenar las colecciones "
				+ "===========================\n");
		try {
			setAsignaciones(cargaAsignaciones("Clientes.propiedades"));
			String sql = sqlMap
					.getProperty("buscarTodosLosClientesConTelefono");
			System.out.println("============== Como la lista de valores de "
					+ "los objetos se muestran: (primer valor de la "
					+ "columna y la segunda columna) ==================");
			List list = dbServicios.getResultadoComoListaObjetoValor(sql);
			System.out.println(list);
			System.out.println("============== como vector de vectores "
					+ "==================");
			Vector vector1 = dbServicios.getResultadosComoVector(sql);
			System.out.println(vector1);
			System.out.println("============== En el mapa de las Listas ("
					+ "key=nombreProducto de columna, el valor=lista de "
					+ "valores de la columna) ==================");
			Map map = dbServicios.getResultadoComoMapa(sql);
			System.out.println(map);
			System.out.println("============== Como array de Contenedor"
					+ "Datos ==================");

			ContenedorDatos[] contenedorDatos = dbServicios
					.getResultadoComoManejadorDatosArray(sql, false);
			for (int i = 0; i < contenedorDatos.length; i++) {
				System.out.println(contenedorDatos[i]);
			}
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		} finally {
			try {
				dbServicios.liberacion(true);
			} catch (Exception ex) {
				ex.printStackTrace();
			}
		}
	}

	public void clausulaIn() {
		System.out
				.println("\n======================= Probando la clusula IN"
						+ "======================= y usando SELECT conexion numero de parmetros\n");
		try {
			dbServicios.setMaxNumeroDeParametros(3);
			dbServicios.setEspacioReservado(":#");
			dbServicios.setCacheSentencias(true);
			setSolicitante(Clientes.class);
			setAsignaciones(cargaAsignaciones("Clientes.propiedades"));

			List listaArg = new ArrayList();
			String sql = sqlMap.getProperty("clausulaIn");

			List cliId = new ArrayList();
			cliId.add(new Integer(0));
			cliId.add(new Integer(1));
			cliId.add(new Integer(2));
			cliId.add(new Integer(3));
			cliId.add(new Integer(4));
			cliId.add(new Integer(5));
			cliId.add(new Integer(6));
			cliId.add(new Integer(7));

			listaArg.add("Sergio Perez");
			listaArg.add(cliId);

			cargar(dbServicios.getResultSets(sql, listaArg));

			Iterator it1 = getListaObjetos().iterator();
			while (it1.hasNext()) {
				System.out.println(it1.next());
			}
			dbServicios.liberacion(false);
			System.out.println("==================================");

			List list = dbServicios.getResultadosComoManejadorDatoLista(sql,
					listaArg, false);

			Iterator it2 = list.iterator();
			while (it2.hasNext()) {
				System.out.println(it2.next());
			}
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		} finally {
			try {
				dbServicios.liberacion(true);
			} catch (Exception ex) {
				ex.printStackTrace();
			}
		}
	}

	public int buscarTodosLosClientes() {
		int records = 0;
		try {
			String sql = sqlMap.getProperty("buscarTodosLosClientes");
			records = this.cargar(this.dbServicios.getResultSet(sql));
			imprimirTodosLosClientes();
		} catch (SQLException sqle) {
			throw new RuntimeException(sqle);
		}
		return records;
	}

	public int buscarTodosLosClientesConTelefono() {
		System.out.println("\n======================= Prueba en la que se "
				+ "usa la herencia ===========================\n");
		this.setSolicitante(ClientesConTelefono.class);

		setAsignaciones(cargaAsignaciones("ClientesConTelefono.propiedades"));

		int records = 0;
		try {
			records = this.cargar(this.dbServicios.getResultSet(sqlMap
					.getProperty("buscarTodosLosClientesConTelefono")));
			imprimirTodosLosClientes();
		} catch (SQLException sqle) {
			throw new RuntimeException(sqle);
		}
		return records;
	}

	public void crearYAlmacenarClientes() {
		System.out.println("\n======================= Prueba de creacion y"
				+ " actualizacion de objetos (INSERT & UPDATE) ==="
				+ "========================\n");
		this.setSolicitante(ClientesConTelefono.class);
		setAsignaciones(cargaAsignaciones("ClientesConTelefono.propiedades"));
		this.getDBServicios().setCacheSentencias(true);
		int records = buscarTodosLosClientes();
		ClientesConTelefono clientesConTelefono = new ClientesConTelefono();
		clientesConTelefono.setIdCliente(new Integer(records + 1));
		clientesConTelefono.setNombre("Sergio");
		clientesConTelefono.setApellidos("Sanchez");
		clientesConTelefono.setDireccion("Gran Avenida.");
		clientesConTelefono.setCiudad("Madrid");
		clientesConTelefono.setTelefono("987654321");

		this.crear(clientesConTelefono);
		System.out.println("\nCliente creado: " + clientesConTelefono + "\n");

		ListIterator li = this.listaObjetos.listIterator();
		while (li.hasNext()) {
			ClientesConTelefono customer = (ClientesConTelefono) li.next();
			String lastName = customer.getApellidos();
			if (lastName.equals("Miller")) {
				customer.setTelefono("912345678");
				customer.almacenar();
			}
		}

		System.out.println("\n======================= Se han realizado las "
				+ "modificaciones de los datos ===========================\n");
		imprimirTodosLosClientes();

		try {
			this.dbServicios.comenzandoTransaccion();
			boolean exito = this.escribirDatos();
			Throwable t = this.getQueEstaMal();
			if (t != null) {
				throw t;
			}
			if (exito) {
				this.dbServicios.validandoTransaccion();
			} else {
				throw new SQLException(
						"Modificacion de los datos conexion errores");
			}
			this.sincronizandoEstadoDePersistencia();
		} catch (Throwable t) {
			try {
				this.dbServicios.deshacerTransaccion();
			} catch (SQLException sqle) {
				sqle.printStackTrace();
				throw new RuntimeException(sqle);
			}
			t.printStackTrace();
		} finally {
			try {
				this.dbServicios.liberacion(true);
			} catch (SQLException sqle) {
				sqle.printStackTrace();
				throw new RuntimeException(sqle);
			}
		}

		System.out.println("\n======================= Paso despues "
				+ "de COMMIT y sincronizandoEstadoDePersistencia "
				+ "o despues de ROLLBACK ===========================\n");
		imprimirTodosLosClientes();
	}

	public void buscarClientes() {
		System.out.println("\n======================= Prueba usando criterios"
				+ " ===========================\n");
		setAsignaciones(cargaAsignaciones("Clientes.propiedades"));
		this.poblarMetaDatos();
		List manejador = new ArrayList(2);
		ManejadorCriterioBusqueda mcb1 = new ManejadorCriterioBusqueda();
		mcb1.setNombreCampo("apellidos");
		mcb1.setOperator(ManejadorCriterioBusqueda.LIKE);
		mcb1.setFunciones("UPPER(${})");
		mcb1.setValorBuscado("MI%");
		mcb1.setCondicionBoolean(ManejadorCriterioBusqueda.OR);
		manejador.add(mcb1);
		ManejadorCriterioBusqueda sch2 = new ManejadorCriterioBusqueda();
		sch2.setNombreCampo("apellidos");
		sch2.setOperator(ManejadorCriterioBusqueda.LIKE);
		sch2.setFunciones("UPPER(${})");
		sch2.setValorBuscado("S%");
		manejador.add(sch2);
		ClientesCriterios criteria = new ClientesCriterios();
		criteria.setMetaData(this.metadatos);
		criteria.setManejadorCriterioBusqueda(manejador);
		criteria.construirCriterio();
		String select = this.sqlMap.getProperty("buscarTodosLosClientes");
		criteria.setSelect(select);
		String sql = criteria.getConsulta();
		Collection p = criteria.getArgumentos();

		try {
			this.cargar(this.dbServicios.getResultSet(sql, p));
		} catch (SQLException sqle) {
			throw new RuntimeException(sqle);
		}
		imprimirTodosLosClientes();
	}

	protected void imprimirTodosLosClientes() {
		List products = this.getListaObjetos();
		Iterator iter = products.iterator();
		while (iter.hasNext()) {
			Clientes clientes = (Clientes) iter.next();
			System.out.println(clientes);
		}
	}
}
