package slgfp.pruebas;

import java.util.*;
import java.sql.*;
import java.math.BigDecimal;

/**
 * Nos sirve de base para recuperar los registros de las tablas de la base de
 * datos preparndonos para definir las estructuras necesarias, para devolver
 * valores en modo Hoja de Calculo
 * 
 * @version 1.0, 03-05-2011
 * @author Sergio Lovillo
 */

@SuppressWarnings({ "serial", "rawtypes", "unchecked", "unused",
		"static-access" })
public class FabricaDeHojasDeCalculos extends slgfp.FabricaObjPersistencia
		implements java.io.Serializable, Cloneable {

	public FabricaDeHojasDeCalculos() {
		this.setSolicitante(HojasDeCalculos.class);
		setAsignaciones(cargarMapeos("HojasDeCalculos.propiedades"));
		sqlMap = cargarMapeos("HojasDeCalculos.sql");
	}

	protected Properties sqlMap = null;

	public Properties cargarMapeos(String path) {
		java.io.InputStream inStream = null;
		Properties propiedades = new Properties();
		try {
			inStream = this.getClass().getResourceAsStream(path);
			propiedades.load(inStream);
		} catch (java.io.IOException ioe) {
			throw new RuntimeException(ioe);
		} finally {
			try {
				inStream.close();
			} catch (java.io.IOException ioe) {
				throw new RuntimeException(ioe);
			}
		}
		return propiedades;
	}

	public int buscarTodasLasFacturasDeClientes() {
		int records = 0;
		try {
			Collection arg = new ArrayList(1);
			records = this.cargar(this.dbServicios.getResultSet(sqlMap
					.getProperty("buscarTodas")));
			imprimirHojaDeCalculo();
		} catch (SQLException sqle) {
			throw new RuntimeException(sqle);
		}
		return records;
	}

	public void modificarFacturas() {
		this.setBloqueoOptimista(this.KEY_Y_COLUMNA_ACTUALIZADA);
		this.setLanzarBloqueoOptimistaDeleteExcepcion(true);
		this.setLanzarBloqueoOptimistaUpdateExcepcion(true);
		Map invoiceTotalMap = new HashMap();
		Collection arg = new ArrayList(1);
		arg.add(new Integer(4));
		try {
			this.dbServicios.setCacheSentencias(true);
			this.cargar(this.dbServicios.getResultSet(
					sqlMap.getProperty("buscarTodasFacturasClientes"), arg));
		} catch (SQLException sqle) {
			throw new RuntimeException(sqle);
		}

		// Eliminamos algunas facturas
		ListIterator li1 = this.listaObjetos.listIterator();
		while (li1.hasNext()) {
			HojasDeCalculos record = (HojasDeCalculos) li1.next();
			Integer invoiceId = record.getInvoiceId();
			BigDecimal cost = record.getCost();
			if (cost.doubleValue() > 30.0) {
				record.eliminar();
			} else {
				BigDecimal invoiceTotal = (BigDecimal) invoiceTotalMap
						.get(invoiceId);
				if (invoiceTotal == null) {
					invoiceTotal = new BigDecimal(0.0);
				}
				invoiceTotal = invoiceTotal.add(cost);
				invoiceTotalMap.put(invoiceId, invoiceTotal);
			}
		}

		System.out.println("\n======================= facturasTotalMap: \n"
				+ invoiceTotalMap);

		try {
			// Desde HojasDeCalculos podemos modificar ms de una tabla.
			this.setTabla("ITEM");
			this.dbServicios.comenzandoTransaccion();
			boolean success1 = this.escribirDatos();
			Throwable t1 = this.getQueEstaMal();
			if (t1 != null) {
				throw t1;
			}

			// Actualizamos la tabla FACTURAS
			// Actualizamos el total de la factura
			ListIterator li2 = this.listaObjetos.listIterator();
			while (li2.hasNext()) {
				HojasDeCalculos record = (HojasDeCalculos) li2.next();
				Integer invoiceId = record.getInvoiceId();
				Object total = invoiceTotalMap.get(invoiceId);
				BigDecimal invoiceTotal = null;
				if (total == null) {
					invoiceTotal = new BigDecimal(0);
				} else {
					invoiceTotal = (BigDecimal) total;
				}
				record.setTotal(invoiceTotal);
				record.almacenar();
			}
			this.setTabla("INVOICE");
			// No DELETE en facturas
			char[] updateOnly = new char[1];
			updateOnly[0] = this.SECUENCIA_MODIFICACION_DATOS_UPDATE;
			this.setSecuenciaModificacionDatos(updateOnly);
			boolean success2 = this.escribirDatos();
			Throwable t2 = this.getQueEstaMal();
			if (t2 != null) {
				throw t2;
			}
			if (success1 == true && success2 == true) {
				this.dbServicios.validandoTransaccion();
			} else {
				throw new SQLException("Modificacion de datos: incorrecta");
			}
			this.sincronizandoEstadoDePersistencia();
		} catch (Throwable t) {
			try {
				this.dbServicios.deshacerTransaccion();
			} catch (SQLException sqle) {
				sqle.printStackTrace();
				throw new RuntimeException(sqle);
			}
			t.printStackTrace();
		} finally {
			try {
				this.dbServicios.liberacion(true);
			} catch (SQLException sqle) {
				sqle.printStackTrace();
				throw new RuntimeException(sqle);
			}
		}
		System.out
				.println("\n======================= Despues de COMMIT & sincronizarEstadoPersistente() o despues de ROLLBACK ===========================\n");
	}

	protected void imprimirHojaDeCalculo() {
		List filas = this.getListaObjetos();
		Iterator it = filas.iterator();
		while (it.hasNext()) {
			HojasDeCalculos fila = (HojasDeCalculos) it.next();
			System.out.println(fila);
		}
	}
}