package slgfp.pruebas;

import java.util.*;
import slgfp.*;
import java.sql.*;

/**
 * Nos sirve de base para recuperar los registros de la tabla PRODUCT de la base
 * de datos preparndonos para definir las estructuras necesarias.
 * 
 * @version 1.0, 03-05-2011
 * @author Sergio Lovillo
 */

@SuppressWarnings({ "serial", "rawtypes" })
public class FabricaDeProductos extends slgfp.FabricaObjPersistencia implements
		java.io.Serializable, Cloneable {

	public FabricaDeProductos() {
		this.setSolicitante(Productos.class);
		/*
		 * No es necesario cargar las asignaciones de esta forma, podra ser
		 * cualquier cosa: XML, JNDI, bases de datos, etc..
		 */
		setAsignaciones(loadMappings("Productos.propiedades"));
		sqlMap = loadMappings("Productos.sql");
	}

	protected Properties sqlMap = null;

	public Properties loadMappings(String path) {
		java.io.InputStream inStream = null;
		Properties props = new Properties();
		try {
			inStream = this.getClass().getResourceAsStream(path);
			props.load(inStream);
		} catch (java.io.IOException ioe) {
			throw new RuntimeException(ioe);
		} finally {
			try {
				inStream.close();
			} catch (java.io.IOException ioe) {
				throw new RuntimeException(ioe);
			}
		}
		return props;
	}

	public int buscarTodosProductos() {
		int records = 0;
		try {
			records = this.cargar(this.dbServicios.getResultSet(sqlMap
					.getProperty("buscarTodosProductos")));
			printAllProducts();
		} catch (SQLException sqle) {
			throw new RuntimeException(sqle);
		}
		return records;
	}

	public void crearYAlmacenarProductosDespues() {
		this.setGenerarSoloSQL(true);
		int records = buscarTodosProductos();
		Productos productos = new Productos();
		productos.setProductoId(new Integer(records + 1));
		productos.setNombreProducto("Zaurus SL-5600");
		productos.setPrecio(299.98);
		productos.setObservaciones("Es un buen producto");
		this.crear(productos);
		System.out.println("\n=============== Producto creado: " + productos
				+ "\n");

		java.text.NumberFormat nf = java.text.NumberFormat.getInstance();
		nf.setMaximumFractionDigits(2);
		ListIterator li = this.listaObjetos.listIterator();
		while (li.hasNext()) {
			Productos productToUpdate = (Productos) li.next();
			double currentPrice = productToUpdate.getPrecio();
			if (currentPrice < 10) {
				double newPrice = currentPrice * 1.1;
				productToUpdate.setPrecio(Double.parseDouble(nf
						.format(newPrice)));
				productToUpdate.almacenar();
			}
		}

		String generatedSQLasXML = null;
		List generatedSQL = null;
		try {
			boolean success = this.escribirDatos();
			Throwable t = this.getQueEstaMal();
			if (t != null) {
				throw t;
			}
			if (!success) {
				throw new SQLException("Modificacion de los datos: fallida");
			}
			generatedSQLasXML = this.getGeneratedSQLasXML();
			generatedSQL = this.getGeneratedSQL();
			this.sincronizandoEstadoDePersistencia();
		} catch (Throwable t) {
			t.printStackTrace();
		} finally {
			try {
				this.dbServicios.liberacion(true);
			} catch (SQLException sqle) {
				sqle.printStackTrace();
				throw new RuntimeException(sqle);
			}
		}

		System.out
				.println("\n======================= Aqui se genera XML que puede ser enviado a una aplicacin o servidor de aplicaciones para actualizar la base de datos ===========================\n");
		System.out.println(generatedSQLasXML);
		System.out
				.println("\n======================= Aqui se genera XML que puede ser enviado a una aplicacin o servidor de aplicaciones para actualizar la base de datos ===========================\n");
		Iterator it = generatedSQL.iterator();
		while (it.hasNext()) {
			Object[] obj = (Object[]) it.next();
			Object[] params = (Object[]) obj[1];
			System.out.println("sql: " + obj[0] + " parametros: "
					+ Arrays.asList(params));
		}
	}

	public void getPaginadoDeProductos() {
		System.out
				.println("\n======================= Mostramos todos los productos ===========================\n");
		buscarTodosProductos();
		this.setTamaoPagina(10);
		MarcadorPagina page = this.getPagina(1);
		System.out.println("\n=============== Total registros: "
				+ page.getObjetosTotal() + ", Pagina " + page.getNumeroPagina()
				+ " de " + page.getPagesTotal() + " ===============\n");
		Iterator iter1 = page.getPagina().iterator();
		while (iter1.hasNext()) {
			Productos productos = (Productos) iter1.next();
			System.out.println(productos);
		}

		MarcadorPagina thirdPage = this.getPagina(3);
		System.out.println("\n=============== Total registros: "
				+ thirdPage.getObjetosTotal() + ", Pagina "
				+ thirdPage.getNumeroPagina() + " de "
				+ thirdPage.getPagesTotal() + " ===============\n");
		Iterator iter2 = thirdPage.getPagina().iterator();
		while (iter2.hasNext()) {
			Productos productos = (Productos) iter2.next();
			System.out.println(productos);
		}
	}

	protected void printAllProducts() {
		List products = this.getListaObjetos();
		Iterator productsIter = products.iterator();
		while (productsIter.hasNext()) {
			Productos productos = (Productos) productsIter.next();
			System.out.println(productos);
		}
	}
}