package slgfp.pruebas;

import java.util.*;
import java.sql.*;
import javax.sql.DataSource;
import slgfp.*;
import slgfp.util.db.*;

/**
 * Nos sirve de base para ejecutar una batera de pruebas que comprueben el
 * correcto funcionamiento del framework. Tenemos un total de 11 en las cuales
 * se ha definido un cmodo fichero llamado PRUEBA.BAT donde si es lanzado con
 * un nmero de dos dgitos, por ejemplo, PRUEBA 01, se ejecutar la prueba
 * numero 1.
 * 
 * @version 1.0, 03-05-2011
 * @author Sergio Lovillo
 */

public class SLGFPPruebas {

	public SLGFPPruebas() {
	}

	protected String controlador = null;
	protected String dbURL = null;
	protected String usuario = null;
	protected String password = "";
	protected String opcionesDeConexion = null;
	protected String ruta = null;
	protected String prueba = null;
	protected SQLScriptEjecutor configuracion = new SQLScriptEjecutor();

	public void buscarTodosProductos() {
		FabricaDeProductos fabricaDeProductos = new FabricaDeProductos();
		fabricaDeProductos.setNoNombreDeColumnaCompleta(true);
		try {
			fabricaDeProductos.getDBServicios().setDepurador(false);
			System.out
					.println("\n============ Recreando todos los objetos de la base de datos ============\n");
			configuracion.ejecuta(controlador, dbURL, usuario, password,
					opcionesDeConexion, ruta, ";");
			System.out
					.println("\n======================= Ejecutando la prueba ============================\n");
			fabricaDeProductos.setOrigenDatos(getOrigenDatos());
			fabricaDeProductos.buscarTodosProductos();
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		} finally {
			try {
				fabricaDeProductos.getDBServicios().liberacion(true);
			} catch (SQLException sqle) {
				sqle.printStackTrace();
				throw new RuntimeException(sqle);
			}
		}
	}

	public void getPaginasDeProductos() {
		FabricaDeProductos fabricaDeProductos = new FabricaDeProductos();
		fabricaDeProductos.setNoNombreDeColumnaCompleta(true);
		try {
			fabricaDeProductos.getDBServicios().setDepurador(false);
			System.out
					.println("\n============ Recreando todos los objetos de la base de datos ============\n");
			configuracion.ejecuta(controlador, dbURL, usuario, password,
					opcionesDeConexion, ruta, ";");
			System.out
					.println("\n======================= Ejecutando la prueba ============================\n");
			fabricaDeProductos.setOrigenDatos(getOrigenDatos());
			fabricaDeProductos.getPaginadoDeProductos();
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		} finally {
			try {
				fabricaDeProductos.getDBServicios().liberacion(true);
			} catch (SQLException sqle) {
				sqle.printStackTrace();
				throw new RuntimeException(sqle);
			}
		}
	}

	public void buscarTodosClientes() {
		FabricaDeClientes fabricaDeClientes = new FabricaDeClientes();
		fabricaDeClientes.setSolicitante(Clientes.class);
		fabricaDeClientes.setAsignaciones(fabricaDeClientes
				.cargaAsignaciones("Clientes.propiedades"));
		fabricaDeClientes.setNoNombreDeColumnaCompleta(true);
		try {
			fabricaDeClientes.getDBServicios().setDepurador(false);
			System.out
					.println("\n============ Recreando todos los objetos de la base de datos ============\n");
			configuracion.ejecuta(controlador, dbURL, usuario, password,
					opcionesDeConexion, ruta, ";");
			System.out
					.println("\n======================= Ejecutando la prueba ============================\n");
			fabricaDeClientes.setOrigenDatos(getOrigenDatos());
			fabricaDeClientes.buscarTodosLosClientes();
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		} finally {
			try {
				fabricaDeClientes.getDBServicios().liberacion(true);
			} catch (SQLException sqle) {
				sqle.printStackTrace();
				throw new RuntimeException(sqle);
			}
		}
	}

	public void buscarCliente() {
		FabricaDeClientes fabricaDeClientes = new FabricaDeClientes();
		fabricaDeClientes.setSolicitante(Clientes.class);
		fabricaDeClientes.setAsignaciones(fabricaDeClientes
				.cargaAsignaciones("Clientes.propiedades"));
		fabricaDeClientes.setNoNombreDeColumnaCompleta(true);
		try {
			fabricaDeClientes.getDBServicios().setDepurador(false);
			System.out
					.println("\n============ Recreando todos los objetos de la base de datos ============\n");
			configuracion.ejecuta(controlador, dbURL, usuario, password,
					opcionesDeConexion, ruta, ";");
			System.out
					.println("\n======================= Ejecutando la prueba ============================\n");
			fabricaDeClientes.setOrigenDatos(getOrigenDatos());
			fabricaDeClientes.buscarClientes();
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		} finally {
			try {
				fabricaDeClientes.getDBServicios().liberacion(true);
			} catch (SQLException sqle) {
				sqle.printStackTrace();
				throw new RuntimeException(sqle);
			}
		}
	}

	public void buscarTodosLosClientesConTelefono() {
		FabricaDeClientes fabricaDeClientes = new FabricaDeClientes();
		fabricaDeClientes.setNoNombreDeColumnaCompleta(true);
		try {
			fabricaDeClientes.getDBServicios().setDepurador(false);
			System.out
					.println("\n============ Recreando todos los objetos de la base de datos ============\n");
			configuracion.ejecuta(controlador, dbURL, usuario, password,
					opcionesDeConexion, ruta, ";");
			System.out
					.println("\n======================= Ejecutando la prueba ============================\n");
			fabricaDeClientes.setOrigenDatos(getOrigenDatos());
			fabricaDeClientes.buscarTodosLosClientesConTelefono();
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		} finally {
			try {
				fabricaDeClientes.getDBServicios().liberacion(true);
			} catch (SQLException sqle) {
				sqle.printStackTrace();
				throw new RuntimeException(sqle);
			}
		}
	}

	public void crearYAlmacenarProductosDespues() {
		FabricaDeProductos fabricaDeProductos = new FabricaDeProductos();
		fabricaDeProductos.setNoNombreDeColumnaCompleta(true);
		try {
			fabricaDeProductos.getDBServicios().setDepurador(false);
			System.out
					.println("\n============ Recreando todos los objetos de la base de datos ============\n");
			configuracion.ejecuta(controlador, dbURL, usuario, password,
					opcionesDeConexion, ruta, ";");
			System.out
					.println("\n======================= Ejecutando la prueba ============================\n");
			fabricaDeProductos.setOrigenDatos(getOrigenDatos());
			fabricaDeProductos.crearYAlmacenarProductosDespues();
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		} finally {
			try {
				fabricaDeProductos.getDBServicios().liberacion(true);
			} catch (SQLException sqle) {
				sqle.printStackTrace();
				throw new RuntimeException(sqle);
			}
		}
	}

	public void crearYAlmacenarClientes() {
		FabricaDeClientes fabricaDeClientes = new FabricaDeClientes();
		fabricaDeClientes.setNoNombreDeColumnaCompleta(true);
		try {
			fabricaDeClientes.getDBServicios().setDepurador(false);
			configuracion.ejecuta(controlador, dbURL, usuario, password,
					opcionesDeConexion, ruta, ";");
			fabricaDeClientes.setOrigenDatos(getOrigenDatos());
			fabricaDeClientes.crearYAlmacenarClientes();
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		} finally {
			try {
				fabricaDeClientes.getDBServicios().liberacion(true);
			} catch (SQLException sqle) {
				sqle.printStackTrace();
				throw new RuntimeException(sqle);
			}
		}
	}

	public void modificarFacturas() {
		FabricaDeHojasDeCalculos fabricaDeHojasDeCalculos = new FabricaDeHojasDeCalculos();
		fabricaDeHojasDeCalculos.setNoNombreDeColumnaCompleta(true);
		try {
			fabricaDeHojasDeCalculos.getDBServicios().setDepurador(false);
			System.out
					.println("\n============ Recreando todos los objetos de la base de datos ============\n");
			configuracion.ejecuta(controlador, dbURL, usuario, password,
					opcionesDeConexion, ruta, ";");
			System.out
					.println("\n======================= Ejecutando la prueba ============================\n");
			fabricaDeHojasDeCalculos.setOrigenDatos(getOrigenDatos());
			fabricaDeHojasDeCalculos.modificarFacturas();
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		} finally {
			try {
				fabricaDeHojasDeCalculos.getDBServicios().liberacion(true);
			} catch (SQLException sqle) {
				sqle.printStackTrace();
				throw new RuntimeException(sqle);
			}
		}
	}

	public void hojaDeCalculos() {
		FabricaDeHojasDeCalculos fabricaDeHojasDeCalculos = new FabricaDeHojasDeCalculos();
		fabricaDeHojasDeCalculos.setNoNombreDeColumnaCompleta(true);
		try {
			fabricaDeHojasDeCalculos.getDBServicios().setDepurador(false);
			System.out
					.println("\n============ Recreando todos los objetos de la base de datos ============\n");
			configuracion.ejecuta(controlador, dbURL, usuario, password,
					opcionesDeConexion, ruta, ";");
			System.out
					.println("\n======================= Ejecutando la prueba ============================\n");
			fabricaDeHojasDeCalculos.setOrigenDatos(getOrigenDatos());
			fabricaDeHojasDeCalculos.buscarTodasLasFacturasDeClientes();
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		} finally {
			try {
				fabricaDeHojasDeCalculos.getDBServicios().liberacion(true);
			} catch (SQLException sqle) {
				sqle.printStackTrace();
				throw new RuntimeException(sqle);
			}
		}
	}

	public void clausulaIn() {
		FabricaDeClientes fabrica = new FabricaDeClientes();
		fabrica.setSoloLectura(true);
		try {
			fabrica.getDBServicios().setDepurador(false);
			System.out
					.println("\n============ Recreando todos los objetos de la base de datos ============\n");
			configuracion.ejecuta(controlador, dbURL, usuario, password,
					opcionesDeConexion, ruta, ";");
			System.out
					.println("\n======================= Ejecutando la prueba ============================\n");
			fabrica.setOrigenDatos(getOrigenDatos());
			fabrica.clausulaIn();
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		} finally {
			try {
				fabrica.getDBServicios().liberacion(true);
			} catch (SQLException sqle) {
				sqle.printStackTrace();
				throw new RuntimeException(sqle);
			}
		}
	}

	public void seleccionarComo() {
		FabricaDeClientes fabrica = new FabricaDeClientes();
		fabrica.setSoloLectura(true);
		try {
			fabrica.getDBServicios().setDepurador(false);
			System.out
					.println("\n============ Recreando todos los objetos de la base de datos ============\n");
			configuracion.ejecuta(controlador, dbURL, usuario, password,
					opcionesDeConexion, ruta, ";");
			System.out
					.println("\n======================= Ejecutando la prueba ============================\n");
			fabrica.setOrigenDatos(getOrigenDatos());
			fabrica.seleccionarComonarComo();
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		} finally {
			try {
				fabrica.getDBServicios().liberacion(true);
			} catch (SQLException sqle) {
				sqle.printStackTrace();
				throw new RuntimeException(sqle);
			}
		}
	}

	protected DataSource getOrigenDatos() {

		DataSourceBasicoImpl origenDeDatos = new DataSourceBasicoImpl();
		origenDeDatos.setNombreUsuario(usuario);
		origenDeDatos.setPassword(password);
		origenDeDatos.setNombreControlador(controlador);
		origenDeDatos.setDbURL(dbURL);
		Properties prop = new Properties();
		prop.setProperty("usuario", usuario);
		prop.setProperty("password", password);
		if (opcionesDeConexion != null
				&& opcionesDeConexion.trim().length() > 0) {
			StringTokenizer st = new StringTokenizer(opcionesDeConexion, ",",
					false);
			while (st.hasMoreTokens()) {
				String token = st.nextToken();
				int idx = token.indexOf("=");
				if (idx == -1) {
					throw new IllegalArgumentException(
							"Formato incorrecto en las propiedades de conexion.");
				}
				String nombre = token.substring(0, idx);
				String valor = token.substring(idx + 1);
				prop.setProperty(nombre, valor);
			}
		}

		origenDeDatos.setPropiedadesConexion(prop);

		return origenDeDatos;
	}

	public java.lang.String getOpcionesDeConexion() {
		return opcionesDeConexion;
	}

	public void setOpcionesDeConexion(java.lang.String opcionesDeConexion) {
		this.opcionesDeConexion = opcionesDeConexion;
	}

	public java.lang.String getDbURL() {
		return dbURL;
	}

	public void setDbURL(java.lang.String dbURL) {
		this.dbURL = dbURL;
	}

	public java.lang.String getControlador() {
		return controlador;
	}

	public void setControlador(java.lang.String controlador) {
		this.controlador = controlador;
	}

	public java.lang.String getPrueba() {
		return prueba;
	}

	public void setPrueba(java.lang.String prueba) {
		this.prueba = prueba;
	}

	public java.lang.String getRuta() {
		return ruta;
	}

	public void setRuta(java.lang.String ruta) {
		this.ruta = ruta;
	}

	public java.lang.String getPassword() {
		return password;
	}

	public void setPassword(java.lang.String password) {
		this.password = password;
	}

	public java.lang.String getUsuario() {
		return usuario;
	}

	public void setUsuario(java.lang.String usuario) {
		this.usuario = usuario;
	}
}
