package slgfp.util.comun;

import java.lang.Class;
import java.lang.reflect.Method;
import java.io.Serializable;
import java.util.Comparator;
import java.lang.reflect.InvocationTargetException;

/**
 * Esta clase es un comparador genrico de objetos.
 * 
 * @version 1.0, 03-05-2011
 * @author Sergio Lovillo
 */

@SuppressWarnings({ "rawtypes", "serial", "unchecked" })
public class ComparadorGenerico implements Comparator, Serializable, Cloneable {

	protected String nombreMetodo = null;
	protected Class[] parmetroClass = null;
	protected Object[] parametros = null;
	protected boolean reversible = false;
	/** Si se comparan los objetos que no son instaceof String este es ignorado */
	protected boolean ignorarMayusculas;

	public ComparadorGenerico() {
	}

	public ComparadorGenerico(String nombreMetodo) {
		this.nombreMetodo = nombreMetodo;
		this.parmetroClass = new Class[0];
	}

	public ComparadorGenerico(String nombreMetodo, boolean reversible) {
		this.nombreMetodo = nombreMetodo;
		this.parmetroClass = new Class[0];
		this.reversible = reversible;
	}

	public ComparadorGenerico(String nombreMetodo, Object[] parametros) {
		this.nombreMetodo = nombreMetodo;
		this.parmetroClass = new Class[parametros.length];
		this.parametros = parametros;
		for (int i = 0; i < parametros.length; i++) {
			this.parmetroClass[i] = parametros.getClass();
		}
	}

	public ComparadorGenerico(String nombreMetodo, Object[] parametros,
			boolean reversible) {
		this.nombreMetodo = nombreMetodo;
		this.parmetroClass = new Class[parametros.length];
		this.reversible = reversible;
		this.parametros = parametros;
		for (int i = 0; i < parametros.length; i++) {
			this.parmetroClass[i] = parametros.getClass();
		}
	}

	public int compare(Object o1, Object o2) {
		Method objMethod = null;
		Object object1 = null;
		Object object2 = null;
		if (!(o1.getClass().isInstance(o2))) {
			throw new ClassCastException("Las clases no coinciden "
					+ o1.getClass().getName() + " : " + o2.getClass().getName());
		}
		try {
			objMethod = o1.getClass().getMethod(nombreMetodo, parmetroClass);
		} catch (NoSuchMethodException e) {
			throw new RuntimeException("Metodo nombreProducto: "
					+ this.nombreMetodo + " no definido en la clase "
					+ o1.getClass().getName());
		}
		try {
			object1 = objMethod.invoke(o1, parametros);
			object2 = objMethod.invoke(o2, parametros);
		} catch (IllegalAccessException e1) {
			throw new RuntimeException("Acceso denegado al metodo "
					+ this.nombreMetodo + " in class "
					+ o1.getClass().getName());
		} catch (InvocationTargetException e2) {
			throw new RuntimeException(this.nombreMetodo + " en la clase "
					+ o1.getClass().getName() + " lanzo la excepcion: "
					+ e2.getTargetException().getMessage(),
					e2.getTargetException());
		}

		if (reversible) {
			if (object1 == null && object2 != null) {
				return -1;
			} else if (object1 != null && object2 == null) {
				return -1;
			} else if (object1 == null && object2 == null) {
				return -1;
			} else {
				if (object1 instanceof String && isCaseInsensitiveOrder()) {
					return -((String) object1)
							.compareToIgnoreCase((String) object2);
				} else {
					return -((Comparable) object1).compareTo(object2);
				}
			}
		} else {
			if (object1 == null && object2 != null) {
				return 1;
			} else if (object1 != null && object2 == null) {
				return -1;
			} else if (object1 == null && object2 == null) {
				return -1;
			} else {
				if (object1 instanceof String && isCaseInsensitiveOrder()) {
					return ((String) object1)
							.compareToIgnoreCase((String) object2);
				} else {
					return ((Comparable) object1).compareTo(object2);
				}
			}
		}
	}

	public java.lang.String getMethodName() {
		return nombreMetodo;
	}

	public void setMethodName(java.lang.String methodName) {
		this.nombreMetodo = methodName;
	}

	public java.lang.Class[] getParmClass() {
		return this.parmetroClass;
	}

	public void setParmClass(java.lang.Class[] parmClass) {
		this.parmetroClass = parmClass;
	}

	public java.lang.Object[] getParms() {
		return this.parametros;
	}

	public void setParms(java.lang.Object[] parms) {
		this.parametros = parms;
	}

	public boolean isReverse() {
		return reversible;
	}

	public void setReverse(boolean reverse) {
		this.reversible = reverse;
	}

	public boolean isCaseInsensitiveOrder() {
		return ignorarMayusculas;
	}

	public void setCaseInsensitiveOrder(boolean caseInsensitiveOrder) {
		this.ignorarMayusculas = caseInsensitiveOrder;
	}
}
