package slgfp.util.comun;

import java.beans.PropertyDescriptor;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.Enumeration;
import java.util.Properties;
import slgfp.GrabarDatos;
import slgfp.FabricaObjPersistencia;
import slgfp.ManejadorExcepciones;

/**
 * Esta clase es un configurador para el objeto FabricaObjPersitencia. Nos
 * permite a partir de un fichero plano tipo PROPERTIES definir los parmetros
 * de configuracin de comportamiento de frameworks
 * 
 * @version 1.0, 03-05-2011
 * @author Sergio Lovillo
 */

@SuppressWarnings({ "rawtypes", "deprecation" })
public class FabricaObjPersistenciaConfiguracion {

	private boolean debug = false;
	/* til para enviar slo objetos modificados a otro nivel */
	private boolean descartarObjetosNoModificados;
	/* Secuencia determinada o desactivar update/insert/delete */
	private char[] secuenciaModificacionDatos;
	/* Desabilitar modificaciones BD */
	private boolean soloLectura = false;

	/* Configuracion de bloqueo optimista ************************** */

	private char bloqueoOptimista = FabricaObjPersistencia.KEY_COLUMNAS;

	/*
	 * Lanzar excepcin si PreparedStatement.executeupdate() no devuelve 1 para
	 * cada ObjPersistencia. Esto significa que lo ms probable es que la fila
	 * de tabla de la BD se ha modificado o suprimido por otro user/proceso.
	 */
	private boolean lanzarBloqueoOptimistaDeleteExcepcion = true;
	private boolean lanzarBloqueoOptimistaUpdateExcepcion = true;
	private boolean lanzarFalloInsercionExcepcion = true;

	/* Final de la configuracin de bloqueo optimista *************** */
	/* Reemplaza el catalogo de la BD */
	private boolean reemplazaNombreCatalogo = false;

	/* Este objeto genera SQL para la modificacion de datos */
	private String grabarDatos = "slgfp.DBGrabarDatos";
	/* Usar colecciones sincronizadas */
	private boolean accesoSincronizado = false;
	/*
	 * Si lazyLoding == true slo se establecen valores originales vs. original
	 * y el actual
	 */
	private boolean cargaDiferida = false;
	/* Registros por pagina */
	private int tamaoPagina = 0;
	/* No ejetuta INSERTS/UPDATES/DELETES - genera el SQL y los parametros */
	private boolean generarSoloSQL = false;
	/* Controla si cargar() aade objetos a la objetcList */
	private boolean aadir = false;
	/*
	 * Algunas bases de datos (como hsqdb 1.7.1 ) no funcionan correctamente
	 * conexion instrucciones UPDATE que utiliza NOM_TABLA.COLUMN_NAME en
	 * clusula SET. INSERT tambin da problemas
	 */
	private boolean noNombreDeColumnaCompleta = false;
	/*
	 * Lanza excepcin o ignorar si ObjPersistencia tiene menos campos de los
	 * que se encuentran en el asignaciones
	 */
	private boolean lanzarExcepcionCampoInexistente = false;
	/* Si no null object debera manejar excepciones */
	private String manejadorExcepcion;
	/* Si verdad inhibir excepcin cuando llame objectList writeData en vaco */
	private boolean excepcionSiListaObjetosVacia = true;
	/* Crear y rellenar automticamente Metadata */
	private boolean creaMetadatosDefecto = true;

	private Properties configuracion;
	private String ruta;

	public FabricaObjPersistenciaConfiguracion() {
	}

	public void configure(FabricaObjPersistencia factory) {
		factory.setDebug(debug);
		factory.setDescartarObjetosNoModificados(descartarObjetosNoModificados);
		factory.setSoloLectura(soloLectura);
		factory.setBloqueoOptimista(bloqueoOptimista);
		factory.setLanzarFalloInsercionExcepcion(lanzarFalloInsercionExcepcion);
		factory.setLanzarBloqueoOptimistaDeleteExcepcion(lanzarBloqueoOptimistaDeleteExcepcion);
		factory.setLanzarBloqueoOptimistaUpdateExcepcion(lanzarBloqueoOptimistaUpdateExcepcion);
		factory.setReemplazaNombreCatalogo(reemplazaNombreCatalogo);
		factory.setAccesoSincronizado(accesoSincronizado);
		factory.setTamaoPagina(tamaoPagina);
		factory.setGenerarSoloSQL(generarSoloSQL);
		factory.setAadir(aadir);
		factory.setNoNombreDeColumnaCompleta(noNombreDeColumnaCompleta);
		factory.setLanzarExcepcionCampoInexistente(lanzarExcepcionCampoInexistente);
		factory.setExcepcionSiListaObjetosVacia(excepcionSiListaObjetosVacia);
		factory.setCreaMetadatosDefecto(creaMetadatosDefecto);
		if (secuenciaModificacionDatos != null
				&& secuenciaModificacionDatos.length > 0) {
			factory.setSecuenciaModificacionDatos(secuenciaModificacionDatos);
		}
		try {
			factory.setManejadorExcepciones((ManejadorExcepciones) Class
					.forName(manejadorExcepcion).newInstance());
		} catch (Exception e) {
			e.printStackTrace();
			// Se ignora
		}
		try {
			factory.setGrabarDatos((GrabarDatos) Class.forName(grabarDatos)
					.newInstance());
		} catch (Exception e) {
			e.printStackTrace();
			// Se ignora
		}
	}

	public void poblar(Properties configuracion) {
		this.configuracion = configuracion;
		poblar();
	}

	public void poblar() {
		Enumeration enumeration = configuracion.keys();
		while (enumeration.hasMoreElements()) {
			String property = (String) enumeration.nextElement();
			String value = configuracion.getProperty(property);
			if (property.equals("grabarDatos")) {
				try {
					grabarDatos = value;
				} catch (Exception e) {
					e.printStackTrace();
					e.printStackTrace();
					// Se ignora
				}
			} else if (property.equals("manejadorExcepciones")) {
				try {
					manejadorExcepcion = value;
				} catch (Exception e) {
					e.printStackTrace();
					// Se ignora
				}
				// como DIU, UD, UI etc.
			} else if (property.equals("secuenciaModificacionDatos")) {
				secuenciaModificacionDatos = value.toCharArray();
			} else {
				try {
					Method mWrite = (new PropertyDescriptor(property,
							this.getClass())).getWriteMethod();
					Object[] param = new Boolean[1];
					param[0] = new Boolean(value);
					mWrite.invoke(this, param);
				} catch (Exception e) {
					e.printStackTrace();
					// Se ignora
				}
			}
		}
	}

	public void almacenar() {
		if (configuracion == null) {
			configuracion = new Properties();
		}
		Field[] campos = getClass().getDeclaredFields();
		for (int i = 0; i < campos.length; i++) {
			try {
				String nombre = campos[i].getName();
				if (nombre.equals("manejadorExcepciones")) {
					configuracion.setProperty(nombre, campos[i].get(this)
							.toString());
				} else if (nombre.equals("grabarDatos")) {
					configuracion.setProperty(nombre, campos[i].get(this)
							.toString());
				} else if (nombre.equals("secuenciaModificacionDatos")) {
					configuracion.setProperty(nombre, new String(
							secuenciaModificacionDatos));
				}
				if (campos[i].getType().equals(boolean.class)) {
					configuracion.setProperty(nombre,
							String.valueOf(campos[i].getBoolean(this)));
				}
			} catch (Exception e) {
				e.printStackTrace();
				// Se ignora
			}
		}
		try {
			configuracion
					.store(new FileOutputStream(ruta),
							"FabricaObjPersistenciaConfiguracion: FabricaObjPersistencia configuracion");
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void cargar(String ruta) {
		this.ruta = ruta;
		cargar();
	}

	public void cargar() {
		java.io.InputStream inStream = null;
		configuracion = new Properties();
		try {
			inStream = new File(ruta).toURL().openStream();
			configuracion.load(inStream);
		} catch (java.io.IOException ioe) {
			throw new RuntimeException(ioe);
		} finally {
			try {
				inStream.close();
			} catch (java.io.IOException ioe) {
				throw new RuntimeException(ioe);
			}
		}
	}

	public boolean isDebug() {
		return debug;
	}

	public void setDebug(boolean debug) {
		this.debug = debug;
	}

	public boolean isDescartarObjetosNoModificados() {
		return descartarObjetosNoModificados;
	}

	public void setDescartarObjetosNoModificados(
			boolean descartarObjetosNoModificados) {
		this.descartarObjetosNoModificados = descartarObjetosNoModificados;
	}

	public char[] getSecuenciaModificacionDatos() {
		return secuenciaModificacionDatos;
	}

	public void setSecuenciaModificacionDatos(char[] secuenciaModificacionDatos) {
		this.secuenciaModificacionDatos = secuenciaModificacionDatos;
	}

	public boolean isSoloLectura() {
		return soloLectura;
	}

	public void setSoloLectura(boolean soloLectura) {
		this.soloLectura = soloLectura;
	}

	public char getBloqueoOptimista() {
		return bloqueoOptimista;
	}

	public void setBloqueoOptimista(char bloqueoOptimista) {
		this.bloqueoOptimista = bloqueoOptimista;
	}

	public boolean isLanzarBloqueoOptimistaDeleteExcepcion() {
		return lanzarBloqueoOptimistaDeleteExcepcion;
	}

	public void setLanzarBloqueoOptimistaDeleteExcepcion(
			boolean lanzarBloqueoOptimistaDeleteExcepcion) {
		this.lanzarBloqueoOptimistaDeleteExcepcion = lanzarBloqueoOptimistaDeleteExcepcion;
	}

	public boolean isLanzarBloqueoOptimistaUpdateExcepcion() {
		return lanzarBloqueoOptimistaUpdateExcepcion;
	}

	public void setLanzarBloqueoOptimistaUpdateExcepcion(
			boolean lanzarBloqueoOptimistaUpdateExcepcion) {
		this.lanzarBloqueoOptimistaUpdateExcepcion = lanzarBloqueoOptimistaUpdateExcepcion;
	}

	public boolean isLanzarFalloInsercionExcepcion() {
		return lanzarFalloInsercionExcepcion;
	}

	public void setLanzarFalloInsercionExcepcion(
			boolean lanzarFalloInsercionExcepcion) {
		this.lanzarFalloInsercionExcepcion = lanzarFalloInsercionExcepcion;
	}

	public boolean isReemplazaNombreCatalogo() {
		return reemplazaNombreCatalogo;
	}

	public void setReemplazaNombreCatalogo(boolean reemplazaNombreCatalogo) {
		this.reemplazaNombreCatalogo = reemplazaNombreCatalogo;
	}

	public String getGrabarDatos() {
		return grabarDatos;
	}

	public void setGrabarDatos(String grabarDatos) {
		this.grabarDatos = grabarDatos;
	}

	public boolean isAccesoSincronizado() {
		return accesoSincronizado;
	}

	public void setAccesoSincronizado(boolean accesoSincronizado) {
		this.accesoSincronizado = accesoSincronizado;
	}

	public boolean isCargaDiferida() {
		return cargaDiferida;
	}

	public void setCargaDiferida(boolean cargaDiferida) {
		this.cargaDiferida = cargaDiferida;
	}

	public int getTamaoPagina() {
		return tamaoPagina;
	}

	public void setTamaoPagina(int tamaoPagina) {
		this.tamaoPagina = tamaoPagina;
	}

	public boolean isGenerarSoloSQL() {
		return generarSoloSQL;
	}

	public void setGenerarSoloSQL(boolean generarSoloSQL) {
		this.generarSoloSQL = generarSoloSQL;
	}

	public boolean isAadir() {
		return aadir;
	}

	public void setAadir(boolean aadir) {
		this.aadir = aadir;
	}

	public boolean isNoNombreDeColumnaCompleta() {
		return noNombreDeColumnaCompleta;
	}

	public void setNoNombreDeColumnaCompleta(boolean noNombreDeColumnaCompleta) {
		this.noNombreDeColumnaCompleta = noNombreDeColumnaCompleta;
	}

	public boolean isLanzarExcepcionCampoInexistente() {
		return lanzarExcepcionCampoInexistente;
	}

	public void setLanzarExcepcionCampoInexistente(
			boolean lanzarExcepcionCampoInexistente) {
		this.lanzarExcepcionCampoInexistente = lanzarExcepcionCampoInexistente;
	}

	public String getManejadorExcepcion() {
		return manejadorExcepcion;
	}

	public void setManejadorExcepcion(String manejadorExcepcion) {
		this.manejadorExcepcion = manejadorExcepcion;
	}

	public boolean isExcepcionSiListaObjetosVacia() {
		return excepcionSiListaObjetosVacia;
	}

	public void setExcepcionSiListaObjetosVacia(
			boolean excepcionSiListaObjetosVacia) {
		this.excepcionSiListaObjetosVacia = excepcionSiListaObjetosVacia;
	}

	public boolean isCreaMetadatosDefecto() {
		return creaMetadatosDefecto;
	}

	public void setCreaMetadatosDefecto(boolean creaMetadatosDefecto) {
		this.creaMetadatosDefecto = creaMetadatosDefecto;
	}

	public String getRuta() {
		return ruta;
	}

	public void setRuta(String ruta) {
		this.ruta = ruta;
	}

	// Prueba
	public static void main(String[] args) {
		FabricaObjPersistenciaConfiguracion dfc = new FabricaObjPersistenciaConfiguracion();
		dfc.setRuta("D:\\UOC\\PFC\\SLGFP\\src\\slgfp\\util\\comun\\configuracion.properties");
		dfc.cargar();
		dfc.poblar();
		dfc.getConfiguracion().list(System.out);
		TestFactory tf = new TestFactory();
		System.out.println("Antes de configurar: " + tf);
		dfc.configure(tf);
		System.out.println("Despues de configurar: " + tf);
		dfc.almacenar();
	}

	public Properties getConfiguracion() {
		return configuracion;
	}

	public void setConfiguracion(Properties configuracion) {
		this.configuracion = configuracion;
	}

}

@SuppressWarnings("serial")
class TestFactory extends FabricaObjPersistencia {

	/** Crea una nueva instancia del objeto */
	public TestFactory() {
	}

}
