package slgfp.util.comun;

/**
 * Esta clase se usa para convertir a un tipo de datos concreto el valor del
 * objeto.
 * 
 * @version 1.0, 03-05-2011
 * @author Sergio Lovillo
 */

@SuppressWarnings({ "null", "rawtypes" })
public class TipoDatosConvertidor {

	public TipoDatosConvertidor() {
	}

	// Devuelve el tipo de datos por defecto si hay problemas de conversion
	protected boolean defectoCuandoError;

	// Devuelve el tipo de datos por defecto si es nulo
	protected boolean defectoCuandoNulo;

	public boolean getBoolean(Object valor) {
		boolean valorConvertido = false;
		if (valor == null) {
			if (defectoCuandoNulo) {
				return valorConvertido;
			} else {
				throw new NullPointerException(
						"getBoolean(Object valor): el valor a convertir es nulo");
			}
		}

		if (valor instanceof Number) {
			if (((Number) valor).intValue() == 1) {
				valorConvertido = true;
			} else if (((Number) valor).intValue() == 0) {
				valorConvertido = false;
			} else {
				throw new IllegalArgumentException(
						"getBoolean(Object valor): el valor a convertir no es valido: "
								+ valor);
			}
		} else if (valor instanceof String) {
			String temp = ((String) valor).trim().toLowerCase();
			if (temp.equals("yes") || temp.equals("y") || temp.equals("true")
					|| temp.equals("on")) {
				valorConvertido = true;
			} else if (temp.equals("no") || temp.equals("n")
					|| temp.equals("false") || temp.equals("off")) {
				valorConvertido = false;
			} else {
				throw new IllegalArgumentException(
						"getBoolean(Object valor): el valor a convertir no es valido: "
								+ valor);
			}
		}
		return valorConvertido;
	}

	public int getInt(Object valor) {
		int valorConvertido = 0;
		if (valor == null) {
			if (defectoCuandoNulo) {
				return valorConvertido;
			} else {
				throw new NullPointerException(
						"getInt(Object valor): el valor a convertir es nulo");
			}
		}

		if (valor instanceof Number) {
			valorConvertido = ((Number) valor).intValue();
		} else if (valor instanceof String) {
			try {
				valorConvertido = Integer.parseInt((String) valor);
			} catch (NumberFormatException nfe) {
				if (defectoCuandoError) {
					return valorConvertido;
				} else {
					throw new IllegalArgumentException(
							"getInt(Object valor): no puedo convertir el valor "
									+ valor);
				}
			}
		} else {
			if (!defectoCuandoError) {
				throw new NullPointerException(
						"getInt(Object valor): valor de conversion no valido");
			}
		}
		return valorConvertido;
	}

	public double getDouble(Object valor) {
		double valorConvertido = 0.0d;
		if (valor == null) {
			if (defectoCuandoNulo) {
				return valorConvertido;
			} else {
				throw new NullPointerException(
						"getDouble(Object valor): el valor a convertir es null");
			}
		}
		if (valor instanceof Number) {
			valorConvertido = ((Number) valor).doubleValue();
		} else if (valor instanceof String) {
			try {
				valorConvertido = Double.parseDouble((String) valor);
			} catch (NumberFormatException nfe) {
				if (defectoCuandoError) {
					return valorConvertido;
				} else {
					throw new IllegalArgumentException(
							"getDouble(Object valor): no puedo convertir el valor "
									+ valor);
				}
			}
		} else {
			if (!defectoCuandoError) {
				throw new NullPointerException(
						"getDouble(Object valor): el valor a convertir no es valido");
			}
		}
		return valorConvertido;
	}

	public long getLong(Object valor) {
		long valorCOnvertido = 0l;
		if (valor == null) {
			if (defectoCuandoNulo) {
				return valorCOnvertido;
			} else {
				throw new NullPointerException(
						"getLong(Object valor): el valor a convertir es null");
			}
		}

		if (valor instanceof Number) {
			valorCOnvertido = ((Number) valor).longValue();
		} else if (valor instanceof String) {
			try {
				valorCOnvertido = Long.parseLong((String) valor);
			} catch (NumberFormatException nfe) {
				if (defectoCuandoError) {
					return valorCOnvertido;
				} else {
					throw new IllegalArgumentException(
							"getLong(Object valor): no puedo convertir el valor "
									+ valor);
				}
			}
		} else {
			if (!defectoCuandoError) {
				throw new NullPointerException(
						"getLong(Object valor): el valor a convertir no es valido");
			}
		}
		return valorCOnvertido;
	}

	public float getFloat(Object valor) {
		float valorConvertido = 0.0f;
		if (valor == null) {
			if (defectoCuandoNulo) {
				return valorConvertido;
			} else {
				throw new NullPointerException(
						"getFloat(Object valor): el valor a convertir es null");
			}
		}

		if (valor instanceof Number) {
			valorConvertido = ((Number) valor).floatValue();
		} else if (valor instanceof String) {
			try {
				valorConvertido = Float.parseFloat((String) valor);
			} catch (NumberFormatException nfe) {
				if (defectoCuandoError) {
					return valorConvertido;
				} else {
					throw new IllegalArgumentException(
							"getFloat(Object valor): no puedo convertir el valor "
									+ valor);
				}
			}
		} else {
			if (!defectoCuandoError) {
				throw new NullPointerException(
						"getFloat(Object valor): el valor a convertir no es valido");
			}
		}
		return valorConvertido;
	}

	public short getShort(Object valor) {
		short valorConvertido = 0;
		if (valor == null) {
			if (defectoCuandoNulo) {
				return valorConvertido;
			} else {
				throw new NullPointerException(
						"getShort(Object valor): el valor a convertir es null");
			}
		}

		if (valor instanceof Number) {
			valorConvertido = ((Number) valor).shortValue();
		} else if (valor instanceof String) {
			try {
				valorConvertido = Short.parseShort((String) valor);
			} catch (NumberFormatException nfe) {
				if (defectoCuandoError) {
					return valorConvertido;
				} else {
					throw new IllegalArgumentException(
							"getShort(Object valor): no puedo convertir el valor "
									+ valor);
				}
			}
		} else {
			if (!defectoCuandoError) {
				throw new NullPointerException(
						"getShort(Object valor): el valor a convertir no es valido");
			}
		}
		return valorConvertido;
	}

	public short getByte(Object valor) {
		byte valorConvertido = 0;
		if (valor == null) {
			if (defectoCuandoNulo) {
				return valorConvertido;
			} else {
				throw new NullPointerException(
						"getByte(Object valor): el valor a convertir es null");
			}
		}

		if (valor instanceof Number) {
			valorConvertido = ((Number) valor).byteValue();
		} else if (valor instanceof String) {
			try {
				valorConvertido = Byte.parseByte((String) valor);
			} catch (NumberFormatException nfe) {
				if (defectoCuandoError) {
					return valorConvertido;
				} else {
					throw new IllegalArgumentException(
							"getByte(Object valor): no puedo convertir el valor "
									+ valor);
				}
			}
		} else {
			if (!defectoCuandoError) {
				throw new NullPointerException(
						"getByte(Object valor): el valor a convertir no es valido");
			}
		}
		return valorConvertido;
	}

	public char getChar(Object valor) {
		char valorConvertido = '\u0000';
		if (valor == null) {
			if (defectoCuandoNulo) {
				return valorConvertido;
			} else {
				throw new NullPointerException(
						"getChar(Object valor): el valor a convertir es null");
			}
		}

		String stringValue = valor.toString();
		if (!stringValue.equals("")) {
			valorConvertido = stringValue.charAt(0);
		}
		return valorConvertido;
	}

	public boolean isDefectoCuandoError() {
		return defectoCuandoError;
	}

	public void setDefectoCuandoError(boolean defectoCuandoError) {
		this.defectoCuandoError = defectoCuandoError;
	}

	public boolean isDefectoCuandoNulo() {
		return defectoCuandoNulo;
	}

	public void setDefectoCuandoNulo(boolean defectoCuandoNulo) {
		this.defectoCuandoNulo = defectoCuandoNulo;
	}

	public Object[] convierte(Object valor) {
		if (valor == null) {
			return convierte("", valor.getClass());
		}
		return convierte(valor.toString(), valor.getClass());
	}

	public Object[] convierte(String sV, Class c) {
		return convierte(sV, c.getName());
	}

	// A usar conexion java.lang.reflect.Method.invoke(Object obj, Object[]
	// args)
	public Object[] convierte(String sV, String c) {

		Object[] cV = null;
		try {

			if (c.equals("java.lang.String")) {
				cV = new String[1];
				if (sV == null) {
					cV[0] = (String) null;
				} else {
					cV[0] = sV;
				}
			} else if (c.equals("java.lang.Integer")) {
				cV = new Integer[1];
				if (sV == null || sV.trim().length() == 0) {
					cV[0] = (Integer) null;
				} else {
					cV[0] = new Integer(sV);
				}
			} else if (c.equals("java.lang.Long")) {
				if (sV == null || sV.trim().length() == 0) {
					cV[0] = (Long) null;
				} else {
					cV[0] = new Long(sV);
				}
			} else if (c.equals("java.lang.Double")) {
				cV = new Double[1];
				if (sV == null || sV.trim().length() == 0) {
					cV[0] = (Double) null;
				} else {
					cV[0] = new Double(sV);
				}
			} else if (c.equals("java.lang.Float")) {
				cV = new Float[1];
				if (sV == null || sV.trim().length() == 0) {
					cV[0] = (Float) null;
				} else {
					cV[0] = new Float(sV);
				}
			} else if (c.equals("java.lang.Short")) {
				cV = new Short[1];
				if (sV == null || sV.trim().length() == 0) {
					cV[0] = (Short) null;
				} else {
					cV[0] = new Short(sV);
				}
			} else if (c.equals("java.sql.Time")) {
				cV = new java.sql.Time[1];
				if (sV == null || sV.trim().length() == 0) {
					cV[0] = (java.sql.Time) null;
				} else {
					cV[0] = new java.sql.Time(java.text.DateFormat
							.getInstance().parse(sV).getTime());
				}
			} else if (c.equals("java.sql.Date")) {
				cV = new java.sql.Date[1];
				if (sV == null || sV.trim().length() == 0) {
					cV[0] = (java.sql.Date) null;
				} else {
					cV[0] = new java.sql.Date(java.text.DateFormat
							.getInstance().parse(sV).getTime());
				}
			} else if (c.equals("java.util.Date")) {
				cV = new java.util.Date[1];
				if (sV == null || sV.trim().length() == 0) {
					cV[0] = (java.util.Date) null;
				} else {
					cV[0] = java.text.DateFormat.getInstance().parse(sV);
				}
			} else if (c.equals("java.math.BigDecimal")) {
				cV = new java.math.BigDecimal[1];
				if (sV == null || sV.trim().length() == 0) {
					cV[0] = (java.math.BigDecimal) null;
				} else {
					cV[0] = new java.math.BigDecimal(sV);
				}
			} else if (c.equals("java.math.BigInteger")) {
				cV = new java.math.BigInteger[1];
				if (sV == null || sV.trim().length() == 0) {
					cV[0] = (java.math.BigInteger) null;
				} else {
					cV[0] = new java.math.BigInteger(sV);
				}
			} else if (c.equals("boolean")) {
				cV = new Boolean[1];
				if (sV == null || sV.trim().length() == 0) {
					cV[0] = new Boolean(false);
				} else {
					cV[0] = new Boolean(sV);
				}
			} else if (c.equals("byte")) {
				cV = new Byte[1];
				if (sV == null || sV.trim().length() == 0) {
					byte emptyByte = 0;
					cV[0] = new Byte(emptyByte);
				} else {
					cV[0] = new Byte(sV);
				}
			} else if (c.equals("char")) {
				cV = new Character[1];
				if (sV == null || sV.trim().length() == 0) {
					char emptyChar = '\u0000';
					cV[0] = new Character(emptyChar);
				} else {
					cV[0] = new Character(sV.charAt(0));
				}
			} else if (c.equals("double")) {
				cV = new Double[1];
				if (sV == null || sV.trim().length() == 0) {
					cV[0] = new Double(0.0);
				} else {
					cV[0] = new Double(sV);
				}
			} else if (c.equals("float")) {
				cV = new Float[1];
				if (sV == null || sV.trim().length() == 0) {
					cV[0] = new Float(0.0);
				} else {
					cV[0] = new Float(sV);
				}
			} else if (c.equals("int")) {
				cV = new Integer[1];
				if (sV == null || sV.trim().length() == 0) {
					cV[0] = new Integer(0);
				} else {
					cV[0] = new Integer(sV);
				}
			} else if (c.equals("long")) {
				cV = new Long[1];
				if (sV == null || sV.trim().length() == 0) {
					cV[0] = new Long(0);
				} else {
					cV[0] = new Long(sV);
				}
			} else if (c.equals("short")) {
				cV = new Short[1];
				if (sV == null || sV.trim().length() == 0) {
					short s = 0;
					cV[0] = new Short(s);
				} else {
					cV[0] = new Short(sV);
				}
			} else if (c.equals("java.lang.Boolean")) {
				cV = new Boolean[1];
				if (sV == null || sV.trim().length() == 0) {
					cV[0] = new Boolean(false);
				} else {
					cV[0] = new Boolean(sV);
				}
			} else if (c.equals("java.lang.Character")) {
				cV = new Character[1];
				if (sV == null || sV.trim().length() == 0) {
					char emptyChar = '\u0000';
					cV[0] = new Character(emptyChar);
				} else {
					cV[0] = new Character(sV.charAt(0));
				}
			} else {
				cV = new Object[1];
				cV[0] = sV;
			}

		} catch (Throwable t) {
			throw new RuntimeException(t);
		}
		return cV;
	}
}
