package slgfp.util.comun;

/**
 * Esta clase se usa para disponer de varias funciones que nos sern de utilidad
 * en el framework como son las de partir o normalizar cadenas.
 * 
 * @version 1.0, 03-05-2011
 * @author Sergio Lovillo
 */

public class Utilidades {

	public final static char UPPER = 'U';
	public final static char LOWER = 'L';
	public final static char NO_SPACE = 'N';
	public final static char ONE_SPACE = 'S';

	private Utilidades() {
	}

	// Para partir cadenas
	public static String partirCadena(String cad, String sep, int o,
			boolean mantenerEspacios) {
		StringBuffer sb = new StringBuffer();
		String[] z = cad.split(" ");
		String line = "";
		for (int i = 0; i < z.length; i++) {
			if (mantenerEspacios) {
				line = line + z[i] + " ";
			} else {
				line = line + z[i].trim();
				if (!z[i].equals("")) {
					line = line + " ";
				}
			}
			if (line.length() >= o) {
				sb.append(line).append(sep);
				line = "";
			}
			if (i == z.length - 1) { // Tratamiento ltima linea
				if (line.length() < o) {
					sb.append(line);
				}
			}
		}
		return sb.toString();
	}

	public static String partirCadena(String s, String sep, int o) {
		return partirCadena(s, sep, o, false);
	}

	public static String partirCadena(String s, int o) {
		String sep;
		try {
			sep = System.getProperty("line.separator");
		} catch (SecurityException se) {
			sep = "\n";
		}
		return partirCadena(s, sep, o, false);
	}

	public static String normalizarCadena(String s, char sensibleMayuscula,
			char tipoEspacio, char reemplazarCon) {
		if (s == null || s.trim().length() == 0) {
			return "";
		}
		s = s.trim();
		StringBuffer sb = new StringBuffer();

		for (int i = 0; i < s.length(); i++) {
			char c = s.charAt(i);
			if (Character.isWhitespace(c)) {
				if (tipoEspacio == NO_SPACE) {
					if (!Character.isWhitespace(reemplazarCon)) {
						sb.append(reemplazarCon);
					}
				} else if (tipoEspacio == ONE_SPACE) {
					if (i > 0 && !Character.isWhitespace(s.charAt(i - 1))) {
						if (!Character.isWhitespace(reemplazarCon)) {
							sb.append(reemplazarCon);
						} else {
							sb.append(c);
						}
					}
				}
			} else {
				sb.append(c);
			}
		}
		String r = null;
		if (sensibleMayuscula == UPPER) {
			r = sb.toString().toUpperCase();
		}
		if (sensibleMayuscula == LOWER) {
			r = sb.toString().toLowerCase();
		}
		return r;
	}
}
