package slgfp.util.db;

import java.util.*;
import java.io.*;
import java.sql.*;
import javax.sql.*;
import slgfp.*;

/**
 * Esta clase se usa para reiniciar los valores de la base de datos para las
 * distintas pruebas del framework.
 * 
 * @version 1.0, 03-05-2011
 * @author Sergio Lovillo
 */
public class SQLScriptEjecutor {

	protected String delimitador = ";";

	/** Ejecuta SQL script(s) de archivo */
	public SQLScriptEjecutor() {
	}

	public void ejecuta(DataSource ds, String rura) throws SQLException {
		ejecuta(ds.getConnection(), rura, this.delimitador);
	}

	public void ejecuta(DataSource ds, String ruta, String delimitador)
			throws SQLException {
		ejecuta(ds.getConnection(), ruta, delimitador);
	}

	public void ejecuta(Connection con, String rura) throws SQLException {
		ejecuta(con, rura, this.delimitador);
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	public void ejecuta(Connection con, String ruta, String delimitador)
			throws SQLException {

		if (delimitador != null && delimitador.trim().length() > 0) {
			this.delimitador = delimitador;
		}

		File sqlScript = new File(ruta);
		if (!sqlScript.exists()) {
			throw new IllegalArgumentException("Archivo " + ruta + " no existe");
		}

		slgfp.DBServicios dbServicios = new slgfp.DBServicios();
		dbServicios.setConexion(con);

		BufferedReader in = null;
		StringBuffer sb = new StringBuffer();
		try {
			in = new BufferedReader(new FileReader(ruta));
			String str;
			while ((str = in.readLine()) != null) {
				sb.append(str);
			}
		} catch (IOException e) {
			throw new RuntimeException(e);
		} finally {
			try {
				in.close();
			} catch (IOException e) {
				throw new RuntimeException(e);
			}
		}

		try {
			StringTokenizer st = new StringTokenizer(sb.toString(),
					this.delimitador, false);
			if (con.getMetaData().supportsBatchUpdates()) {
				Collection batch = new ArrayList();
				while (st.hasMoreTokens()) {
					String sql = st.nextToken();
					batch.add(sql);
				}
				dbServicios.comenzandoTransaccion();
				dbServicios.ejecutarBatch(batch);
			} else {
				dbServicios.setCacheSentencias(true);
				dbServicios.comenzandoTransaccion();
				while (st.hasMoreTokens()) {
					String sql = st.nextToken();
					dbServicios.ejecutar(sql);
				}
			}
			dbServicios.validandoTransaccion();
		} catch (Exception e) {
			e.printStackTrace();
			try {
				dbServicios.deshacerTransaccion();
			} catch (SQLException sqle) {
				throw new RuntimeException(sqle);
			}
			throw new RuntimeException(e);
		} finally {
			try {
				dbServicios.liberacion(true);
			} catch (Exception e) {
				e.printStackTrace();
				throw new RuntimeException(e);
			}
		}
	}

	public void ejecuta(String controlador, String dbURL, Properties p,
			String ruta, String delimitador) throws SQLException {
		DataSourceBasicoImpl dataSource = new DataSourceBasicoImpl();
		dataSource.setDbURL(dbURL);
		dataSource.setNombreControlador(controlador);
		dataSource.setPropiedadesConexion(p);
		ejecuta(dataSource, ruta, delimitador);
	}

	public void ejecuta(String controlador, String dbURL, String usuario,
			String password, String optionalConnProps, String rura)
			throws SQLException {
		ejecuta(controlador, dbURL, usuario, password, optionalConnProps, rura,
				this.delimitador);
	}

	public void ejecuta(String controlador, String dbURL, String user,
			String password, String optionalConnProps, String ruta,
			String delimitador) throws SQLException {

		if (controlador == null || controlador.trim().equals("")) {
			throw new IllegalArgumentException(
					"Falta el nombre del controlador");
		}
		if (user == null || user.trim().equals("")) {
			throw new IllegalArgumentException("Falta el nombre de usuario");
		}
		if (dbURL == null || dbURL.trim().equals("")) {
			throw new IllegalArgumentException("Falta DbURL");
		}
		Properties p = new Properties();
		p.setProperty("usuario", user);
		p.setProperty("password", password);
		if (optionalConnProps != null && optionalConnProps.trim().length() > 0) {
			StringTokenizer st = new StringTokenizer(optionalConnProps, ",",
					false);
			while (st.hasMoreTokens()) {
				String token = st.nextToken();
				int idx = token.indexOf("=");
				if (idx == -1) {
					throw new IllegalArgumentException(
							"Propiedades de la conexin opcional conexion formato no vlido.\nDebe tener formato: nombre1 =valor1, nombre2 =valor2,...");
				}
				String n = token.substring(0, idx);
				String v = token.substring(idx + 1);
				p.setProperty(n, v);
			}
		}
		ejecuta(controlador, dbURL, p, ruta, delimitador);
	}
}